/*==============================================================================
|                               SAS SCENARIO FILE
|
| PRODUCT: SAS Anti-Money Laundering
| SYSTEM:  UNIX/WINDOWS
================================================================================
| NAME:            | SAS10206
|                  |
| CATEGORY:        | Cash Activity
|                  |
| SEVERITY:        | Scenario
|                  |
| FREQUENCY:       | Daily
|                  |
| DURATION:        | 5 days
|                  |
| SUBJECT:         | Customer
|                  |
| PREP FILE:       | PARTY_TRANSACTIONS
|                  |
| HEADER FILE:     | party_header
|                  |
| TRIGGER:         | Yes
|                  |
| DAYS CONSIDERED: | Business
================================================================================

SHORT DESCRIPTION
-----------------
Large Total Cash Transactions by Risk Classification

DESCRIPTION
-----------
A customer's total amount of cash transactions exceeds a threshold over several
days.

BUSINESS NOTES
--------------
This scenario calculates the sum of a customer's cash credit and cash debit
transactions over a time period. If a customer's total amount of cash
transactions exceeds the threshold, a match is found.  This scenario
considers a customer's risk classification, whereas SAS10006 does not. This
scenario considers both credit and debit cash transactions, whereas SAS10007
only considers cash deposits.This scenario examines only cash deposits, whereas
SAS10020 and SAS10220 allow for examining multiple transaction types. This
scenario considers business days.

TECHNICAL NOTES
---------------
Not Applicable

MESSAGE
-------
r10206_threshold_message = %str("Total Cash Transactions = #$ #1;
Transaction Day Count = #2");

SAMPLE MESSAGE
--------------
Total Cash Transactions = USD 20,708; Transaction Day Count = 5

================================================================================
PARAMETERS

Name                              Type                Default Value
---------------------------------|-------------------|--------------------------
 p10206_account_type_desc        | Character List    | 'P'
 p10206_currency_based_account   | Character List    | 'N'
 p10206_min_amount               | Numeric Constant  | 10000
 p10206_min_total                | Numeric Constant  | 20000
 p10206_min_total_today          | Numeric Constant  | 2000
 p10206_primary_medium_desc      | Character List    | 'CASH'
 p10206_risk_classification      | Numeric List      | 3
 p10206_num_days                 | Numeric Constant  | 5
 p10206_status_desc              | Character List    | 'SUCCESS'
 p10206_transaction_cdi_code     | Character List    | 'C','D'

Descriptions
------------
- p10206_account_type_desc
List denoting valid types of accounts

- p10206_currency_based_account
List denoting non-exempt accounts

- p10206_min_amount
Maximum single amount of a valid transaction in interval

- p10206_min_total
Minimum aggregate amount of valid transactions

- p10206_min_total_today
Minimum aggregate amount of valid transactions on the current day

- p10206_primary_medium_desc
List denoting currency activities

- p10206_risk_classification
List denoting risk classifications for stratification

- p10206_num_days
Number of business days in interval

- p10206_status_desc
List denoting completed transactions

- p10206_transaction_cdi_code
List denoting transaction types

================================================================================

================================================================================
Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
- All rights reserved
==============================================================================*/
%let this_macro_name = &sysmacroname;  /* save this macro name */
/*----------------------------------------------------------------------------*/

retain p10206_begin_date;
format p10206_day_list $512.;

/* This init section should be placed before any goto statements.  This section of
   code will only be run when processing the first BY group of the prep data set.
   Any goto statements placed before it may prevent the init code from being run. */
if run_scenario_init eq 1 then do;
  p10206_begin_date = input(put(rundate_number-&p10206_num_days+1,num_to_date.),8.);
end;

/* Skip parties that do not have transactions on the current day */
if date_key{n} < rundate then goto &sysmacroname.;

p10206_day_list = ' ';
p10206_day_count = 0;
p10206_todays_total = 0;
p10206_total_amount = 0;

do i = n to 1 by -1 while (date_key{i} >= p10206_begin_date);
  if p10206_begin_date <= date_key{i} <= rundate then do;
    if upcase(account_type_desc{i}) in (&p10206_account_type_desc) and
       upcase(currency_based_account_ind{i}) in (&p10206_currency_based_account) and
       upcase(primary_medium_desc{i}) in (&p10206_primary_medium_desc) and
       upcase(transaction_cdi_code{i}) in (&p10206_transaction_cdi_code) and
       upcase(status_desc{i}) in (&p10206_status_desc) and
       risk_classification in (&p10206_risk_classification)
    then do;
        if currency_amount{i} ge &p10206_min_amount then goto &sysmacroname.;
        %fcf_save_transaction_key (calling_macro=&this_macro_name);
        p10206_total_amount = sum(p10206_total_amount,currency_amount{i});
        repeat = indexw(p10206_day_list,kstrip(put(date_key{i},8.)));
        if repeat eq 0 and not missing(date_key{i}) then do;
            p10206_day_list = catx(' ',p10206_day_list,put(date_key{i},8.));
            p10206_day_count + 1 ;
        end;
        if date_key{i} = rundate then
            p10206_todays_total = sum(p10206_todays_total,currency_amount{i});        
    end;
  end;
end;

/* Compare party values to parameters for a match */
if p10206_todays_total ge &p10206_min_total_today and
   p10206_total_amount ge &p10206_min_total
then do;
    actual_values_text  =  tranwrd(&r10206_threshold_message,'#1',kstrip(put(p10206_total_amount,nlnum32.)) );
    actual_values_text  =  tranwrd(actual_values_text,'#2',kstrip(put(p10206_day_count,nlnum32.)) );
    actual_values_text  =  tranwrd(actual_values_text,'#$', "&currency_code" );
    output &alert_fname;
end;

/* EXIT */
&sysmacroname.:;


/* ========================================================================== */
/* END SCENARIO - SAS10206                                                    */
/* ========================================================================== */
