/*==============================================================================
|                               SAS SCENARIO FILE
|
| PRODUCT: SAS Anti-Money Laundering
| SYSTEM:  UNIX/WINDOWS
================================================================================
| NAME:            | SAS10230
|                  |
| CATEGORY:        | Wire Activity
|                  |
| SEVERITY:        | Risk Factor
|                  |
| FREQUENCY:       | Daily
|                  |
| DURATION:        | 20 days
|                  |
| SUBJECT:         | Customer
|                  |
| PREP FILE:       | PARTY_TRANSACTIONS
|                  |
| HEADER FILE:     | party_header
|                  |
| TRIGGER:         | Yes
|                  |
| DAYS CONSIDERED: | Business
================================================================================

SHORT DESCRIPTION
-----------------
Activity Dominated by Wires by Risk Classification

DESCRIPTION
-----------
A large percentage of a customer's total transaction amount is wire transfers.

BUSINESS NOTES
--------------
This risk factor calculates the sum of transaction amounts and the sum of wire
transaction amounts. If a customer's percentage of wire transaction amounts
exceeds the threshold, a match is found. This scenario considers a customer's
risk classification, whereas SAS10030 does not. This scenario considers business
days.

TECHNICAL NOTES
---------------
Not Applicable

MESSAGE
-------
r10230_threshold_message = %str("Wire Transaction Percentage = #1");

SAMPLE MESSAGE
--------------
Wire Transaction Percentage = 100%

================================================================================
PARAMETERS

Name                              Type                Default Value
---------------------------------|-------------------|--------------------------
 p10230_account_type_desc        | Character List    | 'P'
 p10230_min_wire_amount_pct      | Numeric Constant  | 75
 p10230_primary_medium_desc      | Character List    | 'WIRE'
 p10230_risk_classification      | Numeric List      | 3
 p10230_num_days                 | Numeric Constant  | 20
 p10230_secondary_medium_desc    | Character List    | 'DOMESTIC','INTERNATIONAL'

Descriptions
------------
- p10230_account_type_desc
List denoting valid types of accounts

- p10230_min_wire_amount_pct
Percentage of transaction volume via wire

- p10230_primary_medium_desc
List denoting valid primary mediums

- p10230_risk_classification
List denoting risk classifications for stratification

- p10230_num_days
Number of business days in interval

- p10230_secondary_medium_desc
List denoting valid secondary mediums

================================================================================

================================================================================
Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
- All rights reserved
==============================================================================*/
%let this_macro_name = &sysmacroname;  /* save this macro name */
/*----------------------------------------------------------------------------*/

retain p10230_begin_date;

/* This init section should be placed before any goto statements.  This section of
   code will only be run when processing the first BY group of the prep data set.
   Any goto statements placed before it may prevent the init code from being run. */
if run_scenario_init eq 1 then do;
  p10230_begin_date = input(put(rundate_number-&p10230_num_days+1,num_to_date.),8.) ;
end;

if risk_classification ^in (&p10230_risk_classification) then goto &sysmacroname.;

p10230_tran_total = 0;
p10230_wire_total = 0;

do i = n to 1 by -1 while (date_key{i} >= p10230_begin_date);
    if date_key{i} > rundate then continue;
    if upcase(account_type_desc{i}) in (&p10230_account_type_desc) then do;
        p10230_tran_total + currency_amount{i};
        if upcase(primary_medium_desc{i})   in (&p10230_primary_medium_desc) and
           upcase(secondary_medium_desc{i}) in (&p10230_secondary_medium_desc)
        then do;
            p10230_wire_total + currency_amount{i};
            %fcf_save_transaction_key (calling_macro=&this_macro_name);
        end;
    end;
end;

/* Compare party values to parameters for a match */
if p10230_tran_total > 0 then do;
    p10230_primary_medium_desc_pct = p10230_wire_total / p10230_tran_total;
    if p10230_primary_medium_desc_pct >= &p10230_min_wire_amount_pct/100 then do;
        actual_values_text  =  tranwrd(&r10230_threshold_message,'#1',kstrip(put(p10230_primary_medium_desc_pct,nlpct8.0)) );
        output &alert_fname;
    end;
end;

/* EXIT */
&sysmacroname.:;


/* ========================================================================== */
/* END SCENARIO - SAS10230                                                    */
/* ========================================================================== */
