/******************************************************************************/
/*                  S A S   S A M P L E   L I B R A R Y                       */
/*                                                                            */
/*    NAME: fcf_system_admin_extract.sas                                      */
/*   TITLE: fcf_system_admin_extract                                          */
/* PRODUCT: SAS Anti-Money Laundering                                         */
/*  SYSTEM: UNIX/WINDOWS                                                      */
/******************************************************************************/
/* ************************************************************************** */
/*  Copyright (c) 2016 by SAS Institute Inc., Cary, NC 27513 USA              */
/*  ---All Rights Reserved.                                                   */
/* ************************************************************************** */
/* USAGE EXAMPLES:
/* %fcf_system_admin_extract;
/*----------------------------------------------------------------------------*/
/* COMMENTS:
/*---------------------------------------------------------
 *  Host and Site specific customizations may be REQUIRED
 *---------------------------------------------------------*/
%macro fcf_system_admin_extract;
   /*----------------------------------------------------------------------------*/
   /* Set error code to 0 and message to blank
   /*----------------------------------------------------------------------------*/
    %let trans_rc = 0;
    %let _fcfmsg_=;
   /*----------------------------------------------------------------------------*/ 
   /* Configurable macro variables 
   /* - vaVersion: Visual Analytics Version Number.
   /* - compSolMidVersion: Compliance Solution Mid Tier Version Number.
   /*----------------------------------------------------------------------------*/ 
    %let vaVersion = 7.3;
    %let compSolMidVersion = 7.1;


   /*--------------------------------------------------------------------------*/
   /* BEGIN JOB STATS EXTRACT
   /*--------------------------------------------------------------------------*/

   /*--------------------------------------------------------------------------*/
   /* Get Statistics File Records
   /*--------------------------------------------------------------------------*/
    %if %sysfunc(exist(job_stat.job_statistics)) %then %do; 
        data selected_job_stats;
         set job_stat.job_statistics;
         where upcase(metric) = "SECTION DURATION";
         format Duration time.;
         Duration = input(value,time.); 
        run;

        %fcf_rcset(&syserr);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _fcfmsg_=ERROR: Unable create selected_stats.;
            %fcf_put_error;
            %goto MACRO_END;
        %end;  
    %end; 
    %else %do;
        %let trans_rc = 900;
        %let _fcfmsg_ =ERROR: Required job_stat.job_statistics does not exist or is not accessible;  
        %fcf_put_error;
        %goto MACRO_END;
    %end;
    %let dsid=%sysfunc(open(work.selected_job_stats,i));
    %let num_obs=%sysfunc(attrn(&dsid,nobs));
    %let rc=%sysfunc(close(&dsid));
    %if &num_obs = 0 %then %do;
          %let trans_rc = 500; 
          %let _fcfmsg_ = NOTE: Zero records selected, no data will be produced;
          %fcf_put_error;
          %goto NEXT_EXTRACT;
    %end;

   /*--------------------------------------------------------------------------*/
   /* Sort Statistics for summarization
   /*--------------------------------------------------------------------------*/

   proc sort data=selected_job_stats out=selected_job_stats_sorted;
   by section;

   %fcf_rcset(&syserr);
   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: Unable to sort selected_job_stats.;
        %let trans_rc = 999999;
        %fcf_put_error;
        %goto NEXT_EXTRACT;
   %end;

   /*--------------------------------------------------------------------------*/
   /* Summarize duration by section/job step(s)
   /*--------------------------------------------------------------------------*/
   
    proc summary data=selected_job_stats_sorted nway maxdec=1;
    var DURATION;
    class SECTION;
    output out = jobs_stats_summary (drop= _type_ rename=(_freq_=numberexecutions))  max= min= mean=  / autoname autolabel;
    run;

   /*--------------------------------------------------------------------------*/
   /* format columns for report
   /*--------------------------------------------------------------------------*/
    proc sql;
	    create table fcf_rpt.jobs_stats_summary as 
        select section as SECTION,
		       numberexecutions as NUMBEREXECUTIONS,
			   duration_max as DURATION_MAX,
			   duration_min as DURATION_MIN,
			   duration_mean as DURATION_MEAN
		from jobs_stats_summary;
	quit;
    

   /*--------------------------------------------------------------------------*/
   /* Format duration information 
   /*--------------------------------------------------------------------------*/

    data selected_job_stats_detail_prep (drop= hh mm );
       set selected_job_stats_sorted;
       where upcase(metric) = "SECTION DURATION";
       format DURATION time. run_date date9. run_day weekdate9. /* week_num_fmt yyweekv8. */ weekFirstDay WEEKDATX28.;
       DURATION = input(value,time.); 
       RUN_DATE = datepart(datetime);
       RUN_DAY = put(datepart(datetime), 8.);
       WEEKFIRSTDAY=intnx('week',datepart(datetime),0);
       HOURS = hour(duration);
       /* calculate hours in seconds */
       HH =floor(hour(duration) * 3600);
       MINUTES = minute(duration);
       /* calculate minutes in seconds */
       MM = floor(minute(duration) * 60);
       SECONDS =second(duration);
       DURATIONSECONDS = sum(hh,mm,seconds);
    run;

    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: Unable to sort selected_job_stats.;
        %let trans_rc = 999999;
        %fcf_put_error;
        %goto NEXT_EXTRACT;
    %end;

   /*--------------------------------------------------------------------------*/
   /* Select return codes for section/job step(s)
   /*--------------------------------------------------------------------------*/

    data select_rc;
      set job_stat.job_statistics;
      where upcase(metric) = "SECTION RETURN CODE";
    run;

    proc sql;
        create table fcf_rpt.jobs_stats_detail as
        select a.datetime as DATETIME,
               a.section as SECTION,
               a.metric as METRIC,
               a.value as VALUE,
               a.duration as DURATION,
               a.run_date as RUN_DATE,
               a.run_day as RUN_DAY,
               a.weekfirstday as WEEKFIRSTDAY,
               a.hours as HOURS,
               a.minutes as MINUTES,
               a.seconds as SECONDS,
               a.durationseconds as DURATIONSECONDS,
               b.value as RETURN_CODE
        from selected_job_stats_detail_prep as a, select_rc as b 
        where a.datetime = b.datetime and 
              a.section = b.section;
     quit;


%NEXT_EXTRACT:

   /*--------------------------------------------------------------------------*/
   /* Get current users from metadata.
   /*--------------------------------------------------------------------------*/

    %mduextr(libref=work);

   /*--------------------------------------------------------------------------*/
   /* Start of capablities derivation
   /*--------------------------------------------------------------------------*/

   /*--------------------------------------------------------------------------*/
   /* Split roles from groups - keep only Compliance Solution(s) related roles
   /*--------------------------------------------------------------------------*/
    %if %sysfunc(exist(group_info)) and  %sysfunc(exist(grpmems)) and %sysfunc(exist(person_info)) and %sysfunc(exist(email))  %then %do; 
    data groups 
         roles
         supp_roles;
    set group_info;
    tmp_id = _n_;
    if GROUPTYPE = '' then do;
        GROUPTYPE = 'Group';
        output groups;
    end;
    else do;
       GROUPTYPE = propcase(grouptype);
       if upcase(name) in ('ANTI-MONEY LAUNDERING: ANALYST',
                           'ANTI-MONEY LAUNDERING: INVESTIGATOR',
                           'ANTI-MONEY LAUNDERING: MANAGER',
                           'ANTI-MONEY LAUNDERING: ADMINISTRATOR',
                           'ANTI-MONEY LAUNDERING: E-FILE MANAGER', 
                           'ANTI-MONEY LAUNDERING: SCENARIO ADMINISTRATOR USER',
                           'COMPLIANCE SOLUTIONS: QUALITY REVIEW AND AUDIT',
                           'CUSTOMER DUE DILIGENCE: ADMINISTRATOR',
                           'CUSTOMER DUE DILIGENCE: SCENARIO MANAGER USER',
                           'CUSTOMER DUE DILIGENCE: INVESTIGATOR',
                           'CUSTOMER DUE DILIGENCE: MANAGER',
                           'CUSTOMER DUE DILIGENCE: RULE ADMINISTRATOR USER',
                           'VISUAL SCENARIO DESIGNER: ADMINISTRATION'
              ) then output roles;
              else do;
                output supp_roles;
              end;
      end;
    run; 
    %end;
    %else %do;
        %let trans_rc = 900;
        %let _fcfmsg_ =ERROR: Required metadata tables not created or are not accessible;  
        %fcf_put_error;
        %goto MACRO_END;
    %end;

   /*--------------------------------------------------------------------------*/
   /* Find enabled capabilities for roles 
   /*--------------------------------------------------------------------------*/

    data RoleCapabilites (keep=capability capablity_Desc role application);
    length  appURI accURI identURI treeURI curTreeURI $256. capability capablity_Desc role application $200.;
    retain  appURI accURI identURI capability capablity_Desc '';
    appURI = '';
    appNbr=1;
    nbrAppObjs = 0;
    capability = '';
    capablity_Desc = '';
    accURI = '';
    nbrAccObjs = 0;

    do while(nbrAppObjs >= 0);
        nbrAppObjs=metadata_getnobj("omsobj:ApplicationAction?@id contains '.'",appNbr,appURI);
        if nbrAppObjs >0 then do;
            rc = metadata_getattr(appURI,"Name",capability);
            rc = metadata_getattr(appURI,"Desc",capablity_Desc);
        /* tree search */
            nbrTreeObjs=metadata_getnasn(appURI,"Trees",1,treeURI);
            if nbrTreeObjs > 0 then do;
                do t=1 to nbrTreeObjs;
                   tree_assoc_rc = metadata_getnasn(appURI,"Trees",t,curTreeURI);
                   app_name_get_rc = metadata_getattr(curTreeURI,"Name",application); 
                end;
            end;
            if application = 'ApplicationActions' then application = "Visual Analytics &vaVersion";
        /* end tree search */
        nbrAccObjs=metadata_getnasn(appURI,"AccessControls",1,accURI);
        if nbrAccObjs > 0 then do;
             do i=1 to nbrAccObjs;
                 currAccObj = metadata_getnasn(appURI,"AccessControls",1,accURI);
                 if currAccObj > 0 then do;
                    nbrIdents = metadata_getnasn(accURI,"Identities",1,identURI); 
                    do c=1 to nbrIdents;
                       rc = metadata_getnasn(accURI,"Identities",c,identURI);
                       rc = metadata_getattr(identURI,"Name",Role);
                       if upcase(role) in ('ANTI-MONEY LAUNDERING: ANALYST',
                           'ANTI-MONEY LAUNDERING: INVESTIGATOR',
                           'ANTI-MONEY LAUNDERING: MANAGER',
                           'ANTI-MONEY LAUNDERING: ADMINISTRATOR',
                           'ANTI-MONEY LAUNDERING: E-FILE MANAGER', 
                           'ANTI-MONEY LAUNDERING: SCENARIO ADMINISTRATOR USER',
                           'COMPLIANCE SOLUTIONS: QUALITY REVIEW AND AUDIT',
                           'CUSTOMER DUE DILIGENCE: ADMINISTRATOR',
                           'CUSTOMER DUE DILIGENCE: SCENARIO MANAGER USER',
                           'CUSTOMER DUE DILIGENCE: INVESTIGATOR',
                           'CUSTOMER DUE DILIGENCE: MANAGER',
                           'CUSTOMER DUE DILIGENCE: RULE ADMINISTRATOR USER',
                           'VISUAL SCENARIO DESIGNER: ADMINISTRATION'
                          ) then output;
                    end;  
                   end;  
                end;  
            end;  
        end;  
        appNbr=appNbr+1;
    end;  
    run;

   /*--------------------------------------------------------------------------*/
   /* Find all compliance related application capabilities
   /*--------------------------------------------------------------------------*/

    data APPLICATIONS_DATA (keep=application applicationDesc capability capablity_Desc nbrCapabilities capStatus);
    length  treeURI memberURI roleURI $256. capability capablity_Desc application applicationDesc treeType $200.;
    treeURI = '';
    treeNbr=1;
    nbrTreeObjs = 0;
    nbrCapabilities = 0;
    memberURI = '';
    roleURI = '';
    application = '';
    treeType = '';
    appOption = '';
    capStatus = 0;
    capability = '';
    capablity_Desc = '';
    do while(nbrTreeObjs >= 0);
        nbrTreeObjs=metadata_getnobj("omsobj:Tree?@id contains '.'",treeNbr,treeURI);
        if nbrTreeObjs >0 then do;
           name_get_rc = metadata_getattr(treeURI,"Name",application);
           desc_get_rc = metadata_getattr(treeURI,"Desc",applicationDesc);
           type_get_rc = metadata_getattr(treeURI,"TreeType",treeType);
           if upcase(application) in ('ACCESS',
                                      'ADMINISTRATION',
                                      'ANTI-MONEY LAUNDERING ACCESS',
                                      'ANTI-MONEY LAUNDERING ADMINISTRATION',
                                      'ANTI-MONEY LAUNDERING ALERTS',
                                      'ANTI-MONEY LAUNDERING CASES',
                                      'ANTI-MONEY LAUNDERING RISK ASSESSMENT',
                                      'ANTI-MONEY LAUNDERING SCENARIOS',
                                      'CASE',
                                      'COMMENTS',
                                      'COMPLIANCE SOLUTIONS ACCESS',
                                      'COMPLIANCE SOLUTIONS SERVICES',
                                      'CUSTOMER DUE DILIGENCE ACCESS',
                                      'CUSTOMER DUE DILIGENCE CASES',
                                      'CUSTOMER DUE DILIGENCE RULES',
                                      'EFILE',
                                      'INCIDENT',
                                      'PARTY',
                                      'RELATIONAL',
                                      'REPORT',
                                      'SEARCH',
                                      'VISUAL ANALYTICS',
                                      'APPLICATIONACTIONS'
               ) and
               treeType = 'ApplicationActions' then do;
                    put 'Application: ' application;
                    put 'Application: ' applicationDesc;
                                               
                    if upcase(application) = 'APPLICATIONACTIONS' then do; 
                        if upcase (applicationDesc) = 'COMPLIANCE SOLUTIONS SERVICES CAPABILITIES' then do;
                            application = "COMPLIANCE SOLUTIONS SERVICES";
                        end;
                 
                        if upcase (applicationDesc) = 'CAPABILITY DEFINITIONS FOR VISUAL ANALYTICS &vaVersion' then do;
                            application = "Visual Analytics &vaVersion";
                        end;
                    end; 


                    nbrCapabilities=metadata_getnasn(treeURI,"Members",1,memberURI);
                    if nbrCapabilities > 0 then do;
                        do i=1 to nbrCapabilities;
                            name_assoc_rc = metadata_getnasn(treeURI,"Members",i,roleURI);
                            name_get_rc = metadata_getattr(roleURI,"Name",capability);
                            desc_get_rc = metadata_getattr(roleURI,"Desc",capablity_Desc);
                            output; 
                        end;
                    end;
              end; 
        end; 
        treeNbr=treeNbr+1;
    end; 
    run;

   /*--------------------------------------------------------------------------*/
   /* Find application parent software
   /*--------------------------------------------------------------------------*/

    data APPLICATION_META (keep=product_name component application subfolder_Desc CapFolder nbrSoftTrees nbrSoftSubTrees);
    length  treeURI memberURI subFolderURI softTreeCntURI softTreeURI softActionCntURI  capFolderURI $256. 
            subfolder_Desc product_name component application treeType  CapFolder $200.;
    treeURI = '';
    treeNbr=1;
    nbrTreeObjs = 0;
    nbrSubFolders = 0;
    memberURI = '';
    subfolderURI = '';
    application = '';
    treeType = '';
    appOption = '';
    subfolder_Desc = '';
    nbrSoftTrees = 0;
    softTreeURI = '';
    capFolderURI = '';
    CapFolder = '';
    do while(nbrTreeObjs >= 0);
        nbrTreeObjs=metadata_getnobj("omsobj:Tree?@id contains '.'",treeNbr,treeURI);
        if nbrTreeObjs >0 then do;
            name_get_rc = metadata_getattr(treeURI,"Name",application);
            type_get_rc = metadata_getattr(treeURI,"TreeType",treeType);
            if treeType = 'BIP Folder' then do;
                nbrSubFolders=metadata_getnasn(treeURI,"Members",1,memberURI);
                if nbrSubFolders > 0 then do;
                    do i=1 to nbrSubFolders;
                        assoc_rc = metadata_getnasn(treeURI,"Members",i,subFolderURI);
                        prod_get_rc = metadata_getattr(subFolderURI,"ProductName",product_name);
                        comp_get_rc = metadata_getattr(subFolderURI,"Name",component);
                        if upcase(component) = 'SAS APPLICATION INFRASTRUCTURE' then application = component;
                        desc_get_rc = metadata_getattr(subFolderURI,"Desc",subFolder_Desc);
              /* find software softTrees */
                        nbrSoftTrees=metadata_getnasn(subFolderURI,"SoftwareTrees",1,softTreeCntURI);
                        nbrSoftSubTrees=metadata_getnasn(softTreeCntURI,"SubTrees",1,softActionCntURI);
                        if upcase(component) in ("COMPLIANCE SOLUTIONS MID &compSolMidVersion",
                                                 'SAS APPLICATION INFRASTRUCTURE') then do; 
                            if nbrSoftSubTrees > 0 then do;
                                 do i=1 to nbrSoftSubTrees; 
                                    assoc_rc = metadata_getnasn(softTreeCntURI,"SubTrees",i,capFolderURI);
                                    softTree_get_rc = metadata_getattr(capFolderURI,"Name",CapFolder);
                                    output;
                                 end; 
                            end;
                        end;
                     end;
                end; 
                else do;
                    if upcase(name) = "VISUAL ANALYTICS &vaVersion" then do;
                         output;
                    end;
                end;  
            end;  
        end;  
        treeNbr=treeNbr+1;
    end;  
    run;

   /*--------------------------------------------------------------------------*/
   /* Create table of roles related applications
   /*--------------------------------------------------------------------------*/

    proc sql;
        create table DistRoleSoftware as
        select distinct role, application 
        from RoleCapabilites
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: Unable create DistRoleSoftware data.;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

   /*--------------------------------------------------------------------------*/
   /* Create table of all application capabilities, all roles
   /*--------------------------------------------------------------------------*/

    proc sql;
        create table allCapsAllRoles as
        select a.capability as syscap,
               a.capablity_Desc as syscap_desc,
               a.application as sysSoft,
               b.role as sysRole,
               a.nbrCapabilities,
               a.capStatus
        from APPLICATIONS_DATA as a , DistRoleSoftware as b
        where a.application = b.application
        order by a.application, b.role;
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: Unable create allCapsAllRoles data.;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

   /*--------------------------------------------------------------------------*/
   /* Append parent application to capabilities
   /*--------------------------------------------------------------------------*/

    proc sql;
        create table allcapsallrolesmeta as
        select a.syscap,
               a.syscap_desc,
               a.sysSoft,
               a.sysRole,
               a.nbrCapabilities,
               a.capStatus,
               b.application 
        from WORK.allcapsallroles as a left join WORK.application_meta as b
            on a.sysSoft= b.Capfolder;
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: Unable create allcapsallrolesmeta data.;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

   /*--------------------------------------------------------------------------*/
   /* Sort and remove duplicates
   /*--------------------------------------------------------------------------*/

    proc sort data=allcapsallrolesmeta noduprecs; 
        by sysRole; 
    run;

   /*--------------------------------------------------------------------------*/
   /* Create table of Roles with all possible capabilites
   /*--------------------------------------------------------------------------*/

    proc sql;
        create table capInfo as
        select a.syscap,
               a.syscap_desc,
               a.sysSoft,
               a.sysRole,
               a.capStatus,
               a.application,
               a.nbrCapabilities,
               b.capability as rolcap,
               b.application as rolSoft,
               b.role
        from allcapsallrolesmeta as a left join RoleCapabilites as b
            on a.sysRole = b.role and
               a.sysCap = b.capability and 
               a.sysSoft = b.application
        order by a.sysRole, a.sysSoft, a.nbrCapabilities;
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: Unable create capInfo data.;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

   /*--------------------------------------------------------------------------*/
   /* Determine which capablities are enabled
   /*--------------------------------------------------------------------------*/

    data roleMatrix (keep=capability capablity_desc role application application_group status);
    set capInfo (rename=(role=inRole));
    capability = syscap;
    capablity_desc = syscap_desc; 
    role = sysRole;
    application_group = sysSoft;
    if sysrole ^= inRole then do;
        status = 0;
    end;
    else do;
      status = 1;
    end;
    run;

   /*--------------------------------------------------------------------------*/
   /* End of capablities derivation
   /*--------------------------------------------------------------------------*/

   /*--------------------------------------------------------------------------*/
   /*  Join capablities to roles 
   /*--------------------------------------------------------------------------*/

    proc sql; 
        create table role_info as
        select a.*,
               b.* 
        from roles as a, roleMatrix as b
        where a.name = b.role
        order by tmp_id;
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: Unable create role_Info data.;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

   /*--------------------------------------------------------------------------*/
   /*  Merge Groups and Roles 
   /*--------------------------------------------------------------------------*/

    data group_info (drop=tmp_id);
    merge groups role_info;
    by tmp_id;
    run;

   /*--------------------------------------------------------------------------*/
   /*  Append e-mail address to user info
   /*--------------------------------------------------------------------------*/

    proc sql;
        create table aml_user_info as
        select a.Id, 
               a.Name, 
               a.Desc, 
               b.emailAddr 
        from person_info as a left join email as b 
            on a.id = b.keyid;
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: Unable create capInfo data.;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

   /*--------------------------------------------------------------------------*/
   /*  Append group key to user info
   /*--------------------------------------------------------------------------*/

    proc sql;
        create table aml_user_info_grpkey as
        select a.Id, 
               a.Name as user_name, 
               a.Desc as user_display_name,
               a.emailAddr,
               b.grpkeyid
        from aml_user_info a left join grpmems b
            on a.id = b.memkeyid
        where grpkeyid not in (select distinct id from supp_roles)
        order by a.name;
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: Unable create capInfo data.;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

   /*--------------------------------------------------------------------------*/
   /*  Append group and role info to user info
   /*--------------------------------------------------------------------------*/

    proc sql;
        create table fcf_rpt.cs_user_info_groups as
        select a.id as ID, 
               a.user_name as USER_NAME, 
               a.user_display_name as USER_DISPLAY_NAME,
               a.emailaddr as EMAILADDR,
               b.DisplayName as GROUP_ROLE_NAME,
               b.Desc as GROUP_ROLE_DISPLAY_NAME,
               b.groupType as GROUP_TYPE,
               b.application as APPLICATION,
               b.application_group as APPLICATION_GROUP,
               b.capability as CAPABILITY,
               b.capablity_desc as CAPABLITY_DESC,
               b.status as STATUS
        from aml_user_info_grpkey as a left join group_info as b 
            on a.grpkeyid = b.id
        order by a.user_name;
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: Unable create capInfo data.;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

   /*--------------------------------------------------------------------------*/
   /*  Create role_group table to summarize nbr of Roles
   /*--------------------------------------------------------------------------*/

    proc sql;
        create table temp_role_group as
        select user_name, group_role_Name, group_type, 1 as sumRole 
        from (select distinct user_name, group_role_Name, group_type 
              from fcf_rpt.cs_user_info_groups
              );
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: Unable create capInfo data.;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

   /*--------------------------------------------------------------------------*/
   /*  Count nbr of groups and roles
   /*--------------------------------------------------------------------------*/

    proc summary data=temp_role_group nway;
        class user_name group_type;
        var  sumRole;
        output out=user_group_role_summary (drop= _FREQ_ _TYPE_) sum=totals;
    quit;

   /*--------------------------------------------------------------------------*/
   /*  Count nbr capablilites 
   /*--------------------------------------------------------------------------*/

    proc sql;
        create table temp_role_cap_totals as
        select distinct user_name, group_type, cap_total 
        from (select user_name, group_type, count(capability) as cap_total
              from fcf_rpt.cs_user_info_groups
              group by user_name, group_type
              );
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: Unable create capInfo data.;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

   /*--------------------------------------------------------------------------*/
   /*  Append nbr. of capablilites to group/role count table 
   /*--------------------------------------------------------------------------*/

    proc sql;
        create table fcf_rpt.user_group_role_totals as
        select a.user_name as USER_NAME,
               a.group_type as GROUP_TYPE,
               a.totals as TOTALS,
               b.cap_total as CAP_TOTAL
        from user_group_role_summary as a, temp_role_cap_totals as b
        where a.user_name = b.user_name and a.group_type = b.group_type;
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: Unable create capInfo data.;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

%MACRO_END:

%if &trans_rc in (3 5 900 909) %then
     %fcf_execute_macro(fcf_error);
%else %do;
     %fcf_execute_macro(fcf_success);
%end;



%mend fcf_system_admin_extract;
