/*******************************************************************************/
/* This program is for demonstrating how institution and branch data can be    */ 
/* validated against FinCEN rules while loading sample data for regulatory     */
/* reports.  The validation rules should be modified to meet your              */
/* organizational needs.  These are provided as a sample. These validation     */
/* rules will be specific to SARX error codes and rules.                       */
/* *****************************************************************************/

/*-------------------------------------------------------------------
 * Copyright (c) 2018 - 2021 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/


%MACRO VALIDATE_FINCEN_FILER;
/* E1: The party type associated with the filing institution exceeds the maximum occurrence for the FinCEN SAR. */
/* Skip. Report level data */

/* E2: The element (75a-i, z. Primary Federal Regulator) is not recorded.  */
error_code="E2";
target_field_name="institution_prim_fed_reg_code";
target_field_value=institution_prim_fed_reg_code;
if institution_prim_fed_reg_code=""  then output;

/* E3: The element is recorded but with no value (i.e. the value is null). */
/* Skip. Covered by E2 */

/* E4: The element (75a-i, z. Primary Federal Regulator) contains a value that is not a valid code. */
error_code="E4";
target_field_name="institution_prim_fed_reg_code";
target_field_value=institution_prim_fed_reg_code;
if institution_prim_fed_reg_code ne "" and index("ABCDEFGHZ",strip(institution_prim_fed_reg_code))=0  then output;

/* Cross validation for INSTITUTION_TYPE_CODE and INSTITUTION_PRIM_FED_REG_CODE */
/* The following validation was captured from the SARX discrete filing form.    */
/* It is not checked by FinCEN in batch submission.                             */
/* It can be removed if it is not applicable.                                   */
error_code="E4";
target_field_name="institution_prim_fed_reg_code";
target_field_value=institution_prim_fed_reg_code;
/* IRS specific Institution Type codes */
if (institution_type_code ='A' or institution_type_code='C' or institution_type_code='D') and 
   (institution_prim_fed_reg_code ^= 'D') then output;
/* Depository codes */
if (institution_type_code ='B') and 
   (index("BCDEF",strip(institution_prim_fed_reg_code))=0) then output;
/* Securities codes */
if (institution_type_code ='E') and 
   (index("AG",strip(institution_prim_fed_reg_code))=0) then output;

/* E5: The element (76. Name: Legal name) is not recorded or does not contain a value. */
error_code="E5";
target_field_name="institution_legal_name";
target_field_value=institution_legal_name;
if institution_legal_name=""  then output;

/* E6: The element (76. Name: Legal name) is recorded but with no value (i.e. the value is null). */
/* Skip. Covered by E5 */

/* E7: The element (77. TIN: Number) is not recorded or does not contain a value.  */
error_code="E7";
target_field_name="institution_tax_id";
target_field_value=institution_tax_id;
if institution_tax_id="" then output;

/* E9: The element(77. TIN: Number) contains an invalid number string for a US TIN.  */
error_code="E9";
target_field_name="institution_tax_id";
target_field_value=institution_tax_id;
if institution_tax_id_type in ('A' 'B')  
   and (length(institution_tax_id) ne 9  
        or institution_tax_id in ('999999999' '000000000' '123456789') 
        or notdigit(strip(institution_tax_id))
        ) 
   then output;

/* E46: The element (78. TIN: Type) is not recorded or does not contain a valid value. */
error_code="E46";
target_field_name="institution_tax_id_type";
target_field_value=institution_tax_id_type;
if institution_tax_id_type="" then output;


/* E10: The element (79, 80. Institution Type/Subtype) is not recorded.*/
/* Skip. Covered by E11 */


/* E11: The element (79a-g,z. Institution Type)is not recorded or does not contain a value (i.e. the value is null).*/
error_code="E11";
target_field_name="institution_type_code";
target_field_value=institution_type_code;
if institution_type_code=""  then output;

/* E12: The element (79a-g,z. Institution Type) contains an invalid code.*/
error_code="E12";
target_field_name="institution_type_code";
target_field_value=institution_type_code;
if institution_type_code ne "" and index("ABCDEFGZ",strip(institution_type_code))=0 
   then output;

/* E13: The element (79z. Institution Type Other Description) is not recorded or does not contain a value and the institution type is equal to Other. */
error_code="E13";
target_field_name="institution_type_other";
target_field_value=institution_type_other;
if institution_type_other="" and (institution_type_code  ='Z') then output;

/* E14: The element (79z. Institution Type Other Description) is recorded with a value but the institution type is not equal to Other. */
error_code="E14";
target_field_name="institution_type_other";
target_field_value=institution_type_other;
if institution_type_other ne "" and (institution_type_code  ne 'Z') then output;

/* E15: The element(80a-n,z. Institution Subtype: Securities/Futures Institution) is not recorded or does not contain a value 
   (i.e. the value is null) and the institution type is equal to Securities and Futures.*/
error_code="E15";
target_field_name="institution_sf_type_code";
target_field_value=institution_sf_type_code;
if institution_type_code='E' and institution_sf_type_code = ""  then output;

/* E16: The element (80a-n,z. Institution Subtype: Securities/Futures Institution) contains an invalid code. */
error_code="E16";
target_field_name="institution_sf_type_code";
target_field_value=institution_sf_type_code;
if institution_sf_type_code ne "" and compress(institution_sf_type_code,"ABXCDEFGHIJKLZ") ne '' then output;

/* E17: The element (80a-n,z. Institution Subtype: Securities/Futures Institution) contains a value but the institution type is not equal to Securities and Futures */
error_code="E17";
target_field_name="institution_sf_type_code";
target_field_value=institution_sf_type_code;
if institution_type_code ne'E' and institution_sf_type_code ne ""  then output;


/* E19: The value (80a-n,z. Institution Subtype: Securities/Futures Institution) recorded conflicts with the institution type recorded 
  (e.g. State Casino recorded for Securities and Futures institution)*/
/* Skip. Covered in E16 and E17. Also Gaming type is not included in Filer. */
 

/* E20: The element (80z. Institution Subtype Other: Securities/Futures Institution Description) is recorded with a value 
        but the institution subtype is not equal to Other. */
error_code="E20";
target_field_name="institution_sf_type_other";
target_field_value=institution_sf_type_other;
if institution_sf_type_other ne "" and indexc(institution_sf_type_code,'Z') = 0 then output;


/* E47: The element (80z. Institution Subtype Other: Securities/Futures Institution Description) is not recorded 
        or does not contain a value and the institution subtype is equal to Other. */
error_code="E47";
target_field_name="institution_sf_type_other";
target_field_value=institution_sf_type_other;
if institution_sf_type_other="" and indexc(institution_sf_type_code,'Z') > 0 then output;


/* E21: The element (81h. Institution ID: Number) is not recorded or does not contain a value. */
error_code="E21";
target_field_name="institution_id";
target_field_value=institution_id;
if institution_id="" and (institution_id_type_code ne '') then output;

/* E22: The value (81h. Institution ID: Number) recorded for the identification number is an invalid number string. */
error_code="E22";
target_field_name="institution_id";
target_field_value=institution_id;
if institution_id ne "" and (notdigit(strip(institution_id)) > 0)  then output;

/* E24: The element (82. Address: Address/Street) is not recorded or does not contain a value. */
error_code="E24";
target_field_name="institution_address";
target_field_value=institution_address;
if institution_address=""  then output;

/* E26: The element is not recorded or does not contain a value. */
error_code="E26";
target_field_name="institution_city_name";
target_field_value=institution_city_name;
if institution_city_name=""  then output;

/* E27: The element (84. Address: State) contains an invalid state code.*/ 
error_code="E27";
target_field_name="institution_state_code";
target_field_value=institution_state_code;
if institution_state_code ne ""  
   and (put((trim(institution_country_code)||'_'||trim(institution_state_code)),$RT_STATE_PROVINCE.) ne 'VALID')then output;

/* E28: The element (84. Address: State) is not recorded or does not contain a value for the filing institution 
        when the country is US, CA, MX, or a U.S. Territory. */
error_code="E28";
target_field_name="institution_state_code";
target_field_value=institution_state_code;
if institution_state_code = "" 
   and institution_country_code in ('US' 'CA' 'MX' 'AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR', 'VI')  then output;

/* E30: The element (84. Address: State) contains a value and the country is not US, CA, MX, or a U.S. Territory. */
error_code="E30";
target_field_name="institution_state_code";
target_field_value=institution_state_code;
if  institution_state_code ne ' ' 
    and institution_country_code not in ('CA' 'MX' 'US' 'AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR' 'VI') then output;


/* E31: State code and country code (84 86. Address) mismatch when U.S. Territory is recorded.*/
error_code="E31";
target_field_name="institution_state_code";
target_field_value=institution_state_code;
if   institution_state_code ne institution_country_code 
     and institution_country_code in ('AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR' 'VI') then output;

/* E32: The value (85. Address: ZIP/Postal Code) recorded contains punctuation or formatting. */
error_code="E32";
target_field_name="institution_zip_code";
target_field_value=institution_zip_code;
if compress(institution_zip_code,,'kad') ne institution_zip_code then output;

/* E34: The element (85. Address: ZIP/Postal Code) is not recorded or does not contain a value for the filing institution. */
error_code="E34";
target_field_name="institution_zip_code";
target_field_value=institution_zip_code;
if institution_zip_code=""  then output;


/* E35: The value (85. Address: ZIP/Postal Code) recorded is not equal to 5 or 9 numeric characters, 
        is a 9-digit value ending in four zeros or four nines, and the corresponding country is equal to US or a U.S. Territory code.*/
error_code="E35";
target_field_name="institution_zip_code";
target_field_value=institution_zip_code;
if  (institution_zip_code ne '')
    and institution_country_code in ('US' 'AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR', 'VI')
    and ( ( length(institution_zip_code) ne 5 
             and length(institution_zip_code) ne 9
           )  
          or notdigit(strip(institution_zip_code)) > 0
          or (length(institution_zip_code) =9 and substrn(institution_zip_code,6,4) in ('0000' '9999'))
        ) 
   then output;


/* E36: The element (86. Address: Country) contains a value that is not a valid code. */
error_code="E36";
target_field_name="institution_country_code";
target_field_value=institution_country_code;
if institution_country_code ne ' ' 
   and (put(trim(institution_country_code),$RT_COUNTRY.) ne 'VALID') then output;

/* E37: The element (86. Address: Country) is not recorded or does not contain a value for the filing institution. */
error_code="E37";
target_field_name="institution_country_code";
target_field_value=institution_country_code;
if institution_country_code = ' ' then output;

/* E39: The party type associated with the designated contact office (93. Designated Contact Office) is not recorded */
/* skip. Covered by E41 */

/* E40: The party type associated with the designated contact office exceeds the maximum occurrence for the FinCEN SAR.*/
/* skip. */

/* E41: The element is not recorded or does not contain a value for the Designated contact office.*/
error_code="E41";
target_field_name="contact_office";
target_field_value=contact_office;
if contact_office=""  then output;

/* E42: The element (94. Phone: Number) is not recorded or does not contain a value for the designated contact office.*/
error_code="E42";
target_field_name="contact_phone_number";
target_field_value=contact_phone_number;
if contact_phone_number=""  then output;

/* E42: The value (94. Phone: Number) recorded for the element contains non-numeric characters. */
error_code="E42";
target_field_name="contact_phone_number";
target_field_value=contact_phone_number;
if (contact_phone_number ne "" and (notdigit(strip(contact_phone_number)) > 0))   then output;

/* E43: The party type associated with the LE contact agency exceeds the maximum occurrence for the FinCEN SAR. */
/* skip. Report level data */

/* E44: The party type associated with the LE contact name exceeds the maximum occurrence for the FinCEN SAR.*/
/* skip. Report level data */

/* E45: The value (92. LE contact date) recorded for the element is not in the required format (YYYYMMDD). */
/* skip. Report level data */

%mend; /* VALIDATE_FINCEN_FILER */


/*******************************************************************************/
/* This program is for demonstrating how institution and branch data can be    */ 
/* validated against FinCEN rules while loading sample data for regulatory     */
/* reports.  The validation rules should be modified to meet your              */
/* organizational needs.  These are provided as a sample. These validation     */
/* rules will be specific to SARX error codes and rules.                       */
/* *****************************************************************************/

/*-------------------------------------------------------------------
 * Copyright (c) 2018 - 2021 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/


%MACRO VALIDATE_FINCEN_INSTITUTION;
/* D0: The party type (FI Where Activity Occurred) associated with the financial institution where activity occurred 
        is not recorded.*/
/* Skip. Report level data */

/* D1: The party type (FI Where Activity Occurred) associated with the financial institution where activity occurred 
        exceeds the maximum occurrence for the FinCEN SAR. */
/* Skip. Report level data */

/* D2: The element (51, 53-54. Institution Type/Subtype) is not recorded. */
/* Skip. Covered by D3 */

/* D3: The element (51a-g,z. Institution Type) is not recorded or does not contain a value (i.e. the value is null). */
error_code="D3";
target_field_name="institution_type_code";
target_field_value=institution_type_code;
if institution_type_code=""  then output;

/* D4: The element (51a-g,z. Institution Type) contains an invalid code. */
error_code="D4";
target_field_name="institution_type_code";
target_field_value=institution_type_code;
if institution_type_code ne "" and index("ABCDEFGZ",strip(institution_type_code))=0 
   then output;

/* D5: The element is not recorded or does not contain a value and the institution type is equal to Other. */
error_code="D5";
target_field_name="institution_type_other";
target_field_value=institution_type_other;
if institution_type_other="" and (institution_type_code ='Z') then output;

/* D6: The element is recorded with a value but the institution type is not equal to Other. */
error_code="D6";
target_field_name="institution_type_other";
target_field_value=institution_type_other;
if institution_type_other ne "" and (institution_type_code ne 'Z') then output;

/* D7: The element (52a-i. Primary Federal Regulator) is not recorded.  */
error_code="D7";
target_field_name="institution_prim_fed_reg_code";
target_field_value=institution_prim_fed_reg_code;
if institution_prim_fed_reg_code=""  then output;

/* D8 : The element (52a-i. Primary Federal Regulator) is recorded but with no value (i.e. the value is null). */
/* skip. Covered by D7 */

/* D9: The element (52a-i. Primary Federal Regulator) contains an invalid code. */
error_code="D9";
target_field_name="institution_prim_fed_reg_code";
target_field_value=institution_prim_fed_reg_code;
if institution_prim_fed_reg_code ne "" and index("ABCDEFGHZ",strip(institution_prim_fed_reg_code))=0  then output;

/* D9: The element (52a-i. Primary Federal Regulator) contains an invalid code. */
/* Cross validation for INSTITUTION_TYPE_CODE and INSTITUTION_PRIM_FED_REG_CODE */
/* The following validation was captured from the SARX discrete filing form.    */
/* It is not checked by FinCEN in batch submission.                             */
/* It can be removed if it is not applicable.                                   */
error_code="D9";
target_field_name="institution_prim_fed_reg_code";
target_field_value=institution_prim_fed_reg_code;
/* IRS specific Institution Type codes */
if (institution_type_code ='A' or institution_type_code='C' or institution_type_code='D') and 
   (institution_prim_fed_reg_code ^= 'D') then output;
/* Depository codes */
if (institution_type_code ='B') and 
   (index("BCDEF",strip(institution_prim_fed_reg_code))=0) then output;
/* Securities codes */
if (institution_type_code ='E') and 
   (index("AG",strip(institution_prim_fed_reg_code))=0) then output;

/* D10: The element (53a-c,z nstitution Subtype: Gaming Institution) is not recorded or does not contain a value (i.e. the value is null) 
         and the institution type is equal to Casino/Card club.   */
error_code="D10";
target_field_name="institution_gaming_type_code";
target_field_value=institution_gaming_type_code;
if institution_gaming_type_code="" and (institution_type_code = 'A') then output;

/* D10: The element (54a-k,z. Institution Subtype: Institution Subtype: Securities/Futures Institution )is not recorded or does not contain a value (i.e. the value is null) 
         and the institution type is equal to Securities and Futures.   */
error_code="D10";
target_field_name="institution_sf_type_code";
target_field_value=institution_sf_type_code;
if institution_sf_type_code="" and (institution_type_code = 'E') then output;

/* D11: The element (53a-c,z Institution Subtype: Gaming Institution) contains an invalid code. */
error_code="D11";
target_field_name="institution_gaming_type_code";
target_field_value=institution_gaming_type_code;
if institution_gaming_type_code ne "" and index("ABCZ",strip(institution_gaming_type_code))=0 then output;

/* D11: The element (54a-k,z. Institution Subtype: Securities/Futures Institution) contains an invalid code. */
/* Note: different from E16, SF types in D11 do not have B, J, K                                             */
error_code="D11";
target_field_name="institution_sf_type_code";
target_field_value=institution_sf_type_code;
if institution_sf_type_code ne "" and compress(institution_sf_type_code,"AXCDEFGHILZ") ne '' then output;

/* D12 The element (53a-c,z Institution Subtype: Gaming Institution) contains a value but the institution type is not equal to Casino/Card club. */
error_code="D12";
target_field_name="institution_gaming_type_code";
target_field_value=institution_gaming_type_code;
if institution_gaming_type_code ne "" and (institution_type_code ne 'A') then output;

/* D12 The element (54a-k,z. Institution Subtype: Securities/Futures Institution) contains a value but the institution type is not equal to Securities and Futures. */
error_code="D12";
target_field_name="institution_sf_type_code";
target_field_value=institution_sf_type_code;
if institution_sf_type_code ne "" and (institution_type_code ne 'E') then output;

/* D14 The value recorded conflicts with the institution type recorded (e.g. State Casino recorded for Securities and Futures institution) */
error_code="D14";
target_field_name="institution_type_code";
target_field_value=institution_type_code;
if (institution_gaming_type_code ne "" and institution_type_code ne 'A') or 
   (institution_sf_type_code ne "" and institution_type_code ne 'E') 
   then output;
 
/* D15: The element (53z Institution Subtype Other: Gaming Institution Description) does not contain a value and Other is indicated. */
error_code="D15";
target_field_name="institution_gaming_type_other";
target_field_value=institution_gaming_type_other;
if institution_gaming_type_other="" and institution_gaming_type_code = 'Z' then output;

/* D15: The element (54z. Institution Subtype Other: Securities/Futures Institution Description) does not contain a value and Other is indicated. */
error_code="D15";
target_field_name="institution_sf_type_other";
target_field_value=institution_sf_type_other;
if institution_sf_type_other="" and indexc(institution_sf_type_code,'Z') > 0 then output;

/* D16: The element contains a value that is not a valid code. */
error_code="D16";
target_field_name="institution_gaming_type_code";
target_field_value=institution_gaming_type_code;
if institution_gaming_type_code ne "" and indexc("ABCZ",strip(institution_gaming_type_code))=0 and (institution_type_code = 'A') then output;

/* D16: The element (54z. Institution Subtype Other: Securities/Futures Institution Description) is recorded with a value but the institution subtype is not equal to Other. */
error_code="D16";
target_field_name="institution_sf_type_other";
target_field_value=institution_sf_type_other;
if institution_sf_type_other ne "" and indexc(institution_sf_type_code,'Z') = 0 then output;

/* D17: The element (55h. Institution ID: Number) is not recorded or does not contain a value and unknown is not indicated. */
error_code="D17";
target_field_name="institution_id";
target_field_value=institution_id;
if institution_id="" and (institution_id_type_code ne '') then output;

/* D51: More than one role (56a-b. Role in transaction) is recorded for the financial institution’s role in transaction. */
/* Skip. Report level data */

/* D18: The element (57. Name: Legal name of FI) is not recorded or does not contain a value and unknown is not indicated. */
error_code="D18";
target_field_name="institution_legal_name";
target_field_value=institution_legal_name;
if institution_legal_name="" and indexc(upcase(institution_legal_name_unk_flg),'YX1') = 0 then output;

/* D19: The element (57. Name: Legal name of FI) is recorded with a value for the legal name and unknown is indicated. */
error_code="D19";
target_field_name="institution_legal_name";
target_field_value=institution_legal_name;
if institution_legal_name ne "" and indexc(upcase(institution_legal_name_unk_flg),'XY1') > 0 then output;

/* D20: The element (59. TIN: Number) is not recorded or does not contain a value and unknown is not indicated.  */
error_code="D20";
target_field_name="institution_tax_id";
target_field_value=institution_tax_id;
if institution_tax_id="" and indexc(upcase(institution_tax_id_unk_flg),'XY1') = 0  then output;

/* D21: The value recorded for the EIN or SSN/ITIN is an invalid number string.  */
error_code="D21";
target_field_name="institution_tax_id";
target_field_value=institution_tax_id;
if institution_tax_id_type in ('A', 'B') and ((length(institution_tax_id) ne 9 
   or institution_tax_id in ('999999999' '000000000' '123456789') or notdigit(strip(institution_tax_id)))) then output; 
 
/* D52: The element (60. TIN: Type) is not recorded or does not contain a valid value. */
error_code="D52";
target_field_name="institution_tax_id_type";
target_field_value=institution_tax_id_type;
if indexc(upcase(institution_tax_id_unk_flg),'XY1') = 0 and index('ABC', institution_tax_id_type) = 0 then output;   

/* D22: The element (61. Address: Address/Street) is not recorded or does not contain a value and unknown is not indicated. */
error_code="D22";
target_field_name="institution_address";
target_field_value=institution_address;
if institution_address="" and indexc(upcase(institution_address_unk_flg),'XY1')=0  then output;

/* D23: The element (62. Address: City) is not recorded or does not contain a value and unknown is not indicated. */
error_code="D23";
target_field_name="institution_city_name";
target_field_value=institution_city_name;
if institution_city_name=""  and indexc(upcase(institution_city_unk_flg),'XY1')=0 then output;

/* D24: The element contains an invalid state code. */
error_code="D24";
target_field_name="institution_state_code";
target_field_value=institution_state_code;
if institution_state_code ne '' and (put((trim(institution_country_code)||'_'||trim(institution_state_code)),$RT_STATE_PROVINCE.) ne 'VALID')  then output;

/* D25: The element (63. Address: State) is not recorded or does not contain a value when the country is US, CA, MX, or a U.S. Territory. */
error_code="D25";
target_field_name="institution_state_code";
target_field_value=institution_state_code;
if institution_state_code=""  and institution_country_code in ('CA' 'MX' 'US' 'AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR' 'VI') then output;

/* D26: The element contains a value and the country is not US, CA, MX, or a U.S. Territory. */
error_code="D26";
target_field_name="institution_state_code";
target_field_value=institution_state_code;
if  institution_state_code ne ' ' 
    and institution_country_code not in ('CA' 'MX' 'US' 'AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR' 'VI') then output;

/* D27: State code and country code mismatch when U.S. Territory is recorded. */
error_code="D27";
target_field_name="institution_state_code";
target_field_value=institution_state_code;
if   institution_state_code ne institution_country_code 
     and institution_country_code in ('AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR' 'VI') then output;

/* D28: The value recorded (64. Address: ZIP/Postal Code) contains punctuation or formatting. */
error_code="D28";
target_field_name="institution_zip_code";
target_field_value=institution_zip_code;
if compress(institution_zip_code,,'kad') ne institution_zip_code then output;

/* D29: The element (64. Address: ZIP/Postal Code) is not recorded or does not contain a value and unknown is not indicated. */
error_code="D29";
target_field_name="institution_zip_code";
target_field_value=institution_zip_code;
if institution_zip_code=' '
   and  indexc(upcase(institution_zip_code_unk_flg),'XY1')=0 then output;

/* D30: The value recorded (64. Address: ZIP/Postal Code) is not equal to 5 or 9 numeric characters, is a 9-digit value ending in four zeros or four nines,
   and the corresponding country is equal to US or a U.S. Territory code. */
error_code="D30";
target_field_name="institution_zip_code";
target_field_value=institution_zip_code;
if  (institution_zip_code ne '')
    and institution_country_code in ('US' 'AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR', 'VI')
    and ( ( length(institution_zip_code) ne 5 
             and length(institution_zip_code) ne 9
           )  
          or notdigit(strip(institution_zip_code)) > 0
          or (length(institution_zip_code) =9 and substrn(institution_zip_code,6,4) in ('0000' '9999'))
        ) 
   then output;

/* D31: The element contains an invalid country code. */ 
error_code="D31";
target_field_name="institution_country_code";
target_field_value=institution_country_code;
if institution_country_code ne ' ' 
   and (put(trim(institution_country_code),$RT_COUNTRY.) ne 'VALID')  then output;

/* D32: The element is not recorded or does not contain a value and unknown is not indicated. */
error_code="D32";
target_field_name="institution_country_code";
target_field_value=institution_country_code;
if institution_country_code=' ' 
   and  indexc(upcase(institution_country_unk_flg),'XY1')=0 then output;

/* D33: The loss to financial institution amount (67. Loss to financial institution (amount)) recorded 
        contains one or more non-numeric characters. */
/* Skip. Report level data */

/* D53: The loss to financial institution amount (67. Loss to financial institution (amount)) recorded is equal to zero. */
/* Skip. Report level data */

/* D34: The loss to financial institution amount (67. Loss to financial institution (amount)) recorded is greater than
        the amount recorded for <TotalSuspiciousAmountText> */
/* Skip. Report level data */

/* D35: The element (67. FI/Branch Party Association) is recorded but no relationship indicator is recorded with a "Y" value. */
/* Skip. Report level data */

/* D36: This element (69a. No branch activity involved indicator) is recorded with a "Y" value for the Financial institution (34) 
        but one or more Branch (46) party type locations are also recorded. */
/* Skip. Report level data */

/* D37: The party type associated with the branch where activity occurred is not recorded and 
        "no branch activity involved" is not indicated.*/
/* Skip. Report level data */

/* D38: The party type associated with the branch where activity occurred exceeds the maximum occurrence for the FinCEN SAR. */
/* Skip. Report level data */

%mend; /* VALIDATE_FINCEN_INSTITUTION */


%MACRO VALIDATE_FINCEN_BRANCH;


/* D28: The value recorded (64. Address: ZIP/Postal Code) contains punctuation or formatting. */
/* Although FinCEN is triggering D30 when the branch zip code (73 Address: ZIP/Postal Code) contains punctuation or blanks, it is more appropriate to use D28 */ 
error_code="D28";
target_field_name="branch_zip_code";
target_field_value=branch_zip_code;
if compress(branch_zip_code,,'kad') ne branch_zip_code then output;

/* D30: The value recorded (64. Address: ZIP/Postal Code) is not equal to 5 or 9 numeric characters, is a 9-digit value ending in four zeros or four nines,
   and the corresponding country is equal to US or a U.S. Territory code. */
/* Although FinCEN is not triggering D30 when the branch zip code does not meet this requirement, the check should apply to branch zip code (74. Address: ZIP/Postal Code) */
error_code="D30";
target_field_name="branch_zip_code";
target_field_value=branch_zip_code;
  
if  (branch_zip_code ne '')
    and branch_country_code in ('US' 'AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR', 'VI')
    and ( ( length(branch_zip_code) ne 5 
             and length(branch_zip_code) ne 9
           )  
          or notdigit(strip(branch_zip_code)) > 0
          or (length(branch_zip_code) =9 and substrn(branch_zip_code,6,4) in ('0000' '9999'))
        ) 
   then output;



/* D39: More than one role is recorded for the branch’s role in the transaction. */
/* Skip. Report level data */

/* D41: The element is not recorded for the Branch and <NoBranchActivityInvolvedIndicator> does not contain a "Y" value. */
/* Skip. Report level data */

/* D44: The element(72. Address: State) contains an invalid state code. */
error_code="D44";
target_field_name="branch_state_code";
target_field_value=branch_state_code;
if branch_state_code ne '' and (put((trim(branch_country_code)||'_'||trim(branch_state_code)),$RT_STATE_PROVINCE.) ne 'VALID')  then output;

/* D45: The element(72. Address: State) contains a value and the country is not US, CA, MX, or a U.S. Territory. */
error_code="D45";
target_field_name="branch_state_code";
target_field_value=branch_state_code;
if ((branch_state_code ne ' ' and branch_country_code not in ('CA' 'MX' 'US' 'AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR'))) then output;
 
/* D46 The element is not recorded or does not contain a value when the country is a U.S. Territory */
/* Skip. FinCEN is not checking this error.  */

/* D47: State code and country code (72, 74. Address: State, Country) mismatch when U.S. Territory is recorded. */
error_code="D47";
target_field_name="branch_state_code";
target_field_value=branch_state_code;
if ((branch_state_code ne branch_country_code and branch_country_code in ('AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR'))) then output;

/* D49: The element(74. Address: Country) contains an invalid country code. */ 
error_code="D49";
target_field_name="branch_country_code";
target_field_value=branch_country_code;
if branch_country_code ne '' and (put(trim(branch_country_code),$RT_COUNTRY.) ne 'VALID')  then output;

/* D50: The element (74. Address: Country) is not recorded or does not contain a value. */
error_code="D50";
target_field_name="branch_country_code";
target_field_value=branch_country_code;
if branch_country_code = ''  then output;

%mend; /* VALIDATE_FINCEN_BRANCH */

%macro VALIDATE_FINCEN_TRANSMITTER;

/* G0: A required party associated with the transmitter is missing. */
/* Skip. This macro only works  on record level check.  */

/* G1: The report contains more than one party associated with the transmitter. */
/* Skip. This macro only works  on record level check.  */


/* G2: The element (Transmitter Name) is not recorded or does not contain a value. */
error_code="G2";
target_field_name="transmitter_name";
target_field_value=transmitter_name;
if transmitter_name=""  then output;

/* G3: The element (Transmitter Address) is not recorded or does not contain a value. */
error_code="G3";
target_field_name="transmitter_address";
target_field_value=transmitter_address;
if transmitter_address=""  then output;

/* G4: The element (Transmitter City)is not recorded or does not contain a value. */
error_code="G4";
target_field_name="transmitter_city_name";
target_field_value=transmitter_city_name;
if transmitter_city_name=""  then output;

/* G5: The element (Transmitter State) is not recorded or does not contain a value. */
error_code="G5";
target_field_name="transmitter_state_code";
target_field_value=transmitter_state_code;
if transmitter_state_code=""  then output;

/* G6: The element (Transmitter State) contains a value that is not a valid code. */
error_code="G6";
target_field_name="transmitter_state_code";
target_field_value=transmitter_state_code;
if (put((trim(transmitter_country_code)||'_'||trim(transmitter_state_code)),$RT_STATE_PROVINCE.) ne 'VALID')  then output;

/* G7 : The element (Transmitter ZIP/Postal Code) is not recorded or does not contain a value */
error_code="G7";
target_field_name="transmitter_zip_code";
target_field_value=transmitter_zip_code;
if transmitter_zip_code=""  then output;

/* G8: The value (Transmitter ZIP/Postal Code) recorded contains one or more non-numeric characters or 
       is in an invalid format and the country is US */
error_code="G8";
target_field_name="transmitter_zip_code";
target_field_value=transmitter_zip_code;
if (length(transmitter_zip_code) =9 and substrn(transmitter_zip_code,6,4) in ('0000' '9999'))  then output;

error_code="G8";
target_field_name="transmitter_zip_code";
target_field_value=transmitter_zip_code;
if (((length(transmitter_zip_code) ne 5 and length(transmitter_zip_code) ne 9)  or notdigit(strip(transmitter_zip_code)) > 0)) and (transmitter_zip_code ne ' ') then output;


/* G9: The element (Transmitter Country) is not recorded or does not contain a value */
error_code="G9";
target_field_name="transmitter_country_code";
target_field_value=transmitter_country_code;
if transmitter_country_code = '' then output;


/* G10: The element (Transmitter Country) contains a value that is not a valid code. */
error_code="G10";
target_field_name="transmitter_country_code";
target_field_value=transmitter_country_code;
if ((transmitter_country_code ne ' ' and transmitter_country_code not in ('CA' 'MX' 'US' 'AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR'))) then output;

/* G11: State code and country code mismatch when U.S. Territory is recorded. */
error_code="G11";
target_field_name="transmitter_state_code";
target_field_value=transmitter_state_code;
if transmitter_state_code ne transmitter_country_code 
   and transmitter_country_code in ('AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR') then output;


/* G12: The element (Transmitter Taxpayer Identification Number (TIN)) contains an invalid number string for a US TIN.  */
error_code="G12";
target_field_name="transmitter_tax_id";
target_field_value=transmitter_tax_id;
if transmitter_tax_id ne '' 
   and transmitter_country_code in ('US' 'AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR')
   and (length(transmitter_tax_id) ne 9 
        or transmitter_tax_id in ('999999999' '000000000' '123456789') 
        or notdigit(strip(transmitter_tax_id))
       )  
    then output;

/* G13: The element (Transmitter Taxpayer Identification Number (TIN)) is not recorded or does not contain a value.  */
error_code="G13";
target_field_name="transmitter_tax_id";
target_field_value=transmitter_tax_id;
if transmitter_tax_id="" then output;


/* G14: The element (Transmitter Taxpayer Identification Number (TIN) or Control Code (TCC) Type) is not recorded or does not contain a value. */
/* TCC only check. G13 covered misisng TIN */

error_code="G14";
target_field_name="transmitter_control_code";
target_field_value=transmitter_control_code;
if transmitter_control_code=""  then output;

/* G15: The element (Transmitter Taxpayer Identification Number (TIN) or Control Coe (TCC) Type) contains a value that is not a valid code */
/* Skip. G12 and G13 covered TIN 
         G16 covered invalid TCC 
*/
 
 
/* G16: The Transmitter Control Code (TCC) specified in the file is not a valid TCC. 
        For production, the TCC must begin with “P” and for User Test, it must be equal to "TBSATEST".*/
error_code="G16";
target_field_name="transmitter_control_code";
target_field_value=transmitter_control_code;
if transmitter_control_code ne '' 
   and not ( transmitter_control_code = "TBSATEST" 
             or substrn(transmitter_control_code,1,1) = 'P'
            ) then output;


/* G19: The value (Transmitter Telephone Number) recorded for the element contains non-numeric characters. */
error_code="G19";
target_field_name="transmitter_telephone";
target_field_value=transmitter_telephone;
if (transmitter_telephone ne "" and (notdigit(strip(transmitter_telephone)) > 0))   then output;


/* G20: The element (Transmitter Telephone Number) is not recorded or does not contain a value (e.g. the value is null). */
error_code="G20";
target_field_name="transmitter_telephone";
target_field_value=transmitter_telephone;
if transmitter_telephone=""  then output;

/* H0: A required party associated with the transmitter contact is missing. */
/* Skip. Covered in H2 */

/* H1: The report contains more than one party associated with the transmitter contact. */
/* Skip. Only one contact can be defined */

/* H2: The element (Transmitter Contact Name) is not recorded or does not contain a value. */
error_code="H2";
target_field_name="transmitter_contact_name";
target_field_value=transmitter_contact_nam;
if transmitter_contact_name=""  then output;


%mend; /* VALIDATE_FINCEN_TRANSMITTER */


