/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    cdd_crr_codegen.sas
| PRODUCT: SAS Customer Due Diligence
| SYSTEM:  Windows Unix
| DESC:    Generate CDD Risk Ranking program based on the definition in
|          CDD_RULE_GROUP.
+------------------------------------------------------------------------------+
| COMMENTS: INPUTS:
|           cdd_codegen_path=the physical path where CDD rule code were generated.
|           cdd_rule_libname =libname where FSK_CDD_RULE_GROUP table can be found.
|           cdd_deployment_id=the string to identify the deployment. It will be used to call
|                             the correct cdd_rule.sas and to name the generated program.
|                             Example: 03MAY2013_17_14_53_28
|           cdd_codegen_success_flg_mvar= name of macro variable to be used for returning the success flag.
|                                1 for success and 0 for fail. 
|                               This macro variable should be defined in the calling program 
|                               The deffault name is cdd_codegen_success_flg
|           OUTPUT:
|           Two programs will be generated in the <cdd_rule_path>/codegen:
|              cdd_crr_process_code.sas and cdd_crr_process_code_<cdd_deployment_id>.sas.
|           The first generated program will be called by CDD_CRR_DRIVER macro. It contains the
|           %inc statement to include the second program.
|
| Note:     It will be called by the CDD deploymnet stored process.
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2013-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro cdd_crr_codegen( cdd_codegen_path=
                       , cdd_rule_libname=
                       , cdd_deployment_id=
                       , cdd_codegen_success_flg_mvar=cdd_codegen_success_flg
                      );

 /*------------
   * Set Module
   *------------*/
  %local _fcfmodule_;
  %let _fcfmodule_=CDD_CRR;

  %if %symexist(_job_rc)=0 %then
      %let _job_rc=0;
 
  %let trans_rc=0;

  %if %symexist(&cdd_codegen_success_flg_mvar)=0 %then 
      %local &cdd_codegen_success_flg_mvar;
 

  /*--------------------------------------------
   * Load macros generated by CDD Rule Manager
   *---------------------------------------------*/

 data _null_;
       
       cdd_rule_generated_code_path=strip("%unquote(&cdd_codegen_path)/");
       cdd_rule_fname='cdd_crr_process_code';
       rc=filename('mtr_code',
                   strip(cdd_rule_generated_code_path)||strip(cdd_rule_fname)||'.sas'
                   );
       rc=filename('dpl_code',
                   strip(cdd_rule_generated_code_path)
                           ||strip(cdd_rule_fname)
                           ||"_%trim(&cdd_deployment_id)"
                           ||'.sas'
                   );

run;

/* Do not use the default MOD file option when calling cdd_crr_codegen_header because */
/* this is the beginning of the output file.                                          */

%cdd_crr_codegen_header(file_option=);
     put;
     put '/********************************************************/';
     put '/* Compile the rule code macros                         */';
     put '/********************************************************/';
     put;
     put 'CDD_RULE_CODE:';
     stmt='%inc '||"'"||"%trim(&cdd_codegen_path)/cdd_rules"||"_%trim(&cdd_deployment_id)"||'.sas'||"'"||';';
     put stmt;
     put;
%cdd_crr_codegen_footer;

%cdd_crr_codegen_add_errchk(label_nm=CDD_RULE_CODE,label_desc=%nrbquote(compile rule code));


/*----------------------------------------------------------------------------
   * Sort and filter FSK_CDD_RULE_GROUP table for Customer Risk Ranking process
   *----------------------------------------------------------------------------*/


    proc sort data=&&cdd_rule_libname..fsk_cdd_rule_group out=_t_orchestration_sorted;
         by cdd_rule_group_input_ds_name cdd_rule_group_level cdd_rule_group_order_in_level;
         where current_ind='Y' and cdd_rule_group_level> 0 ;
    run;


    %fcf_rcset(&syserr)
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc sql noprint;

          /*----------------------------------------------------------------------------
           * Find the names of the input data sets to be used for CRR
           * All input data sets must be in CM_PREP libary.
           *----------------------------------------------------------------------------*/
         %let _cddcrr_dsn_cnt=0;
         select strip(put(count(distinct cdd_rule_group_input_ds_name),best.)) into :_cddcrr_dsn_cnt
         from _t_orchestration_sorted
         where trim(cdd_rule_group_input_ds_name) ne '';

         select distinct cdd_rule_group_input_ds_name into :_cddcrr_dsn_1 - :_cddcrr_dsn_%trim(&_cddcrr_dsn_cnt)
         from _t_orchestration_sorted
         where trim(cdd_rule_group_input_ds_name) ne '';


          /*----------------------------------------------------------------------------
           * Find whether there is a NULL input data set to be used for CRR
           * The NULL input dataset will be created by concatenating all the final datasets
           * that are transformed from the prep datasets.
           *----------------------------------------------------------------------------*/

         %let _cddcrr_null_dsn_cnt=0;
         %let _cddcrr_dsn_%trim(%eval(&_cddcrr_dsn_cnt+1))=;
         select 1 into :_cddcrr_null_dsn_cnt
         from _t_orchestration_sorted
         where trim(cdd_rule_group_input_ds_name) = '';


    quit;



    %fcf_rcset(&syserr)
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

      /*----------------------------------------------------------------------------
       * Initialize a base table with the columns in FSK_CDD_SCORING_EVENT and
       * and FSK_CDD_TRIGGERING_VALUES
       *----------------------------------------------------------------------------*/

    /* __crr are special variables for crr processing */

    %cdd_crr_codegen_header;
     put;
     put '/*****************************************************/';
     put '/* Define the structure of the final output table.   */';
     put '/* The table is structured after CDD_SCORING_EVENT   */';
     put '/* and CDD_TRIGGERING_VALUES.                        */';
     put '/*****************************************************/';
     put;
     put 'CDD_SCORING_EVENT_ALL_INIT:';
     put ;
     put 'data cdd_scoring_event_all;';
     put 'length __CRR_SAVE_TRIGGER_FLG $ 1;';

     put 'if 0 then set seg_kc.fsk_cdd_scoring_event;';
     put 'if 0 then set seg_kc.fsk_cdd_triggering_values;';
     put 'stop;';
     put 'run;';
     put;

    %cdd_crr_codegen_footer;
    %cdd_crr_codegen_add_errchk(label_nm=CDD_SCORING_EVENT_ALL_INIT,label_desc=%nrbquote(initialize WORK.CDD_SCORING_EVENT_ALL));

   /*----------------------------------------------------------------------------
    * CRR process starts by looping thru each input data set including
    * the NULL input data set.
    * NULL input data set is always executed last.
    *----------------------------------------------------------------------------*/

    %do dsn_i=1 %to %eval(&_cddcrr_dsn_cnt + &_cddcrr_null_dsn_cnt);

       /*----------------------------------------------------------------------------
        * Determine the input prep file name.
        * If it is NULL input data set , concatenate the dataset transformed
        * from previous input data sets. Use it as input prep file
        * When concatenating the dataset from the previous steps, use BY statement
        * to preserve the data order if  they have the same CDD_RULE_GROUP_BY_VAR_1_NAME
        * value.
        *----------------------------------------------------------------------------*/
        %if &dsn_i <= &_cddcrr_dsn_cnt %then %do;

            %let _cdd_prep_file=cm_prep.&&_cddcrr_dsn_&dsn_i;

        %end;
        %else %do;
                 /* Determine the byvar name */
                 /* If the byvar for final rule group of the same input dataset */
                 /* are the same, use it.                                       */
                 %let _all_byvar=&_d_1_byvar;
                 %do prep_dsn_i=2 %to &_cddcrr_dsn_cnt;

                   %if &_all_byvar ne &&_d_&prep_dsn_i._byvar %then
                       %let _all_byvar=;
                 %end;


                %let _cdd_prep_file=_D_ALL;
                %cdd_crr_codegen_header;

                put;
                put '/*****************************************************/';
                put '/* Create an union set of the final output           */';
                put '/* of each prep file.                                */';
                put '/* The table is structured after CDD_SCORING_EVENT   */';
                put '/* and CDD_TRIGGERING_VALUES.                        */';
                put '/*****************************************************/';
                put;
                      put '_D_ALL:';
                      put 'data _d_all;';
                      put 'set';
                %do prep_dsn_i=1 %to &_cddcrr_dsn_cnt;

                       put "&&_d_&prep_dsn_i._last";
                %end;
                       put ';';
                %if %str(&_all_byvar) ne %str() %then %do;

                       put "by &_all_byvar;";
                %end;
                      put 'run;';
                ;
                %cdd_crr_codegen_footer;
                %cdd_crr_codegen_add_errchk(label_nm=_D_ALL,label_desc=%nrbquote(combine final output of previous steps));

        %end;

        /*----------------------------------------------------------------------------
        * Determine the number of execution levels for each input data set.
        *----------------------------------------------------------------------------*/

         %let _rg_level_cnt=0;
         proc sql noprint;

             select strip(put(count(distinct cdd_rule_group_level),best.)) into :_rg_level_cnt
             from _t_orchestration_sorted
             where trim(cdd_rule_group_input_ds_name) = "&&_cddcrr_dsn_&dsn_i";

             select distinct strip(put(cdd_rule_group_level,best.))                            
                    into :_rg_level_1 - :_rg_level_&_rg_level_cnt                          
             from _t_orchestration_sorted
             where trim(cdd_rule_group_input_ds_name) = "&&_cddcrr_dsn_&dsn_i";

        quit;

         %fcf_rcset(&syserr)
         %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
             %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
             %fcf_put_error;
             %goto MACRO_END;
         %end;

        /*----------------------------------------------------------------------------
        * For each execution level, process the rule groups.
        * The input dataset name is defined based on the level number
        * If the level number is 1, use the input dataset
        * else use _D_< input data set number>_L_<level number - 1>
        * The output data set is always _D_< input data set number>_L_<level number>
        *----------------------------------------------------------------------------*/
        %do level_i=1 %to &_rg_level_cnt;
            %cdd_crr_codegen_header;
            %if &level_i=1 %then %do;
                %let cdd_input_ds=&_cdd_prep_file ;
            %end;
            %else %do;
                %let cdd_input_ds=_D_&dsn_i._L_%eval(&level_i-1);
            %end;

                %let cdd_output_ds=_D_&dsn_i._L_&level_i;
            %cdd_crr_codegen_header;
               
               put ;
               put '/***********************************/';
               put '/*  Execution Step Begins          */';
               put '/***********************************/';
               put;
               put "&cdd_output_ds:";
               put;
               %if &level_i=1 and %str(&_cdd_prep_file) ne %str(_d_all) %then %do;
                    put '%let cdd_input_ds=' "&cdd_input_ds" ' &obs_option ;';
               %end;
               %else %do;
                    put '%let cdd_input_ds=' "&cdd_input_ds;";
               %end;

               put '%let cdd_output_ds=' "&cdd_output_ds;";
            %cdd_crr_codegen_footer;

            /*----------------------------------------------------------------------------
             * Find out the program names of the rule group within the executive level.
             * Find out if the output from this level should be saved to knowledge center
             * or not.
             * Note: FSK_CDD_RULE_GROUP.CDD_RULE_GROUP_SAVE_TO_KC is defined at the
             *       rule group level. However, the condition of <Save> or <No Save> is
             *       applied toe entire output data set.
             *       Therefore, only FSK_CDD_RULE_GROUP.CDD_RULE_GROUP_SAVE_TO_KC  of the
             *       last rule_group is used.
            *----------------------------------------------------------------------------*/


             %let _fn_cnt=0;
             proc sql noprint;
                 select strip(put(count(distinct cdd_rule_group_file_name),best.)) into :_fn_cnt
                 from _t_orchestration_sorted
                 where trim(cdd_rule_group_input_ds_name) = "&&_cddcrr_dsn_&dsn_i" and cdd_rule_group_level=&&_rg_level_&level_i;

                 select distinct   strip(cdd_rule_group_id) 
                                 , strip(cdd_rule_group_file_name)
                                 , strip(CDD_RULE_GROUP_SAVE_TO_KC_FLG)
                                 , strip(CDD_RULE_GROUP_SAVE_TRIG_V_FLG)
                                 , strip(CDD_RULE_GROUP_BY_VAR_1_NAME)
                          into     :_rg_id_1 - :_rg_id_&_fn_cnt
                                 , :_fn_1    - :_fn_&_fn_cnt
                                 , :_save_1  - :_save_&_fn_cnt
                                 , :_save_trigger_1 - :_save_trigger_&_fn_cnt
                                 , :_byvar_1 - :_byvar_&_fn_cnt

                 from _t_orchestration_sorted
                 where trim(cdd_rule_group_input_ds_name) = "&&_cddcrr_dsn_&dsn_i" and cdd_rule_group_level=&&_rg_level_&level_i;;

             quit;

             %fcf_rcset(&syserr)
             %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                  %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
                  %fcf_put_error;
                  %goto MACRO_END;
             %end;


            %let log_label_desc=;

            %do fn_i=1 %to &_fn_cnt;

                 /*----------------------------------------------------------------------------
                  * Process each rule group
                  * Append the output table to cdd_scoring_event_all if the output
                  * is required to be saved to KC
                  *----------------------------------------------------------------------------*/
                
                  %let log_label_desc=%trim(&log_label_desc) %nrbquote(&&_rg_id_&fn_i);
                  %cdd_crr_codegen_header;
                    stmt='%inc '||"'"||"%trim(&cdd_codegen_path)/"||"&&_fn_&fn_i"||"';";
                    put stmt;
                  %cdd_crr_codegen_footer;
                  
                 

                  %if &fn_i=&_fn_cnt %then
                  %do;
                      /* remember the last by var name  of the entire prep file path */
                      %let _d_&dsn_i._byvar=&&_byvar_&_fn_cnt;

                      %cdd_crr_codegen_add_errchk(label_nm=&cdd_output_ds, label_desc=%nrbquote(&log_label_desc));

                      %if &&_save_&_fn_cnt=1 or &&_save_&_fn_cnt=Y %then
                      %do;
                         %cdd_crr_codegen_header; /* data step */
                           put;
                           put 'data cdd_scoring_event_all;';
                           put '   set cdd_scoring_event_all';
                           put "      _d_&dsn_i._l_&level_i (in=in_new)";
                           put '   ;';
                           stmt= '    if in_new then __crr_save_trigger_flg="'||
                                  strip("&&_save_trigger_&_fn_cnt")||'";';
                           put stmt;
                           put 'run;';
                         %cdd_crr_codegen_footer;
                      %end;
                      %cdd_crr_codegen_header;
                           put '/***************************/';
                           put '/* Execution Step Ends     */';
                           put '/***************************/';
                           put;
                      %cdd_crr_codegen_footer;
                        
                 %end;
            %end; /* end fn_i */


            %if &level_i=&_rg_level_cnt %then
            %do;
                 %let _d_&dsn_i._last=&cdd_output_ds;
            

            %end;

        %end; /* end level_i */

    %end; /* end dsn_i */

    data _null_;
        file  mtr_code lrecl=256;
        length stmt $ 256;
        cdd_rule_generated_code_path=strip("%unquote(&cdd_codegen_path)/");
        cdd_rule_fname='cdd_crr_process_code';
        stmt='%inc '
             ||"'"
             ||strip(cdd_rule_generated_code_path)
             ||strip(cdd_rule_fname)
             ||"_%trim(&cdd_deployment_id)"
             ||'.sas'||"';";
        put stmt;
    run;

    filename mtr_code clear;
    filename dpl_code clear;



%MACRO_END:

      %if &trans_rc eq 3 or &trans_rc ge 5 %then
          %let &cdd_codegen_success_flg_mvar=0;
         
      %else         
         %let &cdd_codegen_success_flg_mvar=1;


%mend;
 
