/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    cdd_generate_rules.sas
| PRODUCT: SAS Customer Due Diligence
| SYSTEM:  Windows, Unix
| DESC:    reads the package and updates the ECM database
+------------------------------------------------------------------------------+
| PARAMETERS:
|  codegen_path - folder for generated SAS code
|  outlib - lib name for output data sets
|  gen_type -
|             KC - generate from KC - use all groups and all active rules
|             T - generate in test mode
|  rule_ids_tab_name - for gen_type=T only: dataset name that contains CDD_RULE_ID for rules to execute
|  rule_group_ids_tab_name - for gen_type=T only: dataset name that contains CDD_RULE_GROUP_ID for rule
|                            groups to execute
|  cdd_gen_success_flg_mvar - name of macro variable to be used for returning the success flag.
|                             1 for success and 0 for fail.
|                             This macro variable should be defined in the calling program.
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2013-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/
%macro cdd_generate_rules(
                          codegen_path=,
                          outlib=work,
                          rule_ids_tab_name=,
                          rule_group_ids_tab_name=,
                          gen_type=KC,
                          cdd_gen_success_flg_mvar=);

    /* reset error codes to zero */
    %let trans_rc = 0;
    %let sqlrc = 0;
    %let sqlxrc = 0;
    %let sysrc = 0;
    %if &cdd_gen_success_flg_mvar ne %then %do;
         %let &cdd_gen_success_flg_mvar = 0;
    %end;

    %macro cdd_sm_abort(msg=, p1=, p2=, p3=, p4=);
       %let parm_list=;
       %if "&p1" ne "" %then %do;  %let parm_list =, &p1;             %end;
       %if "&p2" ne "" %then %do;  %let parm_list =&parm_list , &p2;  %end;
       %if "&p3" ne "" %then %do;  %let parm_list =&parm_list , &p3;  %end;
       %if "&p4" ne "" %then %do;  %let parm_list =&parm_list , &p4;  %end;
    
       %let _fcfmsg_=ERROR: %sysfunc(sasmsg(sashelp.cddmsg, &msg, N &parm_list));
       %let trans_rc = 99;
       %fcf_put_error
       %ABORT CANCEL &trans_rc;
    %mend cdd_sm_abort;
    
    %macro cdd_sm_abort_on_error(msg=, p1=, p2=, p3=, p4=);
    %fcf_chkrc
    %if &bat_abort=Y %then %do;
       %let sys_err_msg = &bat_errmsg;
       %let parm_list=;
       %if "&p1" ne "" %then %do;  %let parm_list =, &p1;             %end;
       %if "&p2" ne "" %then %do;  %let parm_list =&parm_list , &p2;  %end;
       %if "&p3" ne "" %then %do;  %let parm_list =&parm_list , &p3;  %end;
       %if "&p4" ne "" %then %do;  %let parm_list =&parm_list , &p4;  %end;
    
       %let _fcfmsg_=ERROR: Update failed: %sysfunc(sasmsg(sashelp.cddmsg, &msg, N &parm_list));
       %let trans_rc = 99;
       %fcf_put_error
       %ABORT CANCEL &trans_rc;
    %end;
    %mend cdd_sm_abort_on_error;

    /* read FSK_PREFERENCES */
    %global prop_status;

    %fcf_get_property_value(property=fcf.svr.cdd.rule.sourcePath, output_property_value=cdd_sm_rule_path, output_status=prop_status);
    %if &prop_status ne 0 %then %do;
        %put WARNING: Rule source path not found in preferences. Using default value.;
        %let cdd_sm_rule_path = !CDDROOT/rule/rule_code_active;
    %end;
    %put NOTE: Rule path: &cdd_sm_rule_path;

    %fcf_get_property_value(property=fcf.svr.cdd.rulegroup.sourcePath, output_property_value=cdd_sm_rule_group_path, output_status=prop_status);
    %if &prop_status ne 0 %then %do;
        %put WARNING: Rule group source path not found in preferences. Using default value.;
        %let cdd_sm_rule_group_path = !CDDROOT/rule/rule_group_code_active;
    %end;
    %put NOTE: Rule group path: &cdd_sm_rule_group_path;

    /* create data set to store errors */
    data errors;
      length msg $400;
      stop;
    run;
    data _null_;
        tday = translate(put(datetime(),e8601dt25.3), "--", ":.");
        call symput('cdd_deploy_date', tday);
    run;
    /* some of the code above may set SYSRC to a negative value, so reset */
    %let sysrc = 0;

    %if &gen_type = T %then %do;
        /* select rule groups to run/test, use only current version */
        proc sql;
            create table FSK_CDD_RULE_GROUP as
                select *
                from seg_kc.FSK_CDD_RULE_GROUP
                where CDD_RULE_GROUP_ID in (select CDD_RULE_GROUP_ID from &rule_group_ids_tab_name)
                      and CURRENT_IND = 'Y' and LOGICAL_DELETE_IND = 'N'
            ;
        run;

        /* select rules to run/test, use only current version */
        proc sql;
            create table FSK_CDD_RULE as
                select *
                from seg_kc.FSK_CDD_RULE
                where CDD_RULE_ID in (select CDD_RULE_ID from &RULE_IDS_TAB_NAME)
                      and CURRENT_IND = 'Y' and LOGICAL_DELETE_IND = 'N' and CDD_RULE_STATUS_CODE='ACT'
            ;
        run;
        /* select parameters for rules */
        proc sql;
            create table FSK_CDD_RULE_PARAMETER as
                select p.*
                from seg_kc.FSK_CDD_RULE_PARAMETER p
                inner join FSK_CDD_RULE r
                   on p.CDD_RULE_ID = r.CDD_RULE_ID
                      and p.CDD_RULE_VERSION_NO = r.VERSION_NO
            ;
        run;
    %end;

    %if &gen_type = KC %then %do;
        /* select rule groups to run/test, use only current version */
        proc sql;
            create table FSK_CDD_RULE_GROUP as
                select *
                from seg_kc.FSK_CDD_RULE_GROUP
                where CURRENT_IND = 'Y' and LOGICAL_DELETE_IND = 'N'
            ;
        run;

        /* select rules to run/test, use only active rules current version */
        proc sql;
            create table FSK_CDD_RULE as
                select *
                from seg_kc.FSK_CDD_RULE
                where CURRENT_IND = 'Y' and LOGICAL_DELETE_IND ='N' and CDD_RULE_STATUS_CODE='ACT'
            ;
        run;
        /* select parameters for rules */
        proc sql;
            create table FSK_CDD_RULE_PARAMETER as
                select p.*
                from seg_kc.FSK_CDD_RULE_PARAMETER p
                inner join FSK_CDD_RULE r
                   on p.CDD_RULE_ID = r.CDD_RULE_ID
                      and p.CDD_RULE_VERSION_NO = r.VERSION_NO
            ;
        run;
    %end;

  /****************************************************************************
   ****************************************************************************
   **          Generate code
   ****************************************************************************
   ****************************************************************************/
  %macro cdd_cg_gen_rule_macro(inlib=, outlib=);
      proc sort data=&inlib..FSK_CDD_RULE_PARAMETER out=&outlib..cdd_rule_parm;
          by CDD_RULE_ID CDD_RULE_PARM_NAME;
      run;
      /* create parameter list and factual parameters */
      data  rule_param_list;
          set &outlib..cdd_rule_parm;
          by CDD_RULE_ID;
          length param_dcl $ 1024;
          length param_val $ 1024;
          keep CDD_RULE_ID param_dcl param_val;
          retain param_dcl param_val;
          if first.CDD_RULE_ID then do;
              param_dcl= trim(left(CDD_RULE_PARM_NAME)) || "=";
              if    CDD_RULE_PARM_TYPE_CODE = 'CHARL' or CDD_RULE_PARM_TYPE_CODE = 'CHARLIST'
                 or CDD_RULE_PARM_TYPE_CODE = 'NUML' or CDD_RULE_PARM_TYPE_CODE = 'EXPR' then
                param_val= trim(left(CDD_RULE_PARM_NAME)) || '=%str(' || trim(left(CDD_RULE_PARM_VAL)) || ")";
            else
                param_val= trim(left(CDD_RULE_PARM_NAME)) || "=" || trim(left(CDD_RULE_PARM_VAL));
          end;
          else do;
              param_dcl = trim(left(param_dcl)) || ", " || trim(left(CDD_RULE_PARM_NAME)) || "=";
              if   CDD_RULE_PARM_TYPE_CODE = 'CHARL' or CDD_RULE_PARM_TYPE_CODE = 'CHARLIST'
                or CDD_RULE_PARM_TYPE_CODE = 'NUML' or CDD_RULE_PARM_TYPE_CODE = 'EXPR' then
                  param_val = trim(left(param_val)) || ", " || trim(left(CDD_RULE_PARM_NAME)) || '=%str(' || trim(left(CDD_RULE_PARM_VAL)) || ")";
              else
                  param_val = trim(left(param_val)) || ", " || trim(left(CDD_RULE_PARM_NAME)) || "=" || trim(left(CDD_RULE_PARM_VAL)) ;

              put param_dcl;
              put param_val;
          end;
          if last.CDD_RULE_ID then do;
             output;
             param_dcl = '';
             param_val = '';
          end;
      run;

    /* add parameters for codegen, exclude rules that are not included in a group */
      proc sql;
          create table &outlib..cdd_rule_with_parm
              as select r.*, p.param_dcl, p.param_val, g.CDD_RULE_GROUP_INPUT_DS_NAME, g.CDD_RULE_GROUP_CATEGORY_CODE,
                        g.CDD_RULE_GROUP_FILE_NAME, g.CDD_RULE_GROUP_NAME, g.VERSION_NO as CDD_RULE_GROUP_VERSION_NO
              from &inlib..FSK_CDD_RULE r
              left join rule_param_list p
              on r.CDD_RULE_ID = p.CDD_RULE_ID
              inner join FSK_CDD_RULE_GROUP g
              on r.CDD_RULE_GROUP_ID = g.CDD_RULE_GROUP_ID
          ;
      quit;

      proc sort data=&outlib..cdd_rule_with_parm;
          by CDD_RULE_GROUP_ID CDD_RULE_ORDER_IN_GROUP;
      quit;

      data _null_;
          set &outlib..cdd_rule_with_parm;
          by CDD_RULE_GROUP_ID;
        length rulesFilename $200;
        length locFileName $200;
        rulesFilename = "&codegen_path./cdd_rules_" || trim(left("&cdd_deploy_date..sas"));
        file rg1 filevar=rulesFilename;
        locFileName = "&cdd_sm_rule_path./" || trim(left(CDD_RULE_ID)) || '_v_' || trim(left(VERSION_NO)) || '.sas';

        put   '/* Rule: ' CDD_RULE_NAME  '       Last updated: ' CDD_UPDATE_DATE +(-1) '.000000 */';
        put   '%macro rule_' CDD_RULE_ID +(-1) '( ' PARAM_DCL ');';
        put   '    %cdd_rule_definition(cdd_rule_id=' CDD_RULE_ID +(-1) ', version_no=' VERSION_NO +(-1) ', cdd_rule_type=' CDD_RULE_TYPE_CODE +(-1) ',';
        put   '                         file_name=' locFileName +(-1) ', cdd_rule_save_id_flg=' CDD_RULE_SAVE_ID_FLG  +(-1) ',';
        put   '                         category_code=' CDD_RULE_GROUP_CATEGORY_CODE +(-1) ', input_ds_name=' CDD_RULE_GROUP_INPUT_DS_NAME  +(-1);
        put   '                        );';
        put   '%mend rule_' CDD_RULE_ID ';';
        put   ' ';
      run;
  %mend cdd_cg_gen_rule_macro;

  %macro cdd_cg_gen_rule_group(inlib=, outlib=);
      proc sort data=&outlib..cdd_rule_with_parm;
          by CDD_RULE_GROUP_ID CDD_RULE_ORDER_IN_GROUP;
      quit;
      /* generate all other rules in the group */
      data _null_;
        set &outlib..cdd_rule_with_parm;
        by CDD_RULE_GROUP_ID;
        length groupFilename $255;
        length locFileName $255;
        if first.CDD_RULE_GROUP_ID then do;
          groupFilename = "&codegen_path./" || trim(left(CDD_RULE_GROUP_FILE_NAME));
        end;
        file rg1 filevar=groupFilename;

        if first.CDD_RULE_GROUP_ID then do;
           locFileName = "&cdd_sm_rule_group_path./" || 'h' || trim(left(CDD_RULE_GROUP_ID)) || '_v_' || trim(left(CDD_RULE_GROUP_VERSION_NO)) || '.sas';
           put   '/* Group: ' CDD_RULE_GROUP_NAME '         Last updated: ' CDD_UPDATE_DATE '*/';
           put   ' ';
           put   '/* group header */';
           put   '%inc "' locFileName +(-1) '";';
        end;

        put   ' ';
        put   '/* Rule: ' CDD_RULE_NAME '*/';
        /* rules operating on non-merged prep data sets also output category code */
        put   '%rule_' CDD_RULE_ID  '( ' param_val ');';
        if last.CDD_RULE_GROUP_ID then do;
           locFileName = "&cdd_sm_rule_group_path./" || 'f' || trim(left(CDD_RULE_GROUP_ID)) || '_v_' || trim(left(CDD_RULE_GROUP_VERSION_NO)) || '.sas';;
           put   ' ';
           put   '/*  group footer */';
           put   '%inc "' locFileName +(-1) '";';
        end;
      run;
  %mend cdd_cg_gen_rule_group;

  %cdd_cg_gen_rule_macro(inlib=work, outlib=work);
  %cdd_cg_gen_rule_group(inlib=work, outlib=work);

  %if &gen_type = T %then %do;
    %if %kupcase(&outlib) ne WORK %then %do;
      data &outlib..FSK_CDD_RULE;
        set work.FSK_CDD_RULE;
      run;
      data &outlib..FSK_CDD_RULE_GROUP;
        set work.FSK_CDD_RULE_GROUP;
      run;
      data &outlib..FSK_CDD_RULE_PARAMETER;
        set work.FSK_CDD_RULE_PARAMETER;
      run;
    %end;
  %end;

  %macro cdd_run_crr_codegen ();
    %let cdd_codegen_success_flg=0;
    %cdd_crr_codegen(  cdd_codegen_path=&codegen_path
                     , cdd_rule_libname=&outlib
                     , cdd_deployment_id=&cdd_deploy_date
                     , cdd_codegen_success_flg_mvar=cdd_codegen_success_flg
                    );
    %if &cdd_codegen_success_flg ne 1 %then %do;
          %cdd_sm_abort(msg=crr_codegen_err)
    %end;
    %if &cdd_gen_success_flg_mvar ne %then %do;
         %let &cdd_gen_success_flg_mvar = 1;
    %end;
  %mend cdd_run_crr_codegen;

  %cdd_run_crr_codegen;

%mend cdd_generate_rules;