/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    cdd_prep_entity_list.sas
| PRODUCT:  SAS Customer Due Diligence
| SYSTEM:  Windows, Unix
| DESC:    select the list of entities for scoring 
|    
+------------------------------------------------------------------------------+
| USAGE: %cdd_prep_entity_list;
+------------------------------------------------------------------------------+
| COMMENTS:
|   INPUTS:
|      RELATIONSHIP_TYPE_CODE=relationship code in PARTY_ASSOC
|                             identifying identical parties
|                             if the parameter is blank, PARTY_ASSOC will not be used  
|      STOP_SCORING_CODE_LIST=list of case investigation final disposition codes 
|                             that are used to determine if the party should stop being  
|                             scored forever.  
|                             blank if all parties are subject to scoring.
|      SCORE_EXTERNAL_PARTIES = if value is true then FSC_PARTY_DIM parties that have
|                             have EXT_APRTY_IND="Y" will be included for scoring,
|                             otherwise they are excluded
|   OUTPUT:
|      cjobstat.CDDDS_ENTITY_LIST_PREP
|      This table will be used to create the prep files which are specific to a category 
|      For example, CDDDS_COUNTRY_RISK_PREP
|  Global macro variable: SCORE_ALL_YN defined in CDD main process job
|                         It is used to determine if all parties should be scored
|                         It is not defined. It will be initialized as N.
|                    
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2013-2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro cdd_prep_entity_list(relationship_type_code_list="IDENTICAL"
                            ,stop_scoring_code_list="DEMARKET"
                            ,score_external_parties=false
                            );


    %if not %symexist(score_all_yn) %then %let score_all_yn=N;

    /* if no relationship_type_code is provided, PARTY_ASSOC will not be used */
    %if %str(&relationship_type_code_list)=%str() %then %let party_assoc_YN=N;
    %else %let party_assoc_YN=Y;
    %let primary_entity_level_code="PTY";



    /* Create master list */
    proc sql;
         %let mtr_obs=0;
         create table master_entity_list as
         select distinct
                     party_number as PRIMARY_ENTITY_NUMBER 'Primary Entity Number'
                   , &primary_entity_level_code as PRIMARY_ENTITY_LEVEL_CODE 'Primary Entity Level Code'
                   , party_key as PRIMARY_ENTITY_KEY 'Primary Entity Key'
                   , party_number as PARTY_NUMBER 'Related Party Number'
                   , party_type_desc
                   , customer_since_date
                   , employee_ind
                   , politically_exposed_person_ind
                   , negative_news_ind
                   , ben_own_exempt_ind
              %if &score_all_YN=Y %then %do;
                   , '1'  as CDD_SCORING_FLG length=1
                   , '0'  as CDD_REVIEW_FLG length=1
              %end;
              %else %do;
                   , '0'  as CDD_SCORING_FLG length=1
                   , '0'  as CDD_REVIEW_FLG length=1
              %end;
            from  seg_kc.fsc_party_dim as b
            where change_current_ind='Y'
               %if %str(&score_external_parties) ne %str(true) %then %do;
                  /* external parties in FSC_PARTY_DIM are not scored by default */
                  and (external_party_ind is NULL or external_party_ind ne 'Y')
               %end;
            ;
            %let mtr_obs=&sqlobs;
    quit;
    %fcf_rcset(&syserr)
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*  Flag the parties that who have party or account dimension level changes. */
    /*  The flag is CDD_SCORING_FLG                                              */


    %if %str(&score_all_yn) ne %str(Y) %then %do;
       %cdd_prep_chk_pty_chg(indsn=master_entity_list);
       %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
            %fcf_put_error;
            %goto MACRO_END;
       %end;
    %end; /* end score_all_yn ne Y */

    %if &PARTY_ASSOC_YN=Y %then %do;
       %let assoc_obs=0;

       /*************************************************/
       /* Get the primary parties and their identical   */
       /* party numbers.                                */
       /*************************************************/
       proc sql noprint;
            create table identical_party_list as
            select distinct
                     a.party_number as primary_entity_number
                   , &primary_entity_level_code as primary_entity_level_code
                   , a.related_party_number as party_number
                   , b.party_key as primary_entity_key
                   , c.party_type_desc
                   , c.customer_since_date
                   , c.employee_ind
                   , c.politically_exposed_person_ind
                   , c.negative_news_ind
                   , c.ben_own_exempt_ind
                %if &score_all_YN=Y %then %do;
                   , '1'  as CDD_SCORING_FLG length=1
                   , '0'  as CDD_REVIEW_FLG length=1
                %end;
                %else %do;
                   , '0'  as CDD_SCORING_FLG length=1
                   , '0'  as CDD_REVIEW_FLG length=1
                %end;


            from  seg_kc.fsc_party_assoc as a,
                  seg_kc.fsc_party_dim as b,
                  master_entity_list as c
            where b.change_current_ind='Y'
                  and a.change_current_ind='Y'
               %if %str(&score_external_parties) ne %str(true) %then %do;
                  /* external parties in FSC_PARTY_DIM are not scored by default */
                  and (b.external_party_ind is NULL or b.external_party_ind ne 'Y')
               %end;
                  and a.party_number=b.party_number
                  and a.relationship_type_code in  (&relationship_type_code_list)
                  and b.party_number=c.primary_entity_number
            ;
       quit;
       %fcf_rcset(&syserr)
       %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
            %fcf_put_error;
            %goto MACRO_END;
       %end;
       %let assoc_obs=&sqlobs;

       %if &assoc_obs > 0 %then %do;
            proc sql noprint;
               /* get the list primary parties who have changes at their identical party record level */
               create table identical_party_chg_list as
                    select distinct a.primary_entity_number
                    from identical_party_list as a
                    inner join master_entity_list as b
                         on a.party_number = b.party_number
                         and b.cdd_scoring_flg='1'
               ;
            quit;
            %fcf_rcset(&syserr)
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                   %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
                   %fcf_put_error;
                   %goto MACRO_END;
            %end;
            /* flag to re-score the primary_entity_number when any of its related parties have been changed */
            proc sql noprint;
                   update identical_party_list
                       set cdd_scoring_flg='1'
                       where primary_entity_number in
                             (select distinct primary_entity_number from identical_party_chg_list)
                   ;
            quit;
            /* flag to re-score all identical parties if the primary party has been changed */
            proc sql noprint;
                   update identical_party_list
                       set cdd_scoring_flg='1'
                       where primary_entity_number in
                             (select distinct primary_entity_number
                              from master_entity_list where cdd_scoring_flg='1')
                   ;
            quit;
            %fcf_rcset(&syserr)
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
              %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
              %fcf_put_error;
              %goto MACRO_END;
            %end;
            /* remove the parties who are be part of identical party list from                */
            /* the master entity list                                                         */
            /* Note: the primary parties are kept with the party_number=primary_entity_number */
            proc sql noprint;
                 delete from master_entity_list
                 where party_number in (select distinct party_number from
                                      identical_party_list
                                     )

                  ;
            quit;
            %fcf_rcset(&syserr)
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
              %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
              %fcf_put_error;
              %goto MACRO_END;
            %end;

            /* add the identical party records to the master entity list */
            proc append base=master_entity_list data=identical_party_list force;
            run;
            %fcf_rcset(&syserr)
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                  %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
                  %fcf_put_error;
                  %goto MACRO_END;
            %end;
       %end; /* end assoc_obs > 0 */
    %end; /* end PARTY_ASSOC_YN=Y */


     /* Check if the FSK_CDD_ENTITY_FACT is empty */
     proc sql noprint;
          select count(*) 
          from seg_kc.fsk_cdd_entity_fact
          where primary_entity_level_code=&primary_entity_level_code
           ;
     quit;
     %let cdd_fact_obs=&sqlobs;
    
     %if &cdd_fact_obs > 0 %then %do;  /* if CDD_ENTITY_FACT is not empty */

          /* Check for review list */
          proc sql noprint;
               create table review_list as
               select distinct primary_entity_number
               from seg_kc.fsk_cdd_entity_fact
               where next_review_date ne . and next_review_date <= &cdd_run_asofdttm
                     and primary_entity_level_code=&primary_entity_level_code
					 and curr_review_scoring_event_key is null
               ;
          quit;
          %let review_obs=&sqlobs;
          %fcf_rcset(&syserr)
          %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
              %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
              %fcf_put_error;
              %goto MACRO_END;
          %end;
        
          %if &review_obs > 0 %then %do;
              proc sql noprint;
                  update master_entity_list
                  set cdd_scoring_flg='1'
                      ,cdd_review_flg='1'
                  where primary_entity_number in
                      (select distinct primary_entity_number from review_list)
                  ;
              quit;
              %fcf_rcset(&syserr)
              %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                   %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
                   %fcf_put_error;
                   %goto MACRO_END;
              %end;
          %end; /* end review_obs > 0 */
     %end; /* end cdd_fact_obs > 0 */


    %if %str(&stop_scoring_code_list) ne %str() %then %do;
        /* Exclude the parties who should stop being scored */
        proc sql;
          create table stop_scoring_list as
          select distinct 
                    primary_entity_number
           from seg_kc.fsk_cdd_entity_fact 
           where final_disposition_code in (&stop_scoring_code_list)
                 and primary_entity_level_code=&primary_entity_level_code
           ;
        quit;
        %put syserr=&syserr;

        %fcf_rcset(&syserr)
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
              %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
              %fcf_put_error;
              %goto MACRO_END;
        %end;
           /* set the cdd_scoring_flg to 0 if the primary  party should not be scored */ 
        proc sql noprint;
           update master_entity_list
           set cdd_scoring_flg='0' 
           where primary_entity_number in 
                 (select distinct primary_entity_number 
                  from stop_scoring_list
                 ) 
           ;
        quit;
        %fcf_rcset(&syserr)
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
             %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
             %fcf_put_error;
             %goto MACRO_END;
        %end;
    %end; /* stop_scoring_code_list is not emtpy */


    /******************************************************/
    /* Create the final prep file                         */
    /* with only the primary parties who should be scored */
    /******************************************************/
    proc sql noprint;
         create table cddds_entity_list as
         select   a.primary_entity_level_code 'Primary Entity Level Code'
                , a.primary_entity_key 'Primary Entity Key'
                , a.primary_entity_number 'Primary Entity Number'
                , a.party_number 'Identical Party Number'
                , a.party_type_desc 'Party Type'
                , a.customer_since_date 'Customer Since Date'
                , a.employee_ind 'Employee Indicator'
                , a.politically_exposed_person_ind 'Politically Exposed Indicator'
                , a.negative_news_ind 'Negative News Indicator'
                , a.cdd_review_flg 'CDD Periodic Review Indicator'
                , a.ben_own_exempt_ind 'Beneficial Ownership Exempt'
                , b.override_score_class_code 'Override Score Class Code'
                , b.final_disposition_code 'Last Review Disposition Code'
                , b.last_review_date 'Last Review Date'
                , b.next_review_date 'Next Review Date'
                , b.last_scoring_event_key 'Last Scoring Event Key'
                , b.last_review_scoring_event_key 'Last Review Scoring Event Key'
                , b.curr_review_scoring_event_key 'Current Review Scoring Event Key'
                , &cdd_run_asofdttm as EVENT_DATE 'CDD Scoring Event Date' format=datetime20.

         from master_entity_list
                      (where=(cdd_scoring_flg='1'))
              as a
         left join  seg_kc.fsk_cdd_entity_fact
                       (where=(primary_entity_level_code=&primary_entity_level_code))
              as b
         on a.primary_entity_number=b.primary_entity_number
         ;
    quit;
    %fcf_rcset(&syserr)
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
      %fcf_put_error;
      %goto MACRO_END;
    %end;
    /*****************************************************************/ 
    /* Include the investigation info from FSK_CDD_REVIEW table */
    /* into the entity prep file                                     */ 
    /*****************************************************************/ 
    proc sql noprint;
        create table cjobstat.cddds_entity_list as
        select xyz.*
               ,review.review_key as CURR_REVIEW_KEY 'Current Review Key'
               ,review.STATUS_CODE as CURR_REVIEW_STATUS_CD 'Current Review Status Code'
        from (
                select xy.*
                   , z.rule_score as LAST_REVIEW_SCORE 'Last Review Score'
                   , z.score_classification_code as LAST_REVIEW_CLASS_CODE 'Last Review Score Classification Code'
                from (select  x.*
                            , y.event_date as LAST_SCORE_DATE 'Last Score Date'
                            , y.rule_score as LAST_SCORE  'Last Score'
                            , y.score_classification_code as LAST_SCORE_CLASS_CODE 'Last Score Classification Code'
                      from cddds_entity_list as x
                      left join   seg_kc.fsk_cdd_scoring_event as y
                           on   x.last_scoring_event_key=y.cdd_scoring_event_key
                  ) xy
                left join      seg_kc.fsk_cdd_scoring_event as z
                     on  xy.last_review_scoring_event_key=z.cdd_scoring_event_key
              ) as xyz
           left join seg_kc.fsk_cdd_review as review
                on xyz.primary_entity_number=review.primary_entity_number
                   and xyz.primary_entity_level_code = review.primary_entity_level_code
                   and review.status_code not in (&cdd_closed_review_status_lst)
        ;
    quit;
    %fcf_rcset(&syserr)
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
          %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
          %fcf_put_error;
          %goto MACRO_END;
    %end;
   /* Clear next review date for the parties who are under new review */
    proc sql noprint;
       update cjobstat.cddds_entity_list
       set next_review_date=.
       where  cdd_review_flg='1'
       ;
    quit;
    %fcf_rcset(&syserr)
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
         %fcf_put_error;
         %goto MACRO_END;
    %end;

 %MACRO_END:
   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %cdd_error;
   %else %do;
      %cdd_success;
   %end;
%mend;
 
