/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    cdd_prep_lookup_fmt.sas
| PRODUCT: SAS Customer Due Diligence
| SYSTEM:  Windows, Unix
| DESC:    Load lookup_dim data into sas format for easy lookup 
|    
+------------------------------------------------------------------------------+
| USAGE: %cdd_load_lookup_fmt(lookup_level_1_type_code=ECRR,datefmt=nldate10.);
+------------------------------------------------------------------------------+
| COMMENTS:
|   INPUTS:
|      LOOKUP_LEVEL_1_TYPE_CODE= LOOKUP_LEVEL_1_TYPE_CODE value to filter SEG_KC.FSC_LOOKUP_DIM.
|                                If this parameter is blank, all records in FSC_LOOKUP_DIM 
|                                will be loaded. 
|      DATEFMT=SAS date format for converting FSC_LOOKUP_DIM.LOOKUP_DATE_VALUE values
|
| NOTE: 1. FSC_LOOKUP_DIM.LOOKUP_LEVEL_2_TYPE_CODE will be used for format name.
|          Therefore LOOKUP_LEVEL_2_TYPE_CODE must be a valid SAS format name. 
|       2. The first record for the same LOOKUP_LEVEL_2_TYPE_CODE data group will be used to determine 
|          whether which LOOKUP_*_VALUE is used for the format label 
|          If more than one LOOKUP_*_VALUE is non-empty, the order of preference is NUM, DATE, CHAR
|          LOOKUP_NUM_VALUE will be converted to a string using SAS Best. format 
|          LOOKUP_DATE_VALUE will be converted to a string using the input date format. By default
|          it is nldate10. 
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2013-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro cdd_prep_lookup_fmt(lookup_level_1_type_code=CRR,datefmt=nldate10.);
                               
   proc sort data=seg_kc.fsc_lookup_dim nodupkey out=fsc_lookup_dim ;
        BY lookup_level_1_type_code lookup_level_2_type_code lookup_code;
        WHERE change_current_ind = 'Y';
     %if %str(&lookup_level_1_type_code) ne %str() %then
     %do;
        WHERE lookup_level_1_type_code="&lookup_level_1_type_code";
     %end;
   run;
   %fcf_rcset(&syserr)
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
         %fcf_put_error;
          %goto MACRO_END;
	
    %end;  

    data cdd_lookup_fmt;
         set fsc_lookup_dim ;
         length fmtname   start $ 30
                label $ 40
                type $ 1
                hlo  $ 1 
                nvalid_flg 8
         ;
         by lookup_level_1_type_code lookup_level_2_type_code;
         retain fmtname  type value_type nvalid_flg;
         if first.lookup_level_2_type_code then
         do; 
             fmtname=lookup_level_2_type_code;
             type='C';
             if lookup_num_value ne . then value_type='N';
             else if lookup_date_value ne . then value_type='D';
             else value_type='C';
             nvalid_flg=nvalid(lookup_level_2_type_code);
         end;
          
         if nvalid_flg then
         do;
             start=lookup_code;
             
             label=lookup_char_value;
             if value_type='N' then label=strip(put(lookup_num_value,best.));
             else if value_type = 'D' then label=strip(put(datepart(lookup_date_value),&datefmt));
             output;
             if last.lookup_level_2_type_code and value_type ne 'D' then
             do;
                hlo='O';
                start='**OTHER**';
                label='';
                output;
             end;
         end;
         keep fmtname type start label hlo; 
    run;

	%fcf_rcset(&syserr)
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
         %fcf_put_error;
          %goto MACRO_END;
	
    %end;  

    proc format cntlin=cdd_lookup_fmt ;
    run;
	%fcf_rcset(&syserr)
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
         %fcf_put_error;
          %goto MACRO_END;
	
    %end;  
 

%MACRO_END:

   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %cdd_error;
   %else %do;
      %cdd_success;
   %end;


%mend cdd_prep_lookup_fmt;
