/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_ent_load_entity_txn.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    Creates the table fsk_entity_transaction in Knowledge Center.  This
|          table contains transactions for all entities with active alerts
|          in seg_kc.fsk_alert.  The transactions are selected from fact
|          tables fsc_cash_flow_fact, fsc_account_trade_fact, and 
|          fsc_account_event_fact and augmented with information from the
|          dim tables for display in the Entity Triage UI.
+------------------------------------------------------------------------------+
| USAGE: %fcf_ent_load_entity_txn
+------------------------------------------------------------------------------+
| COMMENTS: INPUTS
|             None
|
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2016-2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_ent_load_entity_txn;

/*------------------------------------------
 * Set error code to 0 and message to blank
 *------------------------------------------*/
 %let trans_rc = 0;
 %let _fcfmsg_=;

 %put NOTE: FCF: Building transaction table with all transactions for alerted entities;


 /*--------------------------------------------------------------------------
 * Find household number to any transactions for entities with HHD alerts
 * This field will be added to the transaction table for searching in the UI
 *--------------------------------------------------------------------------*/
 proc sql;
    %fcf_begin_pass_through(dbflavor=&dbflavor, authenticationDomain=&segkcauthdomain, connOpts=&segKCDBConnOpts);
    execute (delete from &segKCSchema..FSK_TEMP_TABLE) by &dbflavor;
    %fcf_end_pass_through(dbflavor=&dbflavor, commit=yes);
 quit;



 proc sql;

create table fsk_temp_table as 
select distinct ae.alerted_entity_level_code as TEMP_CHAR_1_VALUE,
                ae.alerted_entity_key as TEMP_NUM_1_VALUE,
                ae.primary_entity_number as TEMP_CHAR_2_VALUE
from &seg_kc_libname..fsk_alert ae
where primary_entity_level_code='HHD';
quit;

%fcf_rcset(&syserr);

%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %let _fcfmsg_=ERROR: FCF: Unable to create staging dataset for FSK_TEMP_TABLE load household number;
    %fcf_put_error;
    %goto MACRO_END;
%end;

/* Loading work table into temp table */

proc append base=&seg_kc_libname..fsk_temp_table data=work.fsk_temp_table nowarn force;
quit;

%fcf_rcset(&syserr);
%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: Unable to append to FSK_TEMP_TABLE to load household number;
     %fcf_put_error;
     %goto MACRO_END;
%end;

/* Create macro helper variables to avoid using temp table column names in subsequent joins */
%let alerted_entity_level_code=TEMP_CHAR_1_VALUE;
%let alerted_entity_key=TEMP_NUM_1_VALUE;
%let primary_entity_number=TEMP_CHAR_2_VALUE;
 
/*-------------------------------------------------------------------
 * Find start date of transactions to save based on fcf_start_time
 * and look back value in the FSK_REFERENCE property table
 *-------------------------------------------------------------------*/
 %fcf_get_property_value(property=fcf.svr.aml.entityTriageDMLookback,
                             output_property_value=DMLookBack,
                             output_status=property_status);
 %if &property_status ne 0 %then %do;
      %let _fcfmsg_=WARNING: FCF: Missing property value to define number of days in lookback.  Setting to 90 days;
      %fcf_put_error;
      %let DMLookBack=90;
 %end;

 proc sql noprint;
    select date_key into :start_date_key
    from db_core.fsc_date_dim
    where calendar_date_sas=datepart(&fcf_start_datetime) - &DMLookBack;
 quit;

/*-------------------------------------------------------------------
 * Build table of transactions in cash_flow_fact for all alerted
 * entities with the input alert type 
 *-------------------------------------------------------------------*/

proc sql;
connect to &dbflavor as &dbflavor (authdomain="&segkcAuthDomain." &segKCDBConnOpts);

create table entity_transaction_cff as
select * from 
connection to &dbflavor
(
select distinct aler.alerted_entity_level_code,
       aler.alerted_entity_number,
       trnd.transaction_reference_number as transaction_id,
       cff.currency_amount as transaction_amt,
       trnt.transaction_cdi_desc,
       trnt.primary_medium_desc,
       trnt.secondary_medium_desc,
       trnt.mechanism_desc as transaction_mechanism,
       acc.account_number,
       TMP.&primary_entity_number as household_number,
       ext.full_name as beneficiary_name,
       assc.associate_first_name,
       assc.associate_middle_name,
       assc.associate_last_name,
       brch.branch_name,
       trnd.transaction_description as transaction_desc,
       trns.status_desc as transaction_status,
       cff.time_key,
       cff.date_key,
       dt.calendar_date_sas,
       tm.time_hh,
       tm.time_mm,
       tm.time_ss  
 from  (select transaction_key,
               account_key,
               transaction_type_key,
               time_key,
               date_key,
               transaction_status_key,
               branch_key,
               associate_key,
               beneficiary_ext_party_key,
               currency_amount
              from   &segKCSchema..fsc_CASH_FLOW_FACT   
                   where date_key >= &start_date_key
                         and account_key in 
                    (select account_key 
                    from &segKCSchema..fsk_account_keys_acc
                    union
                    select account_key 
                    from &segKCSchema..fsk_account_keys_pty
                    union
                    select account_key 
                    from &segKCSchema..fsk_account_keys_hhd
                    union
                    select account_key 
                    from &segKCSchema..fsk_account_keys_asc
                    )
         ) cff  
join (select distinct account_key, alert_id 
                    from &segKCSchema..fsk_account_keys_acc
                    union
                    select account_key , alert_id 
                    from &segKCSchema..fsk_account_keys_pty
                    union
                    select account_key , alert_id 
                    from &segKCSchema..fsk_account_keys_hhd
                    union
                    select account_key , alert_id 
                    from &segKCSchema..fsk_account_keys_asc
                    ) acct_key
on acct_key.account_key=cff.account_key
join &segKCSchema..fsk_alert aler
on acct_key.alert_id = aler.alert_id
and aler.alert_status_code='ACT'
left join &segKCSchema..fsk_temp_table TMP
on aler.alerted_entity_level_code=TMP.&alerted_entity_level_code
 and aler.alerted_entity_key=TMP.&alerted_entity_key
join &segKCSchema..fsc_account_dim acc
on cff.account_key=acc.account_key
join &segKCSchema..fsc_transaction_dim trnd
on cff.transaction_key=trnd.transaction_key
join &segKCSchema..fsc_transaction_type_dim trnt
on trnt.transaction_type_key=cff.transaction_type_key
join &segKCSchema..fsc_ext_party_account_dim ext
on cff.beneficiary_ext_party_key=ext.ext_party_account_key
join &segKCSchema..fsc_associate_dim assc
on cff.associate_key=assc.associate_key
join &segKCSchema..fsc_branch_dim brch
on cff.branch_key=brch.branch_key
join &segKCSchema..fsc_transaction_status_dim trns
on cff.transaction_status_key=trns.transaction_status_key 
join &segKCSchema..fsc_date_dim dt
on cff.date_key=dt.date_key
join &segKCSchema..fsc_time_dim tm
on cff.time_key=tm.time_key
);

disconnect from &dbflavor;
quit;

%fcf_rcset(&sqlrc);
 %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: Failed to add create transaction table entity_transaction_cff;
     %fcf_put_error;
     %goto MACRO_END;
 %end;


/*-------------------------------------------------------------------
 * Build table of transactions in account_trade_fact for all alerted
 * entities with the input alert type 
 *-------------------------------------------------------------------*/
proc sql;

connect to &dbflavor as &dbflavor (authdomain="&segkcAuthDomain." &segKCDBConnOpts);

create table entity_transaction_atf as
select * from connection to &dbflavor
(
select distinct aler.alerted_entity_level_code,
       aler.alerted_entity_number,
       -1 as alert_id,
       atf.transaction_key,
       trnd.transaction_reference_number as transaction_id,
       atf.principal_amount_in_txn_ccy as transaction_amt,
       trnt.transaction_cdi_desc,
       trnt.primary_medium_desc,
       trnt.secondary_medium_desc,
       trnt.mechanism_desc as transaction_mechanism,
       atf.account_key,
       acc.account_number,
       TMP.&primary_entity_number as household_number,
       ' ' as beneficiary_name,
       assc.associate_first_name,
       assc.associate_middle_name,
       assc.associate_last_name,
       brch.branch_name,
       trnd.transaction_description as transaction_desc,
       trns.status_desc as transaction_status,
       atf.execution_time_key as time_key,
       trnd.date_key,
       dt.calendar_date_sas,
       tm.time_hh,
       tm.time_mm,
       tm.time_ss  
 from  (select distinct transaction_key,
               account_key,
               transaction_type_key,
               execution_time_key,
               transaction_status_key,
               branch_key,
               associate_key,
               principal_amount_in_txn_ccy
              from   &segKCSchema..fsc_ACCOUNT_TRADE_FACT   
                   where trade_date_key >= &start_date_key
                   and account_key in 
                    (select account_key 
                    from &segKCSchema..fsk_account_keys_acc
                    union
                    select account_key 
                    from &segKCSchema..fsk_account_keys_pty
                    union
                    select account_key 
                    from &segKCSchema..fsk_account_keys_hhd
                    union
                    select account_key 
                    from &segKCSchema..fsk_account_keys_asc
                    )
         ) atf  
join  (select account_key, alert_id 
                    from &segKCSchema..fsk_account_keys_acc
                    union
                    select account_key , alert_id 
                    from &segKCSchema..fsk_account_keys_pty
                    union
                    select account_key , alert_id 
                    from &segKCSchema..fsk_account_keys_hhd
                    union
                    select account_key , alert_id 
                    from &segKCSchema..fsk_account_keys_asc
                    ) acct_key
on acct_key.account_key=atf.account_key
join &segKCSchema..fsk_alert aler
on acct_key.alert_id = aler.alert_id
and aler.alert_status_code='ACT'
left join &segKCSchema..fsk_temp_table TMP
on aler.alerted_entity_level_code=TMP.&alerted_entity_level_code
 and aler.alerted_entity_key=TMP.&alerted_entity_key
join &segKCSchema..fsc_account_dim acc
on atf.account_key=acc.account_key
join &segKCSchema..fsc_transaction_dim trnd
on atf.transaction_key=trnd.transaction_key
join &segKCSchema..fsc_transaction_type_dim trnt
on trnt.transaction_type_key=atf.transaction_type_key
join &segKCSchema..fsc_associate_dim assc
on atf.associate_key=assc.associate_key
join &segKCSchema..fsc_branch_dim brch
on atf.branch_key=brch.branch_key
join &segKCSchema..fsc_transaction_status_dim trns
on atf.transaction_status_key=trns.transaction_status_key
join &segKCSchema..fsc_date_dim dt
on trnd.date_key=dt.date_key
join &segKCSchema..fsc_time_dim tm
on atf.execution_time_key=tm.time_key
);

disconnect from &dbflavor;


quit;

%fcf_rcset(&sqlrc);
 %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: Failed to add create transaction table entity_transaction_atf;
     %fcf_put_error;
     %goto MACRO_END;
 %end;


/*-------------------------------------------------------------------
 * Build table of transactions in account_event_fact for all alerted
 * entities with the input alert type 
 *-------------------------------------------------------------------*/
proc sql;

connect to &dbflavor as &dbflavor (authdomain="&segkcAuthDomain." &segKCDBConnOpts);

create table entity_transaction_aef as
select * from connection to &dbflavor

   (select distinct aler.alerted_entity_level_code,
       aler.alerted_entity_number,
       aef.transaction_key,
       trnd.transaction_reference_number as transaction_id,
       0 as transaction_amt,
       trnt.transaction_cdi_desc,
       trnt.primary_medium_desc,
       trnt.secondary_medium_desc,
       trnt.mechanism_desc as transaction_mechanism,
       aef.account_key,
       acc.account_number,
       TMP.&primary_entity_number as household_number,
       ' ' as beneficiary_name,
       '' as associate_first_name,
       '' as associate_middle_name,
       '' as associate_last_name,
       brch.branch_name,
       trnd.transaction_description as transaction_desc,
       trns.status_desc as transaction_status,
       aef.time_key,
       aef.date_key, 
       dt.calendar_date_sas,
       tm.time_hh,
       tm.time_mm,
       tm.time_ss 
    from  (select distinct transaction_key,
               account_key,
               transaction_type_key,
               date_key,
               time_key,
               transaction_status_key,
               branch_key
              from   &segKCSchema..fsc_ACCOUNT_EVENT_FACT   
                   where date_key >= &start_date_key
                   and account_key in 
                    (select account_key 
                    from &segKCSchema..fsk_account_keys_acc
                    union
                    select account_key 
                    from &segKCSchema..fsk_account_keys_pty
                    union
                    select account_key 
                    from &segKCSchema..fsk_account_keys_hhd
                    union
                    select account_key 
                    from &segKCSchema..fsk_account_keys_asc
                    )
         ) aef  
join (select account_key, alert_id 
                    from &segKCSchema..fsk_account_keys_acc
                    union
                    select account_key , alert_id 
                    from &segKCSchema..fsk_account_keys_pty
                    union
                    select account_key , alert_id 
                    from &segKCSchema..fsk_account_keys_hhd
                    union
                    select account_key , alert_id 
                    from &segKCSchema..fsk_account_keys_asc
                    ) acct_key
on acct_key.account_key=aef.account_key
join &segKCSchema..fsk_alert aler
on acct_key.alert_id = aler.alert_id
and aler.alert_status_code='ACT'
left join &segKCSchema..fsk_temp_table TMP
on aler.alerted_entity_level_code=TMP.&alerted_entity_level_code
 and aler.alerted_entity_key=TMP.&alerted_entity_key
join &segKCSchema..fsc_account_dim acc
on aef.account_key=acc.account_key
join &segKCSchema..fsc_transaction_dim trnd
on aef.transaction_key=trnd.transaction_key
join &segKCSchema..fsc_transaction_type_dim trnt
on trnt.transaction_type_key=aef.transaction_type_key
join &segKCSchema..fsc_branch_dim brch
on aef.branch_key=brch.branch_key
join &segKCSchema..fsc_transaction_status_dim trns
on aef.transaction_status_key=trns.transaction_status_key 
join &segKCSchema..fsc_date_dim dt
on aef.date_key=dt.date_key
join &segKCSchema..fsc_time_dim tm
on aef.time_key=tm.time_key
);
 disconnect from &dbflavor;
quit;


%fcf_rcset(&sqlrc);
 %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: Failed to add create transaction table entity_transaction_aef;
     %fcf_put_error;
     %goto MACRO_END;
 %end;

/*-----------------------------------
 * Get transactions for EXT entities
 *----------------------------------*/
%fcf_ent_load_entity_txn_ext;

/*--------------------------------
 * Combine tables and add keys
 *-------------------------------*/

data entity_transaction;
if 1 eq 0 then set seg_kc.fsk_entity_transaction;
set entity_transaction_cff entity_transaction_atf entity_transaction_aef entity_transaction_ext;

    fsk_entity_transaction_pk=_n_;
    associate_name=catx(' ',associate_first_name,associate_middle_name,associate_last_name);
    transaction_date=dhms(calendar_date_sas,input(time_hh,2.),input(time_mm,2.),input(time_ss,2.));
    created_timestamp=&fcf_start_datetime;

    drop associate_first_name associate_last_name associate_middle_name 
        calendar_date_sas time_hh time_mm time_ss
        account_key transaction_key alert_id;
run;


/*--------------------------------
 * Load table into database
 *-------------------------------*/

proc append base=&db_kc_libname..fsk_entity_transaction data=entity_transaction force nowarn;
run;

/*--------------------------------
 * Clean-up Work Directory
 *-------------------------------*/

proc datasets library=work nolist;
    delete entity_transaction;
    delete entity_transaction_ext;
    delete entity_transaction_cff;
    delete entity_transaction_atf;
    delete entity_transaction_aef;
run;

 %MACRO_END:

   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %fcf_execute_macro(fcf_error);
   %else %do;
      %fcf_execute_macro(fcf_success);
   %end;
%mend fcf_ent_load_entity_txn;
