/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_fundstracker_create_data.sas
| PRODUCT: SAS Anti-Money Laundering
| SYSTEM:  Windows, Unix
| DESC:    Creates transfer activity data for FundsTracker
|          and stores it in FSK_INT_XFER.
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_fundstracker_create_data;

    /*------------
    * Set Module
    *------------*/
    %local _fcfmodule_;
    %let _fcfmodule_=FCF_FUNDSTRACKER_CREATE_DATA;

    /*------------------------------------------
    * Set error code to 0 and message to blank
    *------------------------------------------*/
    %let trans_rc = 0;
    %let _fcfmsg_=;

    %let notEqual = !=;
    %if %upcase(&dbflavor) eq TERADATA
    %then %do;
        %let notEqual = <>;
    %end;
    /*------------------------------------
    * Get the number of days to look back
    *-------------------------------------*/
    %fcf_get_property_value(property=ft.lookbackDays,
                      output_property_value=ft_lookback_days,
                      output_status=ft_lookback_days_rc
                      );

    %if &ft_lookback_days_rc ne 0 %then %do;
        %let _fcfmsg_=ERROR: FT: Unable to retrieve value of property ft.lookbackDays from FSK_PREFERENCE;
        %let trans_rc = 999999;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*----------------------------------------------
    * Get the minum currency amound for transaction
    *-----------------------------------------------*/
    %fcf_get_property_value(property=ft.minAmount,
                      output_property_value=ft_min_currency_amt,
                      output_status=ft_min_currency_amt_rc
                      );

    %if &ft_min_currency_amt_rc ne 0 %then %do;
        %let _fcfmsg_=ERROR: FT: Unable to retrieve value of property ft.minAmount from FSK_PREFERENCE;
        %let trans_rc = 999999;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*----------------------------------------
    * Get the list of internal transfer types
    *-----------------------------------------*/
    %fcf_get_property_value(property=ft.primaryMedium.internal,
                      output_property_value=ft_primary_medium_internal,
                      output_status=ft_primary_medium_internal_rc
                      );

    %if &ft_primary_medium_internal_rc ne 0 %then %do;
        %let _fcfmsg_=ERROR: FT: Unable to retrieve value of property ft.primaryMedium.internal from FSK_PREFERENCE;
        %let trans_rc = 999999;
        %fcf_put_error;
        %goto MACRO_END;
    %end;


    /*----------------------------------------
    * Get the list of external transfer types
    *-----------------------------------------*/
    %fcf_get_property_value(property=ft.primaryMedium.external,
                      output_property_value=ft_primary_medium_external,
                      output_status=ft_primary_medium_external_rc
                      );

    %if &ft_primary_medium_external_rc ne 0 %then %do;
        %let _fcfmsg_=ERROR: FT: Unable to retrieve value of property ft.primaryMedium.external from FSK_PREFERENCE;
        %let trans_rc = 999999;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc sql;
        %fcf_begin_pass_through(dbflavor=&dbflavor, authenticationDomain=&segkcauthdomain, connOpts=&segKCDBConnOpts);
        execute(DELETE FROM &segKCSchema..FSK_INT_XFER) by &dbflavor;
        %fcf_end_pass_through(dbflavor=&dbflavor, commit=yes);
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FT: Unable to delete contents of FSK_INT_XFER;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*------------------------------------------------------------
    * Calculate lookback date based on runasofdate and preference
    *-------------------------------------------------------------*/

    %if not(%symexist(runasofdate)) %then %do;
        %fcf_get_runasofdate
    %end;

    data _null_;
        x = input(put(&runasofdate,8.),yymmdd8.) - &ft_lookback_days;
        call symput('lookback',kcompress(put(x,yymmdd10.),'-'));
    run;

    %fcf_rcset(&sysrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FT: Unable to calculate lookback date;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*----------------------------------------------------------------------
     * Extract internal transfers between related and unrelated parties.
     * - internal transfers identified by primary_medium_desc for
     *   transaction type matching preference specified set of values
     * - related or unrelated parties defined by related_ind, equal 'Y' or
     *   'N' respectively, on the txns
     * - edge types set to 'R' for txns between related parties, 'U' for
     *   txns between unrelated parties
     *----------------------------------------------------------------------*/
    proc sql;
        create table int_xfer as
            select a.account_number as remitter_account_number,
                   -1 as remitter_party_key,
                   b.account_number as beneficiary_account_number,
                   -1 as beneficiary_party_key,
                   c.date_key,
                   c.time_key,
                   c.currency_amount,
                   c.edge_type_code
                from
                    (
                        select cff.account_key,
                               cff.secondary_account_key,
                               cff.date_key,
                               cff.time_key,
                               cff.currency_amount,
                               case cff.related_ind when 'N' then 'U' when 'Y' then 'R' else ' ' end as edge_type_code
                            from &seg_kc_libname..fsc_cash_flow_fact cff
                            join &seg_kc_libname..fsc_transaction_type_dim ttd on ttd.transaction_type_key = cff.transaction_type_key
                            where cff.date_key >= &lookback and
                                  cff.currency_amount >= &ft_min_currency_amt and
                                  ttd.transaction_cdi_code = 'D' and
                                  ttd.primary_medium_desc in (&ft_primary_medium_internal) and
                                  cff.related_ind in ( 'Y', 'N' )
                    ) c
                join &seg_kc_libname..fsc_account_dim a on a.account_key = c.account_key
                join &seg_kc_libname..fsc_account_dim b on b.account_key = c.secondary_account_key;
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FT: Unable to create extract dataset for internal transfers;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*----------------------------------------------------------------------
     * Extract outgoing (debit) wire transactions
     * - wire transfers identified by primary_medium_desc for transaction
     *   type matching preference specified set of values
     *----------------------------------------------------------------------*/
    proc sql;
        create table ext_xfer_o as
            select a.account_number as remitter_account_number,
                   -1 as remitter_party_key,
                   e.external_party_number as beneficiary_account_number,
                   -1 as beneficiary_party_key,
                   c.date_key,
                   c.time_key,
                   c.currency_amount,
                   'O' as edge_type_code
                from
                    (
                        select cff.account_key,
                               cff.beneficiary_ext_party_key,
                               cff.date_key,
                               cff.time_key,
                               cff.currency_amount,
                               'O' as edge_type_code
                            from &seg_kc_libname..fsc_cash_flow_fact cff
                            join &seg_kc_libname..fsc_transaction_type_dim ttd on ttd.transaction_type_key = cff.transaction_type_key
                            where cff.date_key >= &lookback and
                                  cff.currency_amount >= &ft_min_currency_amt and
                                  ttd.transaction_cdi_code = 'D' and
                                  ttd.primary_medium_desc in (&ft_primary_medium_external)
                    ) c
                join &seg_kc_libname..fsc_account_dim a on a.account_key = c.account_key
                join &seg_kc_libname..fsc_ext_party_account_dim e on e.ext_party_account_key = c.beneficiary_ext_party_key;
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FT: Unable to create extract dataset for outgoing wire transfers;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*----------------------------------------------------------------------
     * Extract incoming (credit) wire transactions
     * - wire transfers identified by primary_medium_desc for transaction
     *   type matching preference specified set of values
     *----------------------------------------------------------------------*/
    proc sql;
        create table ext_xfer_i as
            select e.external_party_number as remitter_account_number,
                   -1 as remitter_party_key,
                   a.account_number as beneficiary_account_number,
                   -1 as beneficiary_party_key,
                   c.date_key,
                   c.time_key,
                   c.currency_amount,
                   'I' as edge_type_code
                from
                    (
                        select cff.account_key,
                               cff.remitter_ext_party_key,
                               cff.date_key,
                               cff.time_key,
                               cff.currency_amount,
                               'O' as edge_type_code
                            from &seg_kc_libname..fsc_cash_flow_fact cff
                            join &seg_kc_libname..fsc_transaction_type_dim ttd on ttd.transaction_type_key = cff.transaction_type_key
                            where cff.date_key >= &lookback and
                                  cff.currency_amount >= &ft_min_currency_amt and
                                  ttd.transaction_cdi_code = 'C' and
                                  ttd.primary_medium_desc in (&ft_primary_medium_external)
                    ) c
                join &seg_kc_libname..fsc_account_dim a on a.account_key = c.account_key
                join &seg_kc_libname..fsc_ext_party_account_dim e on e.ext_party_account_key = c.remitter_ext_party_key;
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FT: Unable to create extract dataset for incoming wire transfers;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*----------------------------------------------------------------------
     * Add the 3 sets of extracted transactions to the FSK_INT_XFER table
     *----------------------------------------------------------------------*/
    proc append base = &seg_kc_libname..FSK_INT_XFER
        data = int_xfer force nowarn;
    run;

    %fcf_rcset(&sysrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FT: Error appending internal transfer txns to FSK_INT_XFER;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc append base = &seg_kc_libname..FSK_INT_XFER
        data = ext_xfer_o force nowarn;
    run;

    %fcf_rcset(&sysrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FT: Error appending outgoing wire txns to FSK_INT_XFER;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc append base = &seg_kc_libname..FSK_INT_XFER
        data = ext_xfer_i force nowarn;
    run;

    %fcf_rcset(&sysrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FT: Error appending incoming wire txns to FSK_INT_XFER;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    %fcf_get_observation_count(table=&seg_kc_libname..FSK_INT_XFER);

    /*----------------------------------------------------------------------
     * Extract current primary owner names for all internal accounts
     * referenced by transactions loaded to FSK_INT_XFER
     *----------------------------------------------------------------------*/
    proc sql;
        %fcf_begin_pass_through(dbflavor=&dbflavor, authenticationDomain=&segkcauthdomain, connOpts=&segKCDBConnOpts);
        execute (delete from &segKCSchema..FSK_TEMP_TABLE) by &dbflavor;
        %fcf_end_pass_through(dbflavor=&dbflavor, commit=yes);
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FT: Error deleting content of FSK_TEMP_TABLE;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc sql;
        %fcf_begin_pass_through(dbflavor=&dbflavor, authenticationDomain=&segkcauthdomain, connOpts=&segKCDBConnOpts);

        execute ( /* For safety, just in case there are any account keys of -1 present, although that would be bad data */
            insert into &segKCSchema..FSK_TEMP_TABLE (TEMP_CHAR_1_VALUE,TEMP_CHAR_2_VALUE) values( 'UNKNOWN', 'UNKNOWN')
        ) by &dbflavor;

        execute (
            insert into &segKCSchema..FSK_TEMP_TABLE (TEMP_CHAR_1_VALUE,TEMP_CHAR_2_VALUE)
                select a.account_number, party_name
                    from (
                        select distinct account_number
                            from (
                                    select remitter_account_number as account_number
                                        from &segKCSchema..FSK_INT_XFER
                                        where edge_type_code in ('R', 'U', 'O')
                                union
                                    select beneficiary_account_number as account_number
                                        from &segKCSchema..FSK_INT_XFER
                                        where edge_type_code in ('R', 'U', 'I')
                            )
                         ) a
                    join &segKCSchema..fsc_account_dim ad on ad.account_number = a.account_number
                    join &segKCSchema..fsc_party_account_bridge pab on pab.account_key = ad.account_key
                    join &segKCSchema..fsc_party_dim pd on pd.party_key = pab.party_key
                    where ad.change_current_ind = 'Y' and
                          pab.role_key = 1 and
                          pd.change_current_ind = 'Y'
        ) by &dbflavor;

        %fcf_end_pass_through(dbflavor=&dbflavor, commit=yes);
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FT: Error getting current names of primary owners of internal accounts for extracted transactions;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*----------------------------------------------------------------------
     * For all internal accounts in FSK_INT_XFER add in the name of the
     * current primary owner.
     *----------------------------------------------------------------------*/
    proc sql;
        %fcf_begin_pass_through(dbflavor=&dbflavor, authenticationDomain=&segkcauthdomain, connOpts=&segKCDBConnOpts);

        execute (
            update &segKCSchema..FSK_INT_XFER
                set remitter_name = ( select TEMP_CHAR_2_VALUE from &segKCSchema..FSK_TEMP_TABLE where remitter_account_number = TEMP_CHAR_1_VALUE )
                where edge_type_code in ('R', 'U', 'O')
        ) by &dbflavor;

        %fcf_end_pass_through(dbflavor=&dbflavor, commit=yes);
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FT: Error updating FSK_INT_XFER to add remitter party names;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc sql;
        %fcf_begin_pass_through(dbflavor=&dbflavor, authenticationDomain=&segkcauthdomain, connOpts=&segKCDBConnOpts);

        execute (
            update &segKCSchema..FSK_INT_XFER
                set beneficiary_name = ( select TEMP_CHAR_2_VALUE from &segKCSchema..FSK_TEMP_TABLE where beneficiary_account_number = TEMP_CHAR_1_VALUE )
                where edge_type_code in ('R', 'U', 'I')
        ) by &dbflavor;

        %fcf_end_pass_through(dbflavor=&dbflavor, commit=yes);
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FT: Error updating FSK_INT_XFER to add beneficiary party names;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*-----------------------------------------------------------------------------------
     * Extract current names of all external party/account entities
     * referenced by transactions loaded to FSK_INT_XFER
     *-----------------------------------------------------------------------------------*/
    proc sql;
        %fcf_begin_pass_through(dbflavor=&dbflavor, authenticationDomain=&segkcauthdomain, connOpts=&segKCDBConnOpts);
        execute (delete from &segKCSchema..FSK_TEMP_TABLE) by &dbflavor;
        %fcf_end_pass_through(dbflavor=&dbflavor, commit=yes);
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FT: Error deleting content of FSK_TEMP_TABLE;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc sql;
        %fcf_begin_pass_through(dbflavor=&dbflavor, authenticationDomain=&segkcauthdomain, connOpts=&segKCDBConnOpts);

        execute ( /* For safety, just in case there are any account keys of -1 present, although that would be bad data */
            insert into &segKCSchema..FSK_TEMP_TABLE (TEMP_CHAR_1_VALUE,TEMP_CHAR_2_VALUE) values( 'UNKNOWN', 'UNKNOWN')
        ) by &dbflavor;

        execute (
            insert into &segKCSchema..FSK_TEMP_TABLE (TEMP_CHAR_1_VALUE,TEMP_CHAR_2_VALUE)
                select a.account_number, ext.full_name
                    from (
                        select distinct account_number
                            from (
                                    select remitter_account_number as account_number
                                        from &segKCSchema..FSK_INT_XFER
                                        where edge_type_code = 'I'
                                union
                                    select beneficiary_account_number as account_number
                                        from &segKCSchema..FSK_INT_XFER
                                        where edge_type_code = 'O'
                            )
                         ) a
                    join &segKCSchema..fsc_ext_party_account_dim ext on ext.external_party_number = a.account_number
                    where a.account_number &notEqual. 'UNKNOWN' and ext.change_current_ind = 'Y'
        ) by &dbflavor;

        %fcf_end_pass_through(dbflavor=&dbflavor, commit=yes);
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FT: Error getting current names of external party/accounts for extracted transactions;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*----------------------------------------------------------------------
     * For all external entities in FSK_INT_XFER add in the current name of
     * the external entity.
     *----------------------------------------------------------------------*/
    proc sql;
        %fcf_begin_pass_through(dbflavor=&dbflavor, authenticationDomain=&segkcauthdomain, connOpts=&segKCDBConnOpts);

        execute (
            update &segKCSchema..FSK_INT_XFER
                set remitter_name = ( select TEMP_CHAR_2_VALUE from &segKCSchema..FSK_TEMP_TABLE where remitter_account_number = TEMP_CHAR_1_VALUE )
                where edge_type_code = 'I'
        ) by &dbflavor;

        %fcf_end_pass_through(dbflavor=&dbflavor, commit=yes);
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FT: Error updating FSK_INT_XFER to add external remitter names;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc sql;
        %fcf_begin_pass_through(dbflavor=&dbflavor, authenticationDomain=&segkcauthdomain, connOpts=&segKCDBConnOpts);

        execute (
            update &segKCSchema..FSK_INT_XFER
                set beneficiary_name = ( select TEMP_CHAR_2_VALUE from &segKCSchema..FSK_TEMP_TABLE where beneficiary_account_number = TEMP_CHAR_1_VALUE )
                where edge_type_code = 'O'
        ) by &dbflavor;

        %fcf_end_pass_through(dbflavor=&dbflavor, commit=yes);
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FT: Error updating FSK_INT_XFER to add external beneficiary names;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    %MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else %do;
        %fcf_execute_macro(post&_fcfmodule_);
        %fcf_execute_macro(fcf_success);
    %end;

%mend;
