/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_load_account_analysis.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    Program to update the core FSC_ACCOUNT_ANALYSIS table.
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/
 
%macro fcf_load_account_analysis ;

  /*------------
   * Set Module
   *------------*/
  %local _fcfmodule_;
  %let _fcfmodule_=FCF_LOAD_ACCOUNT_ANALYSIS;

  /*-----------------------------
   * Set start time for jobstats
   *-----------------------------*/
  %fcf_section_header(section=&_fcfmodule_, message_format=verbose);

  /*---------------------
   * Call pre stub macro
   *---------------------*/
  %fcf_execute_macro(pre&_fcfmodule_);

  /*------------------------------------------
   * Set error code to 0 and message to blank
   *------------------------------------------*/
  %let trans_rc = 0;
  %let _fcfmsg_=;

  %let dimension=ACCOUNT_ANALYSIS;
  %let nkey=account_number;

  /*------------------------------
   * Make sure runasofdate is set
   *------------------------------*/
  %if not %symexist(runasofdate) %then %do;
     %fcf_get_runasofdate;
     %let _fcfmsg_=NOTE: FCF: Runasofdate: &runasofdate;
     %put &_fcfmsg_;
  %end;

  /* Check for existence of table */
  %if not %sysfunc(exist(STG_DIM.FSC_&dimension._STAGE)) %then %do;
     %let _fcfmsg_=ERROR: FCF: STG_DIM.FSC_&dimension._STAGE does not exist or is not accessible.;
     %let trans_rc=999999;
     %fcf_put_error;
     %goto MACRO_END;
  %end;

  /* Check for observation in table */
  %fcf_get_observation_count(table=STG_DIM.FSC_&dimension._STAGE);
  %if &fcf_obs_count eq 0 %then %do;
     %let _fcfmsg_=NOTE: FCF: No records to process. &_FCFMODULE_ is being skipped. ;
     %fcf_put_error;
     %goto MACRO_END;
  %end;


  proc sql;
    connect to &dbflavor. as &dbflavor. (user=&coreDBUser.
                                         pass=&coreDBUserPwd.
                                         &coreDBConnOpts.);
      %if %upcase(&dbflavor) ne ORACLE   and
          %upcase(&dbflavor) ne POSTGRES and
          %upcase(&dbflavor) ne TERADATA %then %do;
            execute (begin transaction) by &dbflavor;
      %end;
            execute (delete from &coreSchema..FSC_account_analysis_dim) by &dbflavor.;
            execute (commit) by &dbflavor.;
        disconnect from &dbflavor;
  quit;

  %fcf_rcset(&sqlrc.);
  %if &trans_rc. eq 3 or &trans_rc. ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: &module.: Unable to delete records from &seg_kc_libname..FSC_&dimension._dim.;
     %fcf_put_error;
     %goto MACRO_END;
  %end;

  /* Sort staged records by natural key */
  proc sort data=STG_DIM.FSC_&dimension._STAGE(%fcf_get_ds_opts(lib_name=STG_DIM, ds_name=FSC_&dimension._STAGE))
             out=STG_DIM.&dimension._NEW(%fcf_get_ds_opts(lib_name=STG_DIM, ds_name=&dimension._NEW, output_ds=Y))
          dupout=STG_ERR.FSC_&dimension._DUPKEY(%fcf_get_ds_opts(lib_name=STG_ERR, ds_name=FSC_&dimension._DUPKEY, output_ds=Y))
       nodupkey sortsize=max noequals;
    by &nkey;
   run;

  %fcf_rcset(&syserr);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: Cannot sort table STG_DIM.%upcase(&dimension)_STAGE.;
     %fcf_put_error;
     %goto MACRO_END;
  %end;

  %if %sysfunc(exist(STG_ERR.FSC_&dimension._DUPKEY)) %then %do;
     %fcf_get_observation_count(table=STG_ERR.FSC_&dimension._DUPKEY);

     %if &fcf_obs_count gt 0 %then %do;
        /* Write the error file in CSV format */
        proc export data=STG_ERR.FSC_&dimension._DUPKEY(%fcf_get_ds_opts(lib_name=STG_ERR, ds_name=FSC_&dimension._DUPKEY))
             outfile="&FCFDATA/stage/error/fsc_%lowcase(&dimension)_dupkey_%qcmpres(&runasofdate).csv"
             dbms=csv
             replace;
         run;

         /* Send e-mail with information about data errors */
         %let _fcfmsg_=%str(ERROR: FCF: FSC_%upcase(&dimension)_DUPKEY contains duplicate keys.);
         %fcf_send_mail(to=%bquote(%trim(&SYSADMIN)),
                        cc=,
                        from=,
                        subject=%quote(&_fcfmsg_),
                        attachment=&FCFDATA/stage/error/fsc_%lowcase(&dimension)_dupkey_%qcmpres(&runasofdate).csv,
                        textBody= %quote(&_fcfmsg_),
                        footer=
                       );
     %end;
  %end;

  proc append base=&seg_kc_libname..FSC_&dimension._DIM(%fcf_get_ds_opts(lib_name=&seg_kc_libname, ds_name=FSC_&dimension._DIM, output_ds=Y))
              data=STG_DIM.&dimension._NEW(%fcf_get_ds_opts(lib_name=STG_DIM, ds_name=&dimension._NEW))
              force;
   run;

  %fcf_rcset(&syserr);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: Cannot append STG_DIM.%upcase(&dimension) to &seg_kc_libname..FSC_%upcase(&dimension)_DIM;
     %fcf_put_error;
     %goto MACRO_END;
  %end;


  %MACRO_END:

   /* Set end time for jobstats */
   %fcf_section_footer(section=&_fcfmodule_, message_format=verbose);

   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %fcf_execute_macro(fcf_error);
   %else %do;
      %fcf_execute_macro(post&_fcfmodule_);
      %fcf_execute_macro(fcf_success);
   %end;

%mend fcf_load_account_analysis;
