/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_prop_to_msg_ds.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    Creates a SAS message data set from the given properties file.
+------------------------------------------------------------------------------+
| USAGE:   This macro should only be executed from within a SAS UTF-8 session.
+------------------------------------------------------------------------------+
| PARAMETERS:
|     file
|         name of the ref file, all files matching pattern: <file>_<locale>.ref or <file>.ref
|         will be processed
|          
|     file_path
|         folder where the ref files are located
|
|     DS - name of the message data set that will be generated, see notes below
|
+------------------------------------------------------------------------------+
| COMMENTS:
|
|     The key in the properties file must have the form:
|         <REF_TABLE_NM>.<VALUE_CD>[.<DISPLAY_ORDER_NUMBER>.<PARENT_REF_TABLE_NM>.<PARENT_VALUE_CD>]
|     The last three components are optional and used to specify display order
|     and link values to a parent table. All components of the record are case 
|     sensitive and may have spaces.
|
|     Description of a SAS message data set created by this macro
|     # Variable Name Type Length Description
|     1 locale       char    5    language of the message
|     2 key          char    90   key to identify the message, the entire key, converted to upper case, spaces replaced with underscore
|     3 lineno       num     5    line # of the message in reverse order
|     4 text         char    1200 text of the message
|     5 key_orig     char    90   the entire key as it is read from the properties file.
|
|     The data set is sorted on the following variables: locale, key, and lineno.
|     The variable lineno is in descending order.
|     A composite index on locale and key is defined.
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

 %macro fcf_prop_to_msg_ds(file=, file_path=, DS=);
 	data file_list;
 	 length locale $5;
 	 length fref $8 filename $256;
 	 keep filename locale;

 	 length u_idx  d_idx 8;
 	 rc = filename(fref, "&file_path");
 	 if rc = 0 then do;
 		 did = dopen(fref);
 		 rc = filename(fref);
 	 end;
 	 else do;
 	 	length msg $200.;
 	 	msg = sysmsg();
 	 	put msg=;
 	 	did = .;
 	 end;

 	 if did <= 0  then putlog 'ERR' 'OR: Unable to open directory.';
     else do;
         dnum = dnum(did);
         do i = 1 to dnum;
            filename = dread(did, i);
            /* If this entry is a file, then process */
            fid = mopen(did, filename);
            if fid > 0 then do;
                if index(UPCASE(filename), UPCASE("&file")) ne 0 then do;
                  u_idx=index(filename, '_');
                  d_idx=index(filename, '.');
                  if u_idx = 0 then locale="en";
                            else locale=substr(filename,u_idx+1, d_idx-u_idx-1);
                  filename="&file_path/" || filename;
                  output;
                end;
            end;
         end;
         rc = dclose(did);
     end;
 	run;

        data &ds (%fcf_get_ds_opts(ds_name=&ds) ENCODING=ASCII);
 		set file_list;
 	          length key $90;
 	          length lineno 8;
 	          length text $1200;
 	          length key_orig $90;
 		keep locale key lineno text key_orig;

 		infile inf filevar=filename
 			   filename=myfile
 			   end=done;
 		do while (not done);
 		    input;
 	        if (_infile_ ne ' ') then do;
 	            if (ksubstr(_infile_,1,1) ne '#') then do;
 	                    key_orig=kscan(_infile_,1,'=');
 	                    key=upcase(transtrn(trim(key_orig)," ","_"));
 	                    key_separator_indx=kindex(_infile_, '=');
 	     			    lineno=1;
 	                    text=ksubstr(_infile_,key_separator_indx+1);
 	                    output;
 	             end;
 	        end;
 		end;
 	run;

        proc sort data=&DS(%fcf_get_ds_opts(ds_name=&DS)) FORCE;
 	       BY locale key DESCENDING lineno;
 	run;
 	proc sql;
             create index indx on &DS (%fcf_get_ds_opts(ds_name=&DS, append_comma=Y) locale, key);
 	quit;
  %mend;

