/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_purge.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    Main program to delete AML data.  This includes age-based data
|          such as case, report and efile data.  It also includes
|          customer-specified data where the customer specifies parties to
|          purge and the job purges them and related data.
+------------------------------------------------------------------------------+
| USAGE:
|   %fcf_purge(
|              file_of_parties_to_purge=~/customers_to_delete.txt,
|              file_of_data_sets_to_purge_from=~/sas_ds_to_purge_from.txt,
|              case_status_closed=C R,
|              test_run=Y,
|              purge_log_dir=~/purge_logs,
|              purge_key_dir=~/purge_keys,
|              purge_from_database=Y,
|              purge_from_data_sets=Y
|             )
+------------------------------------------------------------------------------+
| COMMENTS:
|   db_lib_core= SAS library which gives direct access to Core.
|      The job will assign the library if the parameter is not set.
|   db_lib_kc=   SAS library which gives direct access to KC.
|      The job will assign the library if the parameter is not set.
|   file_of_parties_to_purge= Path/file name containing the party numbers to
|      purge.  The first line in the file contains the text: party_number
|      Subsequent rows contain a customer (party) number to delete.
|      Each party in this file is eligible to be purged as determined
|      by the customer.  The purge job might subsequently determine
|      the entity cannot be purged.
|      If This parameter isn't set then the code will check 
|      FSC_PARTY_DIM.purge_date where the value isn't NULL and if the date
|      is less than or equal to today then the party is eligible to be purged.
|      Again, the code might subsequently determine the entity cannot be
|      purged.
|   file_of_data_sets_to_purge_from= Specifies path/file name
|      containing data sets to purge from (for example mst_prep data sets)
|      including information about them such as the SAS libname, data set
|      name, entity type, key, whether the key is numeric or character.
|   purge_log_data_set_name= data set used to log which records were
|      purged/updated by this job.  If this parameter is not set this
|      naming convention will be used: aml_purge_log_yyyymmdd_hhmmss 
|      where yyyymmdd_hhmmss is the time the purge job started,
|      for example: aml_purge_log_20190821_161749
|   purge_log_dir= Directory in which purge log data sets will be created.
|      If this parameter is not set the value stored in the preference
|      amlPurgeLogDirectory will be used.
|   efile_status_closed= Status values which signify an EFile is closed.
|      The default values are:  A AW AE
|   report_status_closed= Status values which signify a Report is closed.
|      The default values are: A AW AE C
|   case_status_closed= Status values which signify a Case is closed.  If
|      this parameter is set it will override the status values in the
|      preference caseStatusClosed.
|   test_run= Must be Y or N.  If set to N then data eligible to be purged
|      will be purged.  If set to anything else then data eligible to be
|      purged will not be purged but will allow the user to see what would
|      have been purged from the database and data sets in the purge log
|      data set.  Default is: Y
|   save_keys= Must be Y or N.  If set to Y then keys which this job uses
|      to purge data will be saved.  This can be useful for reasons such as
|      restarting the job (see purge_restart) or simply viewing the keys
|      which the job used to purge, by category (party, account, household,
|      case, report, efile, etc.).  Default is: Y
|      Hint: Save your keys if you think you might need to use purge_restart!
|   purge_key_dir= Directory in which keys will be saved. (see save_keys)
|      If save_keys is Y and purge_key_dir is not set, then the job will get
|      the value for the preference amlPurgeKeyDirectory and use that as the
|      directory in which to save the keys.
|   purge_from_database= Must be Y or N.  If set to Y then the part of this
|      job which deletes from the Core and KC databases will be run.
|      Default is: Y
|   purge_from_data_sets= Must be Y or N.  If set to Y then the part of this
|      job which deletes from data sets such as master prep data sets
|      (mst_prep library) will be run.  Default is: Y
|      Setting this parameter to N might be useful in order to run the
|      database purge by iteself to make sure it runs successfully, also
|      saving the keys.  Once it's determined that part ran successfully,
|      restart the job with purge_restart=Y and purge_from_database=N and
|      purge_from_data_sets=Y to purge from the data sets.
|   purge_from_solr= Must be Y or N.  If set to Y then the part of this
|      job which deletes from Solr will be run.  Default is: Y
|   cert_file= For use with Solr deletes
|   curl_command= For use with Solr deletes
|   purge_restart= Must be Y or N.  If set to Y and purge_key_dir is set then
|      the job will look for saved key data sets and if found will use them.
|      Default is:  N
|      Setting this purge_restart to Y might be useful to purge from the
|      database in one run of the job (save the keys also!) followed by
|      purging from the data sets in a subsequent run.  It might also be
|      useful if the job terminates unexpectedly.  If you think you might
|      need to use this parameter then always save your keys!
|   disable_ui= Must be Y or N.  If set to Y then the UI is made unavailable
|      to users while the purge job runs.  If the UI is already unavailable
|      then the purge job will stop as two jobs running at the same time
|      which want to disable the UI is probably not a good idea.
|      Default is:  N
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2019 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_purge(
                 db_lib_core=,
                 db_lib_kc=,
                 file_of_parties_to_purge=,
                 file_of_data_sets_to_purge_from=,
                 purge_log_data_set_name=,
                 purge_log_dir=,
                 efile_status_closed=A AW AE,
                 report_status_closed=A AW AE C,
                 case_status_closed=,
                 test_run=Y,
                 save_keys=Y,
                 purge_key_dir=,
                 purge_from_database=Y,
                 purge_from_data_sets=Y,
                 purge_from_solr=Y,
                 cert_file=,
                 curl_command=,
                 purge_restart=N,
                 disable_ui=N
                );

  %global slash;

  %if &sysscp = WIN %then %do;
     %let slash = %quote(\);
  %end;
  %else %do;
     %let slash = %quote(/);
  %end;

  /*----------------------------------------------------------------
   * Sets up SAS regulatory reporting.  Needed for deleting EFiles.
   * Sets the variable RCDATA_PATH etc.
   *----------------------------------------------------------------*/
  %include "$FCFROOT&slash.custom&slash.config&slash.rc_autoexec.sas";

  /*------------
   * Set Module
   *------------*/
  %local _fcfmodule_;
  %let _fcfmodule_=FCF_PURGE;

  /*-------------------------------
   * Call prefcf_purge stub macro.
   *-------------------------------*/
  %fcf_execute_macro(pre&_fcfmodule_);

  /*-------------------------------------------
   * Set error code to 0 and message to blank.
   *-------------------------------------------*/
  %let trans_rc = 0;
  %let _fcfmsg_=;

  /*---------------------------------------------------------
   * Setup, error checking, etc. before actual purge of data
   *---------------------------------------------------------*/
  %fcf_purge_setup;

  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  /*-----------------
   * Begin the purge
   *-----------------*/

  %if %upcase(&purge_from_database) eq N %then
  %do;
     %goto END_PURGE_FROM_DATABASE;
  %end;

  /*-------------------------------
   * Begin purge from the database
   *-------------------------------*/

  /*-----------------------------------------------------------------
   * Purge from Case, Report, EFile tables based on retention days.
   * This must come before the other purges as they can be dependent
   * on what is deleted by age.
   *-----------------------------------------------------------------*/
  %fcf_purge_by_age;

  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  /*---------------------
   * Purge from the Core
   *---------------------*/
  %fcf_purge_from_core;

  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  /*-------------------
   * Purge from the KC
   *-------------------*/
  %fcf_purge_from_kc;

  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  %END_PURGE_FROM_DATABASE:

  /*-----------------------------
   * End purge from the database
   *-----------------------------*/

  %if %upcase(&purge_from_data_sets) eq N %then
  %do;
     %goto END_PURGE_FROM_DATA_SETS;
  %end;

  /*----------------------
   * Purge from data sets
   *----------------------*/
  %fcf_purge_from_ds;

  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  %END_PURGE_FROM_DATA_SETS:


  %if %upcase(&purge_from_solr) eq N or
      %upcase(&test_run) eq Y %then
  %do;
     %goto END_PURGE_FROM_SOLR;
  %end;

  /*-----------------
   * Purge from SOLR
   *-----------------*/
  %fcf_purge_from_solr;

  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  %END_PURGE_FROM_SOLR:


  %MACRO_END:

  %if &ui_availability_set_by_this_job eq Y %then
  %do;
     /*--------------------------------
      * Make the UI available to users
      *--------------------------------*/
     %fcf_set_ui_availability(availability_ind=Y);
  %end;

  %if &db_lib_core_assigned_by_main eq 1 %then
  %do;
     libname &db_lib_core;
  %end;

  %if &db_lib_kc_assigned_by_main eq 1 %then
  %do;
     libname &db_lib_kc;
  %end;

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
     %fcf_execute_macro(fcf_error);
  %else %do;
     %fcf_execute_macro(post&_fcfmodule_);
     %fcf_execute_macro(fcf_success);
  %end;

%mend fcf_purge;

