/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_purge_add_to_log_ds.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    Add purge information to the purge log data set
+------------------------------------------------------------------------------+
| USAGE:
|  %fcf_purge_add_to_log_ds(in_ds=rc_efile_purged,
|                           libname=&db_lib_kc,
|                           table=fsk_rc_efile,
|                           key_1=efile_key,
|                           key_1_type=N,
|                           key_2=efile_id,
|                           key_2_type=C,
|                           key_3=form_config_key,
|                           key_3_type=N); 
+------------------------------------------------------------------------------+
| COMMENTS: 
|   rows_updated= Set to N when data updated instead of purged.
|   in_ds= Data set containing information about purged/updated data.
|   delete_in_ds_when_done= If set to Y, the in_ds will be deleted at the end
|      of this macro.
|   libname= libname of database table or data set purged from
|   table= name of database table or data set purged from
|   rolled_up= is in_ds already rolled up and totals calculated?
|   file_only= Just file deleted, and no records (db or data set)?
|   key_1= variable 1 to output from in_ds to purge log data set
|   key_1_type= variable 1 type (N for numeric, C for character)
|   key_2= variable 2 to output from in_ds to purge log data set
|   key_2_type= variable 2 type (N for numeric, C for character)
|   key_3= variable 3 to output from in_ds to purge log data set
|   key_3_type= variable 3 type (N for numeric, C for character)
|   key_4= variable 4 to output from in_ds to purge log data set
|   key_4_type= variable 4 type (N for numeric, C for character)
|   key_5= variable 5 to output from in_ds to purge log data set
|   key_5_type= variable 5 type (N for numeric, C for character)
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2019 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_purge_add_to_log_ds(
                               rows_updated=N,
                               in_ds=,
                               delete_in_ds_when_done=Y,
                               libname=,
                               table=,
                               rolled_up=N,
                               file_only=N,
                               key_1=,
                               key_1_type=,
                               key_2=,
                               key_2_type=,
                               key_3=,
                               key_3_type=,
                               key_4=,
                               key_4_type=,
                               key_5=,
                               key_5_type=,
                               key_6=,
                               key_6_type=
                              );

  /*------------
   * Set Module
   *------------*/
  %local _fcfmodule_;
  %let _fcfmodule_=FCF_PURGE_ADD_TO_LOG_DS;

  /*---------------------------------------------
   * Call prefcf_purge_add_to_log_ds stub macro.
   *---------------------------------------------*/
  %fcf_execute_macro(pre&_fcfmodule_);

  /*-------------------------------------------
   * Set error code to 0 and message to blank.
   *-------------------------------------------*/
  %let trans_rc = 0;
  %let _fcfmsg_=;

  data add_to_purge_log(keep=user test_run libname table
                             records_purged records_updated
                             record_purged_by_key_counter record_updated_by_key_counter
                             key_1 key_1_type key_1_numeric_value key_1_character_value
                             key_2 key_2_type key_2_numeric_value key_2_character_value
                             key_3 key_3_type key_3_numeric_value key_3_character_value
                             key_4 key_4_type key_4_numeric_value key_4_character_value
                             key_5 key_5_type key_5_numeric_value key_5_character_value
                             key_6 key_6_type key_6_numeric_value key_6_character_value
                       );
   length user $200 test_run $1 libname table $32;
   length records_purged
          records_updated
          record_purged_by_key_counter
          record_updated_by_key_counter 8;
   length key_1 $32 key_1_type $1;
   length key_1_numeric_value 8;
   length key_1_character_value $4000;
   length key_2 $32 key_2_type $1;
   length key_2_numeric_value 8;
   length key_2_character_value $4000;
   length key_3 $32 key_3_type $1;
   length key_3_numeric_value 8;
   length key_3_character_value $4000;
   length key_4 $32 key_4_type $1;
   length key_4_numeric_value 8;
   length key_4_character_value $4000;
   length key_5 $32 key_5_type $1;
   length key_5_numeric_value 8;
   length key_5_character_value $4000;
   length key_6 $32 key_6_type $1;
   length key_6_numeric_value 8;
   length key_6_character_value $4000;
   set &in_ds;
    user = "&sysuserid";
    test_run = "&test_run";
    table = "&table";
    libname = "&libname";
    %if &file_only eq Y %then
    %do;
       records_purged = 0;
       records_updated = 0;
       record_purged_by_key_counter = .;
       record_updated_by_key_counter = .;
    %end;
    %else
    %do;
       %if &rolled_up eq Y %then
       %do;
          %if &rows_updated eq N %then
          %do;
             records_purged = rec_purged;
             records_updated = 0;
             record_purged_by_key_counter = .;
             record_updated_by_key_counter = .;
          %end;
          %else
          %do;
             records_purged = 0;
             records_updated = rec_updated;        /* should be no cases of this yet */
             record_purged_by_key_counter = .;
             record_updated_by_key_counter = .;
          %end;
       %end;
       %else
       %do;
          %if &rows_updated eq N %then
          %do;
             records_purged = 1;
             records_updated = 0;
             record_purged_by_key_counter = rec_purged_by_key_counter;
             record_updated_by_key_counter = 0;
          %end;
          %else
          %do;
             records_purged = 0;
             records_updated = 1;
             record_purged_by_key_counter = 0;
             record_updated_by_key_counter = rec_updated_by_key_counter;
          %end;
       %end;
    %end;

    key_1 = "&key_1";
    key_1_type = "&key_1_type";
    %if &key_1_type eq N %then
    %do;
       key_1_numeric_value = &key_1;
       key_1_character_value = '';
    %end;
    %else
    %if &key_1_type eq C %then
    %do;
       key_1_character_value = &key_1;
       key_1_numeric_value = .;
    %end;
    %else
    %do;
       key_1_character_value = '';
       key_1_numeric_value = .;
    %end;

    key_2 = "&key_2";
    key_2_type = "&key_2_type";
    %if &key_2_type eq N %then
    %do;
       key_2_numeric_value = &key_2;
       key_2_character_value = '';
    %end;
    %else
    %if &key_2_type eq C %then
    %do;
       key_2_character_value = &key_2;
       key_2_numeric_value = .;
    %end;
    %else
    %do;
       key_2_character_value = '';
       key_2_numeric_value = .;
    %end;

    key_3 = "&key_3";
    key_3_type = "&key_3_type";
    %if &key_3_type eq N %then
    %do;
       key_3_numeric_value = &key_3;
       key_3_character_value = '';
    %end;
    %else
    %if &key_3_type eq C %then
    %do;
       key_3_character_value = &key_3;
       key_3_numeric_value = .;
    %end;
    %else
    %do;
       key_3_character_value = '';
       key_3_numeric_value = .;
    %end;

    key_4 = "&key_4";
    key_4_type = "&key_4_type";
    %if &key_4_type eq N %then
    %do;
       key_4_numeric_value = &key_4;
       key_4_character_value = '';
    %end;
    %else
    %if &key_4_type eq C %then
    %do;
       key_4_character_value = &key_4;
       key_4_numeric_value = .;
    %end;
    %else
    %do;
       key_4_character_value = '';
       key_4_numeric_value = .;
    %end;

    key_5 = "&key_5";
    key_5_type = "&key_5_type";
    %if &key_5_type eq N %then
    %do;
       key_5_numeric_value = &key_5;
       key_5_character_value = '';
    %end;
    %else
    %if &key_5_type eq C %then
    %do;
       key_5_character_value = &key_5;
       key_5_numeric_value = .;
    %end;
    %else
    %do;
       key_5_character_value = '';
       key_5_numeric_value = .;
    %end;

    key_6 = "&key_6";
    key_6_type = "&key_6_type";
    %if &key_6_type eq N %then
    %do;
       key_6_numeric_value = &key_6;
       key_6_character_value = '';
    %end;
    %else
    %if &key_6_type eq C %then
    %do;
       key_6_character_value = &key_6;
       key_6_numeric_value = .;
    %end;
    %else
    %do;
       key_6_character_value = '';
       key_6_numeric_value = .;
    %end;
   run;

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  proc append base=purgelog.&purge_log_data_set_name(%fcf_get_ds_opts(lib_name=purgelog, ds_name=&purge_log_data_set_name, output_ds=Y))
              data=add_to_purge_log;
   run;

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  %if &delete_in_ds_when_done eq Y %then
  %do;
     proc datasets lib=work nowarn nolist;
       delete &in_ds;
     quit;
  %end;


  %MACRO_END:

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
     %fcf_execute_macro(fcf_error);
  %else %do;
     %fcf_execute_macro(post&_fcfmodule_);
     %fcf_execute_macro(fcf_success);
  %end;

%mend fcf_purge_add_to_log_ds;

