/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_purge_by_age.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    Delete data by age (retention days)
+------------------------------------------------------------------------------+
| USAGE: %fcf_purge_by_age
+------------------------------------------------------------------------------+
| COMMENTS: 
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2019 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_purge_by_age;

  /*------------
   * Set Module
   *------------*/
  %local _fcfmodule_;
  %let _fcfmodule_=FCF_PURGE_BY_AGE;

  /*--------------------------------------
   * Call prefcf_purge_by_age stub macro.
   *--------------------------------------*/
  %fcf_execute_macro(pre&_fcfmodule_);

  /*-------------------------------------------
   * Set error code to 0 and message to blank.
   *-------------------------------------------*/
  %let trans_rc = 0;
  %let _fcfmsg_=;

  /*-------------------------------------------------------------------------
   * For all records in FSK_RC_EFILE which have an efile_status_code of
   * closed (A, AW, AE) if the lstupdate_date plus the EFile retention days
   * is less than or equal to today then purge any associated bridge records
   * from FSK_RC_EFILE_REPORT, associated records from FSK_RC_EFILE_EVENT
   * and then the record from FSK_RC_EFILE.  
   *-------------------------------------------------------------------------*/
  %if %upcase(&purge_restart) eq Y and
      %sysfunc(exist(purgekey.efl_to_delete)) %then
  %do;
     data efl_to_delete;
      set purgekey.efl_to_delete(%fcf_get_ds_opts(lib_name=purgekey, ds_name=efl_to_delete));
      run;

     %goto BEGIN_EFILE_PURGE;
  %end;

  data efl_to_delete(keep=efile_key efile_key_char);
    if 0 then set &db_lib_kc..fsk_comment(keep=object_id rename=(object_id = efile_key_char));
    label efile_key_char = "efile_key_char";
    set &db_lib_kc..fsk_rc_efile(keep=efile_key efile_status_code lstupdate_date
                                where=(indexw("&efile_status_closed", efile_status_code) and
                                       datepart(lstupdate_date) + &amlEfileRetentionDays le today()
                                      )
                               );
    efile_key_char = strip(put(efile_key, best.));
   run;

  %fcf_rcset(&syserr);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %let _fcfmsg_=ERROR: FCF: Error reading from &db_lib_kc..FSK_RC_EFILE;
      %fcf_put_error;
      %goto MACRO_END;
  %end;

  %if &purgekey_lib_assigned eq 1 %then
  %do;
     data purgekey.efl_to_delete(%fcf_get_ds_opts(lib_name=purgekey, ds_name=efl_to_delete, output_ds=Y));
      set efl_to_delete;
      run;
  %end;

  %BEGIN_EFILE_PURGE:

  /*--------------------------------
   * Delete from FSK_RC_EFILE_EVENT
   *--------------------------------*/
  %fcf_purge_from_table(table=&db_lib_kc..fsk_rc_efile_event,
                        entities_ds=efl_to_delete,
                        key_vars=efile_key,
                        one_to_many_ind=Y,
                        other_output_ds=rc_efile_event_purged,
                        other_output_ds_vars=efile_key event_id event_type_code);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  %fcf_purge_add_to_log_ds(in_ds=rc_efile_event_purged,
                           %if %upcase(&test_run) eq Y %then
                           %do;
                           delete_in_ds_when_done=N,
                           %end;
                           libname=&db_lib_kc,
                           table=fsk_rc_efile_event,
                           key_1=efile_key,
                           key_1_type=N,
                           key_2=event_id,
                           key_2_type=N,
                           key_3=event_type_code,
                           key_3_type=C);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  /*---------------------------------
   * Delete from FSK_RC_EFILE_REPORT
   *---------------------------------*/
  %fcf_purge_from_table(table=&db_lib_kc..fsk_rc_efile_report,
                        entities_ds=efl_to_delete,
                        key_vars=efile_key,
                        one_to_many_ind=Y,
                        other_output_ds=rc_efile_report_purged,
                        other_output_ds_vars=efile_key report_key);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  %fcf_purge_add_to_log_ds(in_ds=rc_efile_report_purged,
                           %if %upcase(&test_run) eq Y %then
                           %do;
                           delete_in_ds_when_done=N,
                           %end;
                           libname=&db_lib_kc,
                           table=fsk_rc_efile_report,
                           key_1=efile_key,
                           key_1_type=N,
                           key_2=report_key,
                           key_2_type=N);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  /*--------------------------
   * Delete from FSK_RC_EFILE
   *--------------------------*/
  %fcf_purge_from_table(table=&db_lib_kc..fsk_rc_efile,
                        entities_ds=efl_to_delete,
                        key_vars=efile_key,
                        one_to_many_ind=N,
                        other_output_ds=rc_efile_purged,
                        other_output_ds_vars=efile_key efile_id form_config_key output_path_name output_name output_zip_name direct_transfer_efile_name response_file_name);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  /*---------------------------------------------------------------------------
   * Delete files, which are linked to the purged efiles, from the file system
   *---------------------------------------------------------------------------*/
  %fcf_purge_efiles;

  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %goto MACRO_END;
  %end;

  /*--------------------------------------------
   * Now log the purged efile records and files
   *--------------------------------------------*/
  %fcf_purge_add_to_log_ds(in_ds=rc_efile_purged,
                           libname=&db_lib_kc,
                           table=fsk_rc_efile,
                           key_1=efile_key,
                           key_1_type=N,
                           key_2=efile_id,
                           key_2_type=C,
                           key_3=form_config_key,
                           key_3_type=N);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  data reg_console_attachments;
   set reg_console_attachments(rename=(attachment_path=file_name attachment_deleted=file_deleted));
   run;

  %fcf_purge_add_to_log_ds(in_ds=reg_console_attachments,
                           libname=&db_lib_kc,
                           table=fsk_rc_efile,
                           file_only=Y,
                           key_1=efile_key,
                           key_1_type=N,
                           key_2=file_name,
                           key_2_type=C,
                           key_3=file_deleted,
                           key_3_type=C,
                           key_4=reason,
                           key_4_type=C);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  data reg_console_efiles;
   set reg_console_efiles(rename=(attachment_path=file_name attachment_deleted=file_deleted));
   run;

  %fcf_purge_add_to_log_ds(in_ds=reg_console_efiles,
                           libname=&db_lib_kc,
                           table=fsk_rc_efile,
                           file_only=Y,
                           key_1=efile_key,
                           key_1_type=N,
                           key_2=file_name,
                           key_2_type=C,
                           key_3=file_deleted,
                           key_3_type=C,
                           key_4=reason,
                           key_4_type=C);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  data reg_console_FINCEN_SDTM_efiles;
   set reg_console_FINCEN_SDTM_efiles(rename=(attachment_path=file_name attachment_deleted=file_deleted));
   run;

  %fcf_purge_add_to_log_ds(in_ds=reg_console_FINCEN_SDTM_efiles,
                           libname=&db_lib_kc,
                           table=fsk_rc_efile,
                           file_only=Y,
                           key_1=efile_key,
                           key_1_type=N,
                           key_2=file_name,
                           key_2_type=C,
                           key_3=file_deleted,
                           key_3_type=C,
                           key_4=reason,
                           key_4_type=C);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  data reg_console_efiles_responses;
   set reg_console_efiles_responses(rename=(attachment_path=file_name attachment_deleted=file_deleted));
   run;

  %fcf_purge_add_to_log_ds(in_ds=reg_console_efiles_responses,
                           libname=&db_lib_kc,
                           table=fsk_rc_efile,
                           file_only=Y,
                           key_1=efile_key,
                           key_1_type=N,
                           key_2=file_name,
                           key_2_type=C,
                           key_3=file_deleted,
                           key_3_type=C,
                           key_4=reason,
                           key_4_type=C);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  data FINCEN_SDTM_efiles_responses;
   set FINCEN_SDTM_efiles_responses(rename=(attachment_path=file_name attachment_deleted=file_deleted));
   run;

  %fcf_purge_add_to_log_ds(in_ds=FINCEN_SDTM_efiles_responses,
                           libname=&db_lib_kc,
                           table=fsk_rc_efile,
                           file_only=Y,
                           key_1=efile_key,
                           key_1_type=N,
                           key_2=file_name,
                           key_2_type=C,
                           key_3=file_deleted,
                           key_3_type=C,
                           key_4=reason,
                           key_4_type=C);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  /*---------------------------------------------------------------------------------------------
   * For all records in FSK_RC_REPORT which have an report_status_code of closed (A, AW, AE, C),
   * if the lstupdate_date plus the retention days is less than or equal to today, *and* there
   * are no associated bridge records in FSK_RC_EFILE_REPORT (no linked efiles), then purge:
   * the associated record(s) from FSK_RC_REPORT_ENTITY_EVENT
   * the associated record(s) from FSK_RC_REPORT_ENTITY_BRIDGE
   * the associated record(s) from FSK_RC_REPORT_ENTITY
   * the associated record(s) from FSK_RC_REPORT_EVENT
   * the record from FSK_RC_REPORT.
   *---------------------------------------------------------------------------------------------*/
  %if %upcase(&purge_restart) eq Y and
      %sysfunc(exist(purgekey.rpt_to_delete)) %then
  %do;
     data rpt_to_delete;
      set purgekey.rpt_to_delete(%fcf_get_ds_opts(lib_name=purgekey, ds_name=rpt_to_delete));
      run;

     %goto BEGIN_REPORT_PURGE;
  %end;

  data rpt_to_delete(keep=report_key);
   set &db_lib_kc..fsk_rc_report(keep=report_key report_status_code lstupdate_date
                                where=(indexw("&report_status_closed", report_status_code) and
                                       datepart(lstupdate_date) + &amlReportRetentionDays le today()
                                      )
                                );
   run;

  %fcf_rcset(&syserr);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %let _fcfmsg_=ERROR: FCF: Error reading from &db_lib_kc..FSK_RC_REPORT;
      %fcf_put_error;
      %goto MACRO_END;
  %end;

  data rpt_to_delete_with_efile;
   set rpt_to_delete(keep=report_key);
    do until (_iorc_ ne %sysrc(_sok));
       set &db_lib_kc..fsk_rc_efile_report(keep=report_key efile_key
                                          dbkey=report_key)
                                          key=dbkey;
        if _iorc_ = %sysrc(_sok) then
           output;
        else
        do;
           _error_ = 0;
        end;
    end;
   run;

  %fcf_rcset(&syserr);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %let _fcfmsg_=ERROR: FCF: Error reading from &db_lib_kc..FSK_RC_EFILE_REPORT;
      %fcf_put_error;
      %goto MACRO_END;
  %end;

  %if %upcase(&test_run) eq Y %then
  %do;
     proc sort data=rc_efile_report_purged nodupkey;
       by report_key efile_key;
      run;

     proc sort data=rpt_to_delete_with_efile nodupkey;
       by report_key efile_key;
      run;

     data rpt_to_delete_with_efile;
       merge rpt_to_delete_with_efile(in=a) rc_efile_report_purged(in=b keep=report_key efile_key);
       by report_key efile_key;
       if a and not b;
      run;

     proc datasets lib=work nowarn nolist;
       delete rc_efile_report_purged;
     quit;
  %end;

  proc sort data=rpt_to_delete_with_efile nodupkey;
    by report_key;
   run;

  proc sort data=rpt_to_delete;
    by report_key;
   run;

  data rpt_to_delete(keep=report_key report_key_char);
    if 0 then set &db_lib_kc..fsk_comment(keep=object_id rename=(object_id = report_key_char));
    label report_key_char = "report_key_char";
    merge rpt_to_delete(in=a) rpt_to_delete_with_efile(in=b);
    by report_key;
    if a and not b;
    report_key_char = strip(put(report_key, best.));
   run;

  %fcf_rcset(&syserr);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %let _fcfmsg_=ERROR: FCF: Error creating rpt_to_delete;
      %fcf_put_error;
      %goto MACRO_END;
  %end;

  proc datasets lib=work nowarn nolist;
    delete rpt_to_delete_with_efile;
  quit;

  %if &purgekey_lib_assigned eq 1 %then
  %do;
     data purgekey.rpt_to_delete(%fcf_get_ds_opts(lib_name=purgekey, ds_name=rpt_to_delete, output_ds=Y));
      set rpt_to_delete;
      run;
  %end;

  %BEGIN_REPORT_PURGE:

  /*----------------------------------------
   * Delete from FSK_RC_REPORT_ENTITY_EVENT
   *----------------------------------------*/
  %fcf_purge_from_table(table=&db_lib_kc..fsk_rc_report_entity_event,
                        entities_ds=rpt_to_delete,
                        key_vars=report_key,
                        one_to_many_ind=Y,
                        other_output_ds=rc_report_entity_event_purged,
                        other_output_ds_vars=report_key entity_key entity_level_code event_id);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  %fcf_purge_add_to_log_ds(in_ds=rc_report_entity_event_purged,
                           libname=&db_lib_kc,
                           table=fsk_rc_report_entity_event,
                           key_1=report_key,
                           key_1_type=N,
                           key_2=entity_key,
                           key_2_type=N,
                           key_3=entity_level_code,
                           key_3_type=C,
                           key_4=event_id,
                           key_4_type=N);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  /*-----------------------------------------
   * Delete from FSK_RC_REPORT_ENTITY_BRIDGE
   *-----------------------------------------*/
  %fcf_purge_from_table(table=&db_lib_kc..fsk_rc_report_entity_bridge,
                        entities_ds=rpt_to_delete,
                        key_vars=report_key,
                        one_to_many_ind=Y,
                        other_output_ds=rc_report_entity_bridge_purged,
                        other_output_ds_vars=report_key entity_key entity_level_code);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  %fcf_purge_add_to_log_ds(in_ds=rc_report_entity_bridge_purged,
                           libname=&db_lib_kc,
                           table=fsk_rc_report_entity_bridge,
                           key_1=report_key,
                           key_1_type=N,
                           key_2=entity_key,
                           key_2_type=N,
                           key_3=entity_level_code,
                           key_3_type=C);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  /*----------------------------------
   * Delete from FSK_RC_REPORT_ENTITY
   *----------------------------------*/
  %fcf_purge_from_table(table=&db_lib_kc..fsk_rc_report_entity,
                        entities_ds=rpt_to_delete,
                        key_vars=report_key,
                        one_to_many_ind=Y,
                        other_output_ds=rc_report_entity_purged,
                        other_output_ds_vars=report_key entity_key entity_level_code entity_number);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  %fcf_purge_add_to_log_ds(in_ds=rc_report_entity_purged,
                           %if %upcase(&test_run) eq Y %then
                           %do;
                           delete_in_ds_when_done=N,
                           %end;
                           libname=&db_lib_kc,
                           table=fsk_rc_report_entity,
                           key_1=report_key,
                           key_1_type=N,
                           key_2=entity_key,
                           key_2_type=N,
                           key_3=entity_number,
                           key_3_type=C,
                           key_4=entity_level_code,
                           key_4_type=C);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  /*---------------------------------
   * Delete from FSK_RC_REPORT_EVENT
   *---------------------------------*/
  %fcf_purge_from_table(table=&db_lib_kc..fsk_rc_report_event,
                        entities_ds=rpt_to_delete,
                        key_vars=report_key,
                        one_to_many_ind=Y,
                        other_output_ds=rc_report_event_purged,
                        other_output_ds_vars=report_key event_id event_type_code);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  %fcf_purge_add_to_log_ds(in_ds=rc_report_event_purged,
                           libname=&db_lib_kc,
                           table=fsk_rc_report_event,
                           key_1=report_key,
                           key_1_type=N,
                           key_2=event_id,
                           key_2_type=N,
                           key_3=event_type_code,
                           key_3_type=C);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  /*---------------------------
   * Delete from FSK_RC_REPORT
   *---------------------------*/
  %fcf_purge_from_table(table=&db_lib_kc..fsk_rc_report,
                        entities_ds=rpt_to_delete,
                        key_vars=report_key,
                        one_to_many_ind=N,
                        other_output_ds=rc_report_purged,
                        other_output_ds_vars=report_key form_config_key);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  %fcf_purge_add_to_log_ds(in_ds=rc_report_purged,
                           %if %upcase(&test_run) eq Y %then
                           %do;
                           delete_in_ds_when_done=N,
                           %end;
                           libname=&db_lib_kc,
                           table=fsk_rc_report,
                           key_1=report_key,
                           key_1_type=N,
                           key_2=form_config_key,
                           key_2_type=N);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  /*----------------------------------------------------------------------------------------------------
   * For all records in FSK_CASE which have an case_status_code of closed (we'll look up the preference
   * caseStatusClosed in FSK_PREFERENCE and use whatever its value is and assume itll be a comma or
   * space separated list such as C, R, Y, Z) if the lstupdate_date plus the retention days is less
   * than or equal to today, *and* there are no associated reports in FSK_RC_REPORT
   * (where FSK_RC_REPORT.parent_object_key = FSK_CASE.case_id and
   *        FSK_RC_REPORT.parent_object_name = CASE ), then purge:
   * the associated record(s) from FSK_CASE_ENTITY_EVENT
   * the associated record(s) from FSK_CASE_EVENT
   * the associated record(s) from FSK_CASE_TRANSACTION
   * the associated record(s) from FSK_CASE_ENTITY_TRANSACTION
   * the associated record(s) from FSK_CASE_ENTITY
   * the associated record(s) from FSK_ALERT_CASE
   * the record from FSK_CASE.
   *----------------------------------------------------------------------------------------------------*/
  %if %upcase(&purge_restart) eq Y and
      %sysfunc(exist(purgekey.cas_to_delete)) %then
  %do;
     data cas_to_delete;
      set purgekey.cas_to_delete(%fcf_get_ds_opts(lib_name=purgekey, ds_name=cas_to_delete));
      run;

     %goto BEGIN_CASE_PURGE;
  %end;

  data cas_to_delete(keep=case_id);
   set &db_lib_kc..fsk_case(keep=case_id case_status_code lstupdate_date
                                where=(indexw("&caseStatusClosed", case_status_code, ' ,') and
                                       datepart(lstupdate_date) + &amlCaseRetentionDays le today()
                                      )
                           );
   run;

  %fcf_rcset(&syserr);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %let _fcfmsg_=ERROR: FCF: Error reading from &db_lib_kc..FSK_CASE;
      %fcf_put_error;
      %goto MACRO_END;
  %end;

  data cas_to_delete_with_report(rename=(parent_object_key = case_id));
    if 0 then set &db_lib_kc..fsk_rc_report(keep=parent_object_name);
    set cas_to_delete(keep=case_id rename=(case_id = parent_object_key));
    parent_object_name = 'CASE';
    do until (_iorc_ ne %sysrc(_sok));
       set &db_lib_kc..fsk_rc_report(keep=report_key parent_object_key parent_object_name
                                     dbkey=(parent_object_key parent_object_name))
                                    key=dbkey;
        if _iorc_ = %sysrc(_sok) then
           output;
        else
        do;
           _error_ = 0;
        end;
    end;
   run;

  %fcf_rcset(&syserr);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %let _fcfmsg_=ERROR: FCF: Error reading from &db_lib_kc..FSK_RC_REPORT;
      %fcf_put_error;
      %goto MACRO_END;
  %end;

  %if %upcase(&test_run) eq Y %then
  %do;
     proc sort data=cas_to_delete_with_report;
       by report_key;
      run;

     proc sort data=rc_report_purged;
       by report_key;
      run;

     data cas_to_delete_with_report;
       merge cas_to_delete_with_report(in=a) rc_report_purged(in=b keep=report_key);
       by report_key;
       if a and not b;
      run;

     proc datasets lib=work nowarn nolist;
       delete rc_report_purged;
     quit;
  %end;

  proc sort data=cas_to_delete_with_report nodupkey;
    by case_id;
   run;

  proc sort data=cas_to_delete;
    by case_id;
   run;

  data cas_to_delete(keep=case_id case_id_char);
    if 0 then set &db_lib_kc..fsk_comment(keep=object_id rename=(object_id = case_id_char));
    label case_id_char = "case_id_char";
    merge cas_to_delete(in=a) cas_to_delete_with_report(in=b);
    by case_id;
    if a and not b;
    case_id_char = strip(put(case_id, best.));
   run;

  %fcf_rcset(&syserr);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %let _fcfmsg_=ERROR: FCF: Error creating cas_to_delete;
      %fcf_put_error;
      %goto MACRO_END;
  %end;

  proc datasets lib=work nowarn nolist;
    delete cas_to_delete_with_report;
  quit;

  %if &purgekey_lib_assigned eq 1 %then
  %do;
     data purgekey.cas_to_delete(%fcf_get_ds_opts(lib_name=purgekey, ds_name=cas_to_delete, output_ds=Y));
      set cas_to_delete;
      run;
  %end;

  %BEGIN_CASE_PURGE:

  /*-----------------------------------
   * Delete from FSK_CASE_ENTITY_EVENT
   *-----------------------------------*/
  %fcf_purge_from_table(table=&db_lib_kc..fsk_case_entity_event,
                        entities_ds=cas_to_delete,
                        key_vars=case_id,
                        one_to_many_ind=Y,
                        other_output_ds=case_entity_event_purged,
                        other_output_ds_vars=case_id event_id);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  %fcf_purge_add_to_log_ds(in_ds=case_entity_event_purged,
                           libname=&db_lib_kc,
                           table=fsk_case_entity_event,
                           key_1=case_id,
                           key_1_type=N,
                           key_2=event_id,
                           key_2_type=N);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  /*----------------------------
   * Delete from FSK_CASE_EVENT
   *----------------------------*/
  %fcf_purge_from_table(table=&db_lib_kc..fsk_case_event,
                        entities_ds=cas_to_delete,
                        key_vars=case_id,
                        one_to_many_ind=Y,
                        other_output_ds=case_event_purged,
                        other_output_ds_vars=case_id event_id);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  %fcf_purge_add_to_log_ds(in_ds=case_event_purged,
                           libname=&db_lib_kc,
                           table=fsk_case_event,
                           key_1=case_id,
                           key_1_type=N,
                           key_2=event_id,
                           key_2_type=N);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  /*----------------------------------
   * Delete from FSK_CASE_TRANSACTION
   *----------------------------------*/
  %fcf_purge_from_table(table=&db_lib_kc..fsk_case_transaction,
                        entities_ds=cas_to_delete,
                        key_vars=case_id,
                        one_to_many_ind=Y,
                        other_output_ds=case_transaction_purged,
                        other_output_ds_vars=case_id alert_id transaction_id alerted_entity_key alerted_entity_level_code);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  %fcf_purge_add_to_log_ds(in_ds=case_transaction_purged,
                           %if %upcase(&test_run) eq Y %then
                           %do;
                           delete_in_ds_when_done=N,
                           %end;
                           libname=&db_lib_kc,
                           table=fsk_case_transaction,
                           key_1=case_id,
                           key_1_type=N,
                           key_2=alert_id,
                           key_2_type=N,
                           key_3=alerted_entity_key,
                           key_3_type=N,
                           key_4=alerted_entity_level_code,
                           key_4_type=C,
                           key_5=transaction_id,
                           key_5_type=C);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  /*-----------------------------------------
   * Delete from FSK_CASE_ENTITY_TRANSACTION
   *-----------------------------------------*/
  %fcf_purge_from_table(table=&db_lib_kc..fsk_case_entity_transaction,
                             entities_ds=cas_to_delete,
                             key_vars=case_id,
                             one_to_many_ind=Y,
                             other_output_ds=case_entity_transaction_purged,
                             other_output_ds_vars=case_id transaction_id alerted_entity_number alerted_entity_level_code);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  %fcf_purge_add_to_log_ds(in_ds=case_entity_transaction_purged,
                           %if %upcase(&test_run) eq Y %then
                           %do;
                           delete_in_ds_when_done=N,
                           %end;
                           libname=&db_lib_kc,
                           table=fsk_case_entity_transaction,
                           key_1=case_id,
                           key_1_type=N,
                           key_2=transaction_id,
                           key_2_type=C,
                           key_3=alerted_entity_number,
                           key_3_type=C,
                           key_4=alerted_entity_level_code,
                           key_4_type=C);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  /*-----------------------------
   * Delete from FSK_CASE_ENTITY
   *-----------------------------*/
  %fcf_purge_from_table(table=&db_lib_kc..fsk_case_entity,
                        entities_ds=cas_to_delete,
                        key_vars=case_id,
                        one_to_many_ind=Y,
                        other_output_ds=case_entity_purged,
                        other_output_ds_vars=case_id entity_key entity_number entity_level_code);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  %fcf_purge_add_to_log_ds(in_ds=case_entity_purged,
                           %if %upcase(&test_run) eq Y %then
                           %do;
                           delete_in_ds_when_done=N,
                           %end;
                           libname=&db_lib_kc,
                           table=fsk_case_entity,
                           key_1=case_id,
                           key_1_type=N,
                           key_2=entity_key,
                           key_2_type=N,
                           key_3=entity_number,
                           key_3_type=C,
                           key_4=entity_level_code,
                           key_4_type=C);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  /*----------------------------
   * Delete from FSK_ALERT_CASE
   *----------------------------*/
  %fcf_purge_from_table(table=&db_lib_kc..fsk_alert_case,
                        entities_ds=cas_to_delete,
                        key_vars=case_id,
                        one_to_many_ind=Y,
                        other_output_ds=alert_case_purged,
                        other_output_ds_vars=case_id alert_id);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  %fcf_purge_add_to_log_ds(in_ds=alert_case_purged,
                           libname=&db_lib_kc,
                           table=fsk_alert_case,
                           key_1=case_id,
                           key_1_type=N,
                           key_2=alert_id,
                           key_2_type=N);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  /*----------------------
   * Delete from FSK_CASE
   *----------------------*/
  %fcf_purge_from_table(table=&db_lib_kc..fsk_case,
                        entities_ds=cas_to_delete,
                        key_vars=case_id,
                        one_to_many_ind=N,
                        other_output_ds=case_purged,
                        other_output_ds_vars=case_id case_category_code);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;

  %fcf_purge_add_to_log_ds(in_ds=case_purged,
                           libname=&db_lib_kc,
                           table=fsk_case,
                           key_1=case_id,
                           key_1_type=N,
                           key_2=case_category_code,
                           key_2_type=C);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
  %do;
      %goto MACRO_END;
  %end;


  %MACRO_END:


  %if &trans_rc eq 3 or &trans_rc ge 5 %then
     %fcf_execute_macro(fcf_error);
  %else %do;
     %fcf_execute_macro(post&_fcfmodule_);
     %fcf_execute_macro(fcf_success);
  %end;

%mend fcf_purge_by_age;

