/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_rap.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    Risk Assessment Processing
+------------------------------------------------------------------------------+
| COMMENTS:
|    Determines each customer's risk classification and routes risk
|    assessments to the Investigation UI.  Run less frequently than
|    the risk classification process (fcf_rcp.sas).
|
|    Risk classification designates which of three levels best
|    describes a customer's risk of money laundering. Each customer is
|    assigned a risk classification expressed as high, medium, or low.
|    A risk assessment is a proposal to change a customer's risk
|    classification. The risk classification process generates the
|    risk assessments that appear in the Investigation UI. In addition,
|    the risk classification process executes a series of modules that
|    prepare the risk assessments for review.
|
|    Each risk assessment has a current risk classification and a
|    proposed risk classification. Data available from the Core
|    database and rules � known as risk classifiers � are used to
|    determine a customer's risk classification.
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/
 
/*-------------------------------------------------------------------
 *       Host and Site specific customizations may be REQUIRED
 *-------------------------------------------------------------------*/

%macro fcf_rap(start_yyyymm=, end_yyyymm=);

  /*----------------------------------------------------------
  * Collapse Risk Calsifier Data based on input start and end 
  *----------------------------------------------------------*/
  %fcf_section_header(Section=RAP_RC_COLLAPSE);
     %fcf_rc_collapse(start_month_key=&start_yyyymm., end_month_key=&end_yyyymm.);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=RAP_RC_COLLAPSE);

  /*----------------------
  * Generate Scoring Code
  *----------------------*/
  %fcf_rc_score_codegen (rc_h_threshold=10, rc_m_threshold=7);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %goto MACRO_END;
  %end;

  /*----------------------
  * Execute Scoring Code
  *----------------------*/
  %fcf_section_header(Section=RAP_AE_RC_SCORE);
     %include '!FCFROOT/classifier/codegen/rc_score.sas';
     %fcf_rcset(&syserr)
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmodule_=RC_SCORE;
        %let _fcfmsg_=ERROR: FCF: RCP: rc_score.sas failed check log.;
        %fcf_put_error;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=RAP_AE_RC_SCORE);

  /*-------------------------------
  * Get Score info for each party
  *-------------------------------*/
  %fcf_section_header(Section=RAP_RC_GET_ENTITY_SCORE_INFO);
     %fcf_rc_get_entity_score_info;
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=RAP_RC_GET_ENTITY_SCORE_INFO);

  /*------------------------
  * Set ASSESSMENT_TYPE_CD 
  *------------------------*/
  data mst_rc.rc_routed_open(%fcf_get_ds_opts(lib_name=mst_rc, ds_name=rc_routed_open, output_ds=Y));
      set stg_rc.rc_open_assess(%fcf_get_ds_opts(lib_name=stg_rc, ds_name=rc_open_assess));
      assessment_type_cd='AMLRISK';
  run;

  data mst_rc.rc_routed_closed(%fcf_get_ds_opts(lib_name=mst_rc, ds_name=rc_routed_closed, output_ds=Y));
      set stg_rc.rc_closed_assess(%fcf_get_ds_opts(lib_name=stg_rc, ds_name=rc_closed_assess));
      assessment_type_cd='AMLRISK';
  run;

  /*----------------------------------------------------------------------------
  * If Auto Routing preference set, then Route Entities using Round Robin Method
  *----------------------------------------------------------------------------*/
  %fcf_get_property_value(property=fcf.svr.aml.autoRouteRisks,
                          output_property_value=AUTO_ROUTE_RISKS,
                          output_status=property_status); 
  %if &property_status ne 0 %then %do;
      %let _fcf_msg_=WARNING:  The autoRouteRisks property is not set.  No Auto Routing will be done;
      %let trans_rc=4;
      %fcf_put_error;
  %end;
  %else %if &AUTO_ROUTE_RISKS ne 0 %then %do;
     %fcf_get_property_value(property=fcf.svr.aml.RouteRiskRole,
                             output_property_value=ROUTE_RISK_TO_ROLE,
                             output_status=property_status); 
     %if &property_status ne 0 %then %do;
         %let _fcf_msg_=WARNING:  The RouteRiskRole is not set.  No Auto Routing will be done;
         %let trans_rc=4;
         %fcf_put_error;
     %end; 
     %else %do;
        %fcf_section_header(Section=RAP_ROUTE_OPEN_ASSESS);
        %fcf_route_entity_cdd(inLib=MST_RC,
                          inDS=rc_routed_open, 
                          pk_key=party_number,
                          bank_users=&bankUsersGroup, 
                          role_name=&route_risk_to_role, 
                          column_name=owner_user_long_id,
                          sasdataset=Y
                          ); 

        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
           %goto MACRO_END;
        %end;
        %fcf_section_header(Section=RAP_ROUTE_OPEN_ASSESS);

        /*
         * This section is optional
         */
        %fcf_section_header(Section=RAP_ROUTE_CLOSED_ASSESS);
        %fcf_route_entity_cdd(inLib=MST_RC,
                          inDS=rc_routed_closed, 
                          pk_key=party_number,
                          bank_users=&bankUsersGroup, 
                          role_name=&route_risk_to_role, 
                          column_name=owner_user_long_id,
                          sasdataset=Y
                          ); 

        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
           %goto MACRO_END;
        %end;
        %fcf_section_header(Section=RAP_ROUTE_CLOSED_ASSESS);
     %end;
  %end;

  /*--------------------------
  * Load Risk Assessment Data
  *--------------------------*/
  %fcf_section_header(Section=RAP_LOAD_ROUTED_OPEN);
     %fcf_rc_load(in=MST_RC.rc_routed_open,
                  out=SEG_KC.fsk_risk_assessment,
                  bulkload=);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
    
     
  %fcf_section_footer(Section=RAP_LOAD_ROUTED_OPEN);

  /*
   * This section is optional
   */
  %fcf_section_header(Section=RAP_LOAD_ROUTED_CLOSED);
     %fcf_rc_load(in=MST_RC.rc_routed_closed,
                  out=SEG_KC.fsk_risk_assessment,
                  bulkload=);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=RAP_LOAD_ROUTED_CLOSED);
  /*
   * End optional section
   */

  /*----------------------
  * Load Classifier Data
  *----------------------*/
  %fcf_section_header(Section=RAP_LOAD_ASSESS_CLASSIFIER);
     %fcf_rc_load(in=MST_RC.rc_risk_assessment_classifier,
                  out=SEG_KC.fsk_risk_assessment_classifier,
                  bulkload=);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=RAP_LOAD_ASSESS_CLASSIFIER);

  /*----------------------
  * Update History
  *----------------------*/
  %fcf_section_header(Section=RAP_RC_UPDATE_HISTORY);
     %fcf_rc_update_history(inLib=MST_RC,
                            inDS=rc_routed_open,
                            bulkload=);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=RAP_RC_UPDATE_HISTORY);

  %fcf_section_header(Section=RAP_RC_UPDATE_ASSESS_DATE);
     %fcf_rc_update_assess_date;
  %fcf_section_footer(Section=RAP_RC_UPDATE_ASSESS_DATE);


  %MACRO_END:

   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %fcf_execute_macro(fcf_error);
   %else %do;
      %fcf_execute_macro(fcf_success);
   %end;

%mend fcf_rap;

