/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_route_entity_cdd.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    Auto Route Risk Factor, or CDD event using a round robin
|          method to assign to persons having the input role.
+------------------------------------------------------------------------------+
| USAGE: %fcf_route_entity_cdd(inLib=, inDS=, pk_key=, column_name=
|                          bank_users=, role_name=, sasdataset=);
+------------------------------------------------------------------------------+
| COMMENTS: INPUTS
|             inLib=  SAS library containing the inDS data set
|                     i.e., STG_ALER
|             inDS=   Table name to update using round-robin routing
|             pk_key= Primary key for input data set which is used for joins
|             column_name=  The column name from the input table containing
|                     routing information
|             bank_name= The bank ID for selection of members from metadata
|             role_name= The metadata role name used to extract persons
|                     to assign entities for investigation
|             sasdataset= This flag specifies if the table to update is a 
|                      SAS dataset or a database table.  SAS datasets are
|                      updated using a datastep merge while database tables
|                      must us a proc sql update.  Although the proc sql update
|                      will work for SAS datasets, this flag is used to improve
|                      performance when updating a SAS table.
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_route_entity_cdd(inLib=,                 /* input library name          */
                 inDS=,                         /* input data set name         */
                 pk_key=,                       /* primary key for joins       */
                 column_name=,                  /* column name to update       */
                 bank_users=&bankUsersGroup,    /* bank name to select members */
                 role_name=,                    /* role name to extract users  */
                 sasdataset=Y                   /* role name to extract users  */
                ) ;

  /*------------
   * Set Module
   *------------*/
  %local _fcfmodule_;
  %let _fcfmodule_=FCF_ROUTE_ENTITY_CDD;

  /*-------------------------------------
   * Call prefcf_route_entity stub macro
   *------------------------------------*/
  %fcf_execute_macro(pre&_fcfmodule_);

  /*------------------------------------------
   * Set error code to 0 and message to blank
   *------------------------------------------*/
  %let trans_rc = 0;
  %let _fcfmsg_=;

   %let key=&pk_key;

   %put FCF: NOTE: Start Routing for &inLib..&inDS with keys &key;
  /*-----------------------------------------------
   * Extract identity information from the metadata
   *----------------------------------------------*/
   %fcf_route_entity_investigators(
                     inDS=&inDS.,                    /* input data set name         */
                     bank_users=&bank_users.,        /* bank name to select members */
                     role_name=&role_name.,          /* role name to extract users  */
                     outDS=work.investigator_numbers /* dataset to return           */
                    )
   /*------------------------------------------------------------
   * Assign investigator # to entities that do not have an owner.
   * Save the last investigator assigned for the next time auto
   * routing is done on the input table.
   *-------------------------------------------------------------*/
   data need_to_route route_last_assignment (keep=investigator_num);
   set &inLib..&inDS (
                      %fcf_get_ds_opts(lib_name=&inLib, ds_name=&inDS)
                      keep= &key
                            &column_name
                     ) END=last;
      retain assign_investigator &last_investigator_number;
      investigator_num=.;
      if kstrip(upcase(&column_name)) not in (&investigator_list) then &column_name="";
      if missing(&column_name) 
         then do;
         if assign_investigator eq &num_investigators then assign_investigator=0;
         assign_investigator + 1;
         if &key ne -1 then investigator_num=assign_investigator;
      end;
      output need_to_route; 
      if last then output route_last_assignment;
   run;

 
   /*-----------------------------------------------------------
   * Create a table of investigator names for each key in the
   * input table
   *------------------------------------------------------------*/
   proc sql;
      create table routing_info as
     select 
           a.&key,
       coalesce(a.&column_name,b.investigator_name) as &column_name
    from  need_to_route  A 
    left join investigator_numbers B
      on a.investigator_num=b.investigator_num
      order by 
           a.&key ;
   quit;
      
   %fcf_rcset(&sqlrc);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: Unable to create a routing table with investigator name;
        %fcf_put_error;
        %goto MACRO_END;
     %end;

   /*-----------------------------------------------------------------------
   * For sas datasets, sort and merge data to replace missing owner_userids
   * For database tables, do proc sql update on table
   *-----------------------------------------------------------------------*/
   %if %upcase(&sasdataset)=Y %then %do;

      proc sort data=&inLib..&inDS(%fcf_get_ds_opts(lib_name=&inLib, ds_name=&inDS));
         by &key ;
      run;

      data &inLib..&inDS(%fcf_get_ds_opts(lib_name=&inLib, ds_name=&inDS, output_ds=Y));
      merge &inLib..&inDS (
                           %fcf_get_ds_opts(lib_name=&inLib, ds_name=&inDS)
                           in=a
                          )
            routing_info;
         by &key ;
         if a;
      run;

      %fcf_rcset(&syserr);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
          %let _fcfmsg_=ERROR: FCF: Unable to merge owner in &InLib..&inDS column &column_name with investigator id;
          %fcf_put_error;
          %goto MACRO_END;
      %end;

  %end;
  %else %do;
  
      proc sql;

         update &inLib..&inDS(%fcf_get_ds_opts(lib_name=&inLib, ds_name=&inDS)) as A
         set &column_name=(select B.&column_name
                           from routing_info as B
                           where A.&key = B.&key
                            );
     quit;

     %fcf_rcset(&sqlrc);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: Unable to update &InLib..&inDS column &column_name with investigator id;
        %fcf_put_error;
        %goto MACRO_END;
     %end;

  %end;

  /*---------------------------------------------------------------------------------
  * Update the control table with the  table name and the last investigator assigned
  *--------------------------------------------------------------------------------*/
  proc sql;
     delete
       from stg_ctrl.route_last_assignment(%fcf_get_ds_opts(lib_name=stg_ctrl, ds_name=route_last_assignment))
      where upcase(table_name)=upcase("&inDS");

     insert
       into stg_ctrl.route_last_assignment %fcf_get_ds_opts(lib_name=stg_ctrl, ds_name=route_last_assignment, add_parens=Y)
        set table_name=upcase("&inDS");

     update stg_ctrl.route_last_assignment(%fcf_get_ds_opts(lib_name=stg_ctrl, ds_name=route_last_assignment))
        set last_user_assigned=(select investigator_name
                                  from route_last_assignment a
                                  join investigator_numbers b
                                    on a.investigator_num=b.investigator_num)
      where upcase(table_name)=upcase("&inDS");
   quit;

   %fcf_rcset(&sqlrc);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: Unable to update stg_ctrl.route_last_assignment with last investigator;
        %fcf_put_error;
        %goto MACRO_END;
   %end;

   
  /*------------------- 
  * Clean up work lib
  *-------------------*/
  proc datasets library=work nolist;
     delete investigator_numbers;
     delete need_to_route;
     delete route_last_assignment;
     delete routing_info;
  quit;

 %MACRO_END:

   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %fcf_execute_macro(fcf_error);
   %else %do;
      %fcf_execute_macro(post&_fcfmodule_);
      %fcf_execute_macro(fcf_success);
   %end;

%mend fcf_route_entity_cdd;


