/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_route_entity_queue.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    This template macro can either call the default round robin routing
|          macro or it can be used to implement custom routing logic for queues.
|          This template contains logic to route employee entities and alerted
|          entities containing a watchlist alert to EMP and WAT queues
|          respectively
+------------------------------------------------------------------------------+
| USAGE: %fcf_route_entity_queue(inLib=, inDS=, pk_key=, column_name=
|                          bank_name=, role_name=, sasdataset=);
+------------------------------------------------------------------------------+
| COMMENTS: INPUTS
|             inLib=      SAS library containing the inDS data set
|                         i.e., STG_ALER
|             inDS=       Table name to update using round-robin routing
|             pk_key=     Primary key for input data set which is used for joins
|                         Note: FSK_ALERTED_ENTITY does not have a primary key
|                         This field should be left blank
|             column_name=The column name from the input table containing
|                         routing information
|                         Note: Only passed to RoundRobin routing
|             bank_name=  The bank ID for selection of members from metadata
|                         Note: Only passed to RoundRobin routing
|             role_name=  The metadata role name used to extract persons
|                         to assign entities for investigation
|                         Note: Only passed to RoundRobin routing
|             sasdataset= This flag specifies if the table to update is a 
|                         SAS dataset or a database table.  SAS datasets are
|                         updated using a datastep merge while database tables
|                         must us a proc sql update.  Although the proc sql update
|                         will work for SAS datasets, this flag is used to improve
|                         performance when updating a SAS table.
|                         Note: Only passed to RoundRobin routing
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/
%macro fcf_route_entity_queue(inLib=,      /* input library name          */
			inDS=,                         /* input data set name         */
			pk_key=,                       /* primary key for joins       */
			column_name=,                  /* column name to update       */
			bank_users=&bankUsersGroup,    /* bank name to select members */
			role_name=,                    /* role name to extract users  */
			sasdataset=Y                   /*                             */
			);
	/*------------
	 * Set Module
    *------------*/
	%local _fcfmodule_;
	%let _fcfmodule_=FCF_ROUTE_ENTITY_QUEUE;

	/*------------------------------------------
	 * Set error code to 0 and message to blank
    *------------------------------------------*/
	%let trans_rc = 0;
	%let _fcfmsg_=;

    /*------------------------------------------
        Get list of configured queue codes
        for later validation.
    *------------------------------------------*/
    proc sql noprint;
        select quote(strip(queue_code)) into :valid_queue_codes separated by ','
          from &seg_kc_libname..fsk_queue(%fcf_get_ds_opts(lib_name=&seg_kc_libname, ds_name=fsk_queue))
        where change_current_ind='Y' and logical_delete_ind='N' and active_ind='Y';
    quit;
    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: Unable to get list of valid queues;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    %if %upcase(&inDS)=FSK_ALERTED_ENTITY %then %do;
        /* remove queue routing for entities no longer alerted */
        proc sql noprint;
            /* drop queue routing records for any no longer alerted */
            delete
              from &seg_kc_libname..fsk_entity_queue(%fcf_get_ds_opts(lib_name=&seg_kc_libname, ds_name=fsk_entity_queue))
            where alerted_entity_number not in
            (select alerted_entity_number from &inDS.(%fcf_get_ds_opts(ds_name=&inDS)));
        quit;
        %fcf_rcset(&syserr);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _fcfmsg_=ERROR: FCF: Unable to clean fsk_entity_queue table;
            %fcf_put_error;
            %goto MACRO_END;
        %end;

        %let key=alerted_entity_number;
        %let ent_type=alerted_entity_level_code;
        %let wat_flag=wat_flag;

        %put FCF: NOTE: Start Routing for &inLib..&inDS with keys &key;

        /*------------------------------------------------------------
        * Find entities without queue, and assign queue if needed
        * This code may need modifying for installation specific queues
        * Each entity can by routed to multiple queues, but we do not
        * want to duplicate entries
        *-------------------------------------------------------------*/
        proc sql noprint;
            create table queue_routing as
            (select 'EMP' as queue_code, &ent_type. as alerted_entity_level_code, &key. as alerted_entity_number
            from &inLib..&inDS.(%fcf_get_ds_opts(lib_name=&inLib, ds_name=&inDS)) where employee_ind='Y'
            union
            select 'WAT' as queue_code, &ent_type. as alerted_entity_level_code, &key. as alerted_entity_number
            from &inLib..&inDS.(%fcf_get_ds_opts(lib_name=&inLib, ds_name=&inDS)) where &wat_flag.)
            except
            select queue_code, alerted_entity_level_code, alerted_entity_number
            from &db_kc_libname..fsk_entity_queue;
        quit;

        %fcf_rcset(&syserr);

        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _fcfmsg_=ERROR: FCF: Unable to create a routing table with queue name;
            %fcf_put_error;
            %goto MACRO_END;
        %end;

        /*-----------------------------------------------------------------------
         * update entity_queue table with new queues
         *-----------------------------------------------------------------------*/

        proc sql noprint;
            insert into &db_kc_libname..fsk_entity_queue
                  (alerted_entity_level_code, alerted_entity_number, queue_code)
            select alerted_entity_level_code, alerted_entity_number, queue_code from
            queue_routing where queue_code in (&valid_queue_codes.);
        quit;

        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _fcfmsg_=ERROR: FCF: Unable to insert queue name into routing table;
            %fcf_put_error;
            %goto MACRO_END;
        %end;

        data _null_;
            set queue_routing(where=(queue_code not in (&valid_queue_codes.)));
            put 'WAR' "NING: FCF: Invalid queues being set in &_fcfmodule_. for &inLib..&inDS. "
            queue_code= alerted_entity_number=;
        run;

        /*-------------------
         * Clean up work lib
         *-------------------*/
        proc datasets library=work nolist;
            delete queue_routing;
        quit;
    %end;
    %else %if %upcase(&inDS)=_TMP_CDD_REVIEW %then %do;
        %let key=primary_entity_number;
        %let ent_type=primary_entity_level_code;
        %put FCF: NOTE: Start Routing for &inLib..&inDS with keys &key;
        proc sql noprint;
            update &inLib..&inDS(%fcf_get_ds_opts(lib_name=&inLib, ds_name=&inDS))
                set queue_code=
                case
                    when queue_code is null and employee_ind='Y'
                        then 'EMP'
                    else queue_code
                end;
        quit;

        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _fcfmsg_=ERROR: FCF: Unable to update queue_code in table &inLib..&inDS.;
            %fcf_put_error;
            %goto MACRO_END;
        %end;

        data _null_;
            set &inLib..&inDS(%fcf_get_ds_opts(lib_name=&inLib, ds_name=&inDS)
                              where=(queue_code not in (&valid_queue_codes.,'')));
            put 'WAR' "NING: FCF: &_fcfmodule_. invalid queues in &inLib..&inDS. " queue_code= primary_entity_number=;
        run;
    %end;

%MACRO_END:

		%if &trans_rc eq 3 or &trans_rc ge 5 %then
			%fcf_execute_macro(fcf_error);
		%else %do;
			%fcf_execute_macro(post&_fcfmodule_);
			%fcf_execute_macro(fcf_success);
		%end;

%mend fcf_route_entity_queue;
