/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_sas10005cicoka_rep.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    Custom transaction replication for AML CTR scenarios SAS10005CIKA
|		   and sas10005COKA.
+------------------------------------------------------------------------------+
| USAGE:   %fcf_sas10005cicoka_rep;
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2015-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_sas10005cicoka_rep;

	/*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/

    %let trans_rc = 0;
    %let _fcfmsg_=;


	%let inDs = stg_aler.custom_rep_alerts;

	%if %sysfunc(exist(&inDs)) %then %do;

		/* Get number of alerts applicable to this replication macro */
		proc sql;
			select
				count(*)
			into
				:num_alerts 
			from
				&inDs(%fcf_get_ds_opts(ds_name=&inDs))
			where
				scenario_name
			in
				('SAS10005CIKA','SAS10005COKA');
		quit;

		%fcf_rcset(&syserr);

		%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;

			%let _fcfmsg_=ERROR: FCF: EError while getting count of alerts for SAS10005CIKA and SAS10005COKA transaction replication;
			%fcf_put_error;
			%goto MACRO_END;

		%end;

		/* Get the runasofdate for processing */
		%fcf_get_runasofdate;
		%put Rundate: &runasofdate;

		/* Only start processing if input dataset contains*/	
		%if &num_alerts  > 0 %then %do;

			/* Create subset of transactions from FSC_CASH_FLOW_FACT applicable to runasofdate */
			proc sql;
		    	create table work.cash_flow_rundate(compress=yes) as
					select
						*
					from
						&db_core_libname..fsc_cash_flow_fact 
			      	where
						date_key=&runasofdate;
	 		quit;

			%fcf_rcset(&syserr);

			%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
				%let _fcfmsg_=ERROR: FCF: Error creating CASH_FLOW_RUNDATE for SAS10005CIKA and SAS10005COKA transaction replication;
				%fcf_put_error;
				%goto MACRO_END;
			%end;

			/* Get all transactions from FSC_CASH_FLOW_FACT derived relevant to scenarios */

			proc sql;
				create table
					work.SAS10005CICOKA_replication (compress=yes) as
				select distinct
					ALT.alert_id,
					CSHFLW.*,
					'N' as trigger_ind
				from
					(select
						*
					from
						&inDs(%fcf_get_ds_opts(ds_name=&inDs))
					where
						scenario_name in ('SAS10005CIKA','SAS10005COKA')
					) ALT,
					work.cash_flow_rundate CSHFLW,
					&db_core_libname..fsc_household_party_bridge HHD_PTY,
					&db_core_libname..fsc_party_account_bridge PTY_ACC
				where
					ALT.primary_entity_key = HHD_PTY.household_key and
					HHD_PTY.party_key = PTY_ACC.party_key and
					PTY_ACC.account_key = CSHFLW.account_key and 
					PTY_ACC.change_current_ind = 'Y' and
					HHD_PTY.change_current_ind = 'Y';
			quit;

			%fcf_rcset(&syserr);

			%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
				%let _fcfmsg_=ERROR: FCF: Error creating SAS10005CICOKA_replication for SAS10005CIKA and SAS10005COKA transaction replication;
				%fcf_put_error;
				%goto MACRO_END;
			%end;

			/* Get triggering transactions */

			proc sql;
				create table
					work.SAS10005CICOKA_trig as
				select
					REP.*,
					TRIG.transaction_key as triggertran_key
				from
					work.SAS10005CICOKA_replication REP
				left join
					(select
						*
					from
						stg_aler.triggering_transaction_keys(%fcf_get_ds_opts(lib_name=stg_aler, ds_name=triggering_transaction_keys))
					where
						scenario_id	in
							(select
								scenario_id
							from
								&seg_kc_libname..fsk_scenario(%fcf_get_ds_opts(lib_name=&seg_kc_libname, ds_name=fsk_scenario))
							where
								scenario_name in ('SAS10005CIKA','SAS10005COKA') and
								scenario_status_code='ACT' and 
								logical_delete_ind='N' and
								current_ind='Y'
							)
					) TRIG
				on
					REP.alert_id = TRIG.alert_id and
					REP.transaction_key = TRIG.transaction_key;
			quit;

			%fcf_rcset(&syserr);

			%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
				%let _fcfmsg_=ERROR: FCF: Error creating SAS10005CICOKA_trig for SAS10005CIKA and SAS10005COKA transaction replication;
				%fcf_put_error;
				%goto MACRO_END;
			%end;

			/* Mark triggering transactions */

			data work.SAS10005CICOKA_all(drop=triggertran_key);
				set work.SAS10005CICOKA_trig;
				if transaction_key=triggertran_key then trigger_ind='Y';
			run;

			%fcf_rcset(&syserr);

			%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
				%let _fcfmsg_=ERROR: FCF: Error creating SAS10005CICOKA_all for SAS10005CIKA and SAS10005COKA transaction replication;
				%fcf_put_error;
				%goto MACRO_END;
			%end;

			/* Load replicated transactions into Knowledge Center */

			proc append
				base=&seg_kc_libname..fsk_cash_flow_alert(%fcf_get_ds_opts(lib_name=&seg_kc_libname, ds_name=fsk_cash_flow_alert, output_ds=Y))
			 	data=work.SAS10005CICOKA_all force;
			run;

			%fcf_rcset(&syserr);

			%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
				%let _fcfmsg_=ERROR: FCF: Error appending replicated transactions to FSK_CASH_FLOW_ALERT for SAS10005CIKA and SAS10005COKA transaction replication;
				%fcf_put_error;
				%goto MACRO_END;
			%end;

			/* Load transaction_key and alert_id into in-database temp table */
			
			proc sql;
		        %fcf_begin_pass_through(dbflavor=&dbflavor, authenticationDomain=&segkcauthdomain, connOpts=&segKCDBConnOpts);
				execute (delete from &segKCSchema..FSK_TEMP_TABLE) by &dbflavor;
		        %fcf_end_pass_through(dbflavor=&dbflavor, commit=yes);
			quit;

			%fcf_rcset(&sqlrc);

			%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
				%let _fcfmsg_=ERROR: FCF: Unable to empty FSK_TEMP_TABLE for SAS10005CIKA and SAS10005COKA transaction replication;
				%fcf_put_error;
				%goto MACRO_END;
			%end;

			/* Stage temp table by loading work table into in-database temp table */

			data fsk_temp_table;
				if 0 then set &seg_kc_libname..fsk_temp_table(%fcf_get_ds_opts(lib_name=&seg_kc_libname, ds_name=fsk_temp_table));
				set SAS10005CICOKA_all (keep=alert_id transaction_key);

				TEMP_NUM_1_VALUE=alert_id;
				TEMP_NUM_2_VALUE=transaction_key;

				drop alert_id transaction_key;
			run;

			%fcf_rcset(&syserr);

			%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
				%let _fcfmsg_=ERROR: FCF: Unable to create staging dataset for FSK_TEMP_TABLE load  for SAS10005CIKA and SAS10005COKA transaction replication;
				%fcf_put_error;
				%goto MACRO_END;
			%end;

			/* Loading work table into temp table */

			proc append	base=&seg_kc_libname..fsk_temp_table(%fcf_get_ds_opts(lib_name=&seg_kc_libname, ds_name=fsk_temp_table, output_ds=Y))
                                        data=work.fsk_temp_table force;
			quit;

			%fcf_rcset(&syserr);

			%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
				%let _fcfmsg_=ERROR: FCF: Unable to append to FSK_TEMP_TABLE for SAS10005CIKA and SAS10005COKA transaction replication;
				%fcf_put_error;
				%goto MACRO_END;
			%end;

			/* Get all applicalbe transactions from FSK_TRANSACTION_DIM for replaction into FSK_TRANSACTION_ALERT */

			proc sql;
				create table
					work.fsk_transaction_alert_temp as
				select
					TRA.*,
					TEMP.temp_num_1_value as alert_id
				from
					(select
						*
					from
						&db_core_libname..fsc_transaction_dim 
					where
						transaction_key in
						(select
							temp_num_2_value
						from
							&seg_kc_libname..fsk_temp_table(%fcf_get_ds_opts(lib_name=&seg_kc_libname, ds_name=fsk_temp_table)))
					) TRA,
					&seg_kc_libname..fsk_temp_table(%fcf_get_ds_opts(lib_name=&seg_kc_libname, ds_name=fsk_temp_table)) TEMP
				where
					TEMP.temp_num_2_value=TRA.transaction_key;
			quit;

			%fcf_rcset(&syserr);

			%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
				%let _fcfmsg_=ERROR: FCF: Error creating replication dataset FSK_TRANSACTION_ALERT_TEMP for SAS10005CIKA and SAS10005COKA transaction replication;
				%fcf_put_error;
				%goto MACRO_END;
			%end;

			/* Load staged transactions into fsk_transaction_alert */
			
			proc append
				base=&seg_kc_libname..fsk_transaction_alert(%fcf_get_ds_opts(lib_name=&seg_kc_libname, ds_name=fsk_transaction_alert, output_ds=Y))
				data=work.fsk_transaction_alert_temp force;
			run;

			%fcf_rcset(&syserr);

			%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
				%let _fcfmsg_=ERROR: FCF: Error appending replicated transactions to FSK_TRANSACTION_ALERT for SAS10005CIKA and SAS10005COKA transaction replication;
				%fcf_put_error;
				%goto MACRO_END;
			%end;

		%end; 
	%end;

	%MACRO_END:
    
	%if &trans_rc eq 3 or &trans_rc ge 5 %then
		%fcf_execute_macro(fcf_error);
	%else %do;
		%fcf_execute_macro(post&_fcfmodule_);
		%fcf_execute_macro(fcf_success);
	%end;

%mend;
