/*==============================================================================
| NAME:    fcf_wl_eufs_transform
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  UNIX/WINDOWS
================================================================================

WATCH LIST PROVIDER - European Union, Common Foreign and Security Policy (CFSP)
WATCH LIST NAME     - Consolidated list of persons, groups and entities subject
                      to EU financial sanctions
WATCH LIST CODE     - EUFS
WEB LINK            - http://ec.europa.eu/external_relations/cfsp/sanctions/list/consol-list.htm

MODULE TYPE         - TRANSFORM
MODULE PURPOSE      - Transform data in prepartion for data load.

MODULE NOTES
------------
None

==============================================================================*/

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_wl_eufs_transform;

  /*------------
   * Set module
   *------------*/
  %local _fcfmodule_;
  %let _fcfmodule_=fcf_WL_EUFS_TRANSFORM;

  /*---------------------
   * Call pre macro stub
   *---------------------*/
  %fcf_execute_macro(pre&_fcfmodule_);

  /*------------------------------------------
   * Set error code to 0 and message to blank
   *------------------------------------------*/
  %let trans_rc = 0;
  %let _fcfmsg_=;


/******************************************************************************/
/* STEP 1                                                                     */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*                                                                            */
/* Create format $COUNTRY_NAME which is used to resolve 3 digit country codes */
/* to full country names.                                                     */
/*                                                                            */
/* NOTES:                                                                     */
/*                                                                            */
/* seg_kc.FSC_COUNTRY_DIM table MUST exist.                                   */
/*                                                                            */
/* INPUTS(S);                                                                 */
/*                                                                            */
/* seg_kc.FSC_COUNTRY_DIM                                                     */
/*                                                                            */
/* OUTPUT(S):                                                                 */
/*                                                                            */
/* $COUNTRY_NAME format.                                                      */
/******************************************************************************/

   %global numobs;

   %if %sysfunc(exist(seg_kc.FSC_COUNTRY_DIM)) eq 0 %then %do;
      %let _fcfmsg_=ERROR: FCF: SEG_KC.FSC_COUNTRY_DIM does not exist.
                    $COUNTRY_NAME formats will not be built.;
      %let trans_rc=999999;
      %let numobs=0;
      %fcf_put_error;
   %end;
   %else %do;
      proc sql noprint;
         select count(1) into :numobs
         from seg_kc.fsc_country_dim;
      quit;
      %if &numobs eq 0 %then %do;
         %let _fcfmsg_=ERROR: FCF: SEG_KC.FSC_COUNTRY_DIM contains no records.
                       $COUNTRY_NAME formats will not be built.;
         %let trans_rc = 999999;
         %fcf_put_error;
      %end;
   %end;

   %if &numobs ne 0 %then %do;
      data WORK.ctrl;

         retain fmtname 'country_name' type 'c';
         set seg_kc.fsc_country_dim(rename=(country_code_3=start
                                            country_name=label));

         start=upcase(start);

      run;

      proc sort data=WORK.ctrl nodupkey;
         by start type;
      run;

      proc format library=WORK cntlin=WORK.ctrl;
      run;
   %end;


/******************************************************************************/
/* STEP 2                                                                     */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*                                                                            */
/* Create ENTITY subset and remove duplicates.                                */
/*                                                                            */
/* NOTES:                                                                     */
/*                                                                            */
/*                                                                            */
/* INPUTS(S);                                                                 */
/*                                                                            */
/* stg_wtch.wl_eufs_extract_entity                                            */
/*                                                                            */
/* OUTPUT(S):                                                                 */
/*                                                                            */
/* stg_wtch.wl_eufs_transform_01                                              */
/******************************************************************************/
   data stg_wtch.wl_eufs_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_01, output_ds=Y)
                                      keep=
      entity_id
      type
      reg_date
      programme
   );

      set stg_wtch.wl_eufs_extract_entity(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_extract_entity)
                                          rename=(id=entity_id));

   run;

   proc sort data=stg_wtch.wl_eufs_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_01))
             out=stg_wtch.wl_eufs_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_01, output_ds=Y))
             nodupkey;
      by entity_id
         type
         reg_date
         programme
      ;
   run;

/******************************************************************************/
/* STEP 3                                                                     */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*                                                                            */
/* Create NAME subset and remove duplicates.                                  */
/*                                                                            */
/* NOTES:                                                                     */
/*                                                                            */
/*                                                                            */
/* INPUTS(S);                                                                 */
/*                                                                            */
/* stg_wtch.wl_eufs_extract_name                                              */
/*                                                                            */
/* OUTPUT(S):                                                                 */
/*                                                                            */
/* stg_wtch.wl_eufs_transform_02                                              */
/******************************************************************************/
   data stg_wtch.wl_eufs_transform_02(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_02, output_ds=Y)
                                      keep=
      entity_id
      lastname
      firstname
      middlename
      wholename
      title
      function
   );

      set stg_wtch.wl_eufs_extract_name(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_extract_name));

   run;

   proc sort data=stg_wtch.wl_eufs_transform_02(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_02))
             out=stg_wtch.wl_eufs_transform_02(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_02, output_ds=Y))
             nodupkey;
      by entity_id
         lastname
         firstname
         middlename
         wholename
         title
         function
      ;
   run;


/******************************************************************************/
/* STEP 4                                                                     */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*                                                                            */
/* Create ADDRESS subset and remove duplicates.                               */
/*                                                                            */
/* NOTES:                                                                     */
/*                                                                            */
/*                                                                            */
/* INPUTS(S);                                                                 */
/*                                                                            */
/* stg_wtch.wl_eufs_extract_address                                           */
/*                                                                            */
/* OUTPUT(S):                                                                 */
/*                                                                            */
/* stg_wtch.wl_eufs_transform_03                                              */
/******************************************************************************/
   data stg_wtch.wl_eufs_transform_03(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_03, output_ds=Y)
                                      keep=
      entity_id
      number
      street
      zipcode
      city
      country
   );

      set stg_wtch.wl_eufs_extract_address(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_extract_address));

   run;

   proc sort data=stg_wtch.wl_eufs_transform_03(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_03))
             out=stg_wtch.wl_eufs_transform_03(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_03, output_ds=Y))
             nodupkey;
      by entity_id
         number
         street
         zipcode
         city
         country
      ;
   run;


/******************************************************************************/
/* STEP 5                                                                     */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*                                                                            */
/* Create CITIZEN subset and remove duplicates.                               */
/*                                                                            */
/* NOTES:                                                                     */
/*                                                                            */
/*                                                                            */
/* INPUTS(S);                                                                 */
/*                                                                            */
/* stg_wtch.wl_eufs_extract_citizen                                           */
/*                                                                            */
/* OUTPUT(S):                                                                 */
/*                                                                            */
/* stg_wtch.wl_eufs_transform_04                                              */
/******************************************************************************/
   data stg_wtch.wl_eufs_transform_04(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_04, output_ds=Y)
                                      keep=
      entity_id
      country
   );

      set stg_wtch.wl_eufs_extract_citizen(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_extract_citizen));

   run;

   proc sort data=stg_wtch.wl_eufs_transform_04(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_04))
             out=stg_wtch.wl_eufs_transform_04(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_04, output_ds=Y))
             nodupkey;
      by entity_id
         country
      ;
   run;

/******************************************************************************/
/* STEP 6                                                                     */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*                                                                            */
/* Create PASSPORT subset and remove duplicates.                              */
/*                                                                            */
/* NOTES:                                                                     */
/*                                                                            */
/*                                                                            */
/* INPUTS(S);                                                                 */
/*                                                                            */
/* stg_wtch.wl_eufs_extract_passport                                          */
/*                                                                            */
/* OUTPUT(S):                                                                 */
/*                                                                            */
/* stg_wtch.wl_eufs_transform_05                                              */
/******************************************************************************/
   data stg_wtch.wl_eufs_transform_05(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_05, output_ds=Y)
                                      keep=
      entity_id
      number
      country
   );

      set stg_wtch.wl_eufs_extract_passport(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_extract_passport));

   run;

   proc sort data=stg_wtch.wl_eufs_transform_05(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_05))
             out=stg_wtch.wl_eufs_transform_05(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_05, output_ds=Y))
             nodupkey;
      by entity_id
         number
         country
      ;
   run;

/******************************************************************************/
/* STEP 7                                                                     */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*                                                                            */
/* Create BIRTH subset and remove duplicates.                                 */
/*                                                                            */
/* NOTES:                                                                     */
/*                                                                            */
/*                                                                            */
/* INPUTS(S);                                                                 */
/*                                                                            */
/* stg_wtch.wl_eufs_extract_birth                                             */
/*                                                                            */
/* OUTPUT(S):                                                                 */
/*                                                                            */
/* stg_wtch.wl_eufs_transform_06 - date                                       */
/* stg_wtch.wl_eufs_transform_07 - place/country                              */
/******************************************************************************/
   data stg_wtch.wl_eufs_transform_06(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_06, output_ds=Y)
                                      keep=
      entity_id
      date
   );

      set stg_wtch.wl_eufs_extract_birth(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_extract_birth));

      if not missing(date) then
         output stg_wtch.wl_eufs_transform_06;

   run;

   proc sort data=stg_wtch.wl_eufs_transform_06(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_06))
             out=stg_wtch.wl_eufs_transform_06(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_06, output_ds=Y))
             nodupkey;
      by entity_id
         date
      ;
   run;

   data stg_wtch.wl_eufs_transform_07(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_07, output_ds=Y)
                                      keep=
      entity_id
      place
      country
   );

      set stg_wtch.wl_eufs_extract_birth(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_extract_birth));

      if not missing(place) or
         not missing(country) then
         output stg_wtch.wl_eufs_transform_07;

   run;

   proc sort data=stg_wtch.wl_eufs_transform_07(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_07))
             out=stg_wtch.wl_eufs_transform_07(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_07, output_ds=Y))
             nodupkey;
      by entity_id
         place
         country
      ;
   run;

/******************************************************************************/
/* STEP 8                                                                     */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*                                                                            */
/* Process DATE_OF_BIRTH/YEAR_OF_BIRTH and remove duplicates.                 */
/*                                                                            */
/* NOTES:                                                                     */
/*                                                                            */
/*                                                                            */
/* INPUTS(S);                                                                 */
/*                                                                            */
/* stg_wtch.wl_eufs_transform_06 - date                                       */
/*                                                                            */
/* OUTPUT(S):                                                                 */
/*                                                                            */
/* stg_wtch.wl_eufs_transform_08 - date_of_birth/year_of_birth                */
/* stg_err.wl_eufs_errors_01 - errors                                         */
/******************************************************************************/
   data
      stg_wtch.wl_eufs_transform_08(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_08, output_ds=Y)
                                    keep=
         entity_id
         date_of_birth
         year_of_birth
   )
      stg_err.wl_eufs_errors_01(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_eufs_errors_01, output_ds=Y)
                                keep=
         entity_watch_list_number
         watch_list_name
         error_file
         error_record_number
         error_field
         error_value
         error_message
         error_datetime
   );

      retain error_flag 0;

      attrib
         dob_temp                 length=$100 informat=$100.       format=$100.
         first_dob                length=$100 informat=$100.       format=$100.
         last_dob                 length=$100 informat=$100.       format=$100.

         /* ERROR FIELDS */
         error_file          length=$200
                             informat=$200.
                             format=$200.
                             label='Error File'

         error_record_number length=8
                             informat=best32.
                             format=best32.
                             label='Error Record Number'

         error_field         length=$32
                             informat=$32.
                             format=$32.
                             label='Error Field'

         error_value         length=$1000
                             informat=$1000.
                             format=$1000.
                             label='Error Value'

         error_message       length=$200
                             informat=$200.
                             format=$200.
                             label='Error Message'

         error_datetime      length=8
                             informat=nldatm.
                             format=nldatm.
                             label='Error Datetime'
      ;

      /* FSC_ENTITY_WATCH_LIST_DIM */
      /* Sets variable attribs to match variables in DBMS */
      if 1 eq 0 then
         set seg_kc.FSC_ENTITY_WATCH_LIST_DIM;

      set stg_wtch.wl_eufs_transform_06(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_06)) end=eof;

      dob_temp=upcase(date);
      dob_temp=tranwrd(dob_temp,'JANUARY','JAN');
      dob_temp=tranwrd(dob_temp,'FEBRUARY','FEB');
      dob_temp=tranwrd(dob_temp,'MARCH','MAR');
      dob_temp=tranwrd(dob_temp,'APRIL','APR');
      dob_temp=tranwrd(dob_temp,'MAY','MAY');
      dob_temp=tranwrd(dob_temp,'JUNE','JUN');
      dob_temp=tranwrd(dob_temp,'JULY','JUL');
      dob_temp=tranwrd(dob_temp,'AUGUST','AUG');
      dob_temp=tranwrd(dob_temp,'SEPTEMBER','SEP');
      dob_temp=tranwrd(dob_temp,'OCTOBER','OCT');
      dob_temp=tranwrd(dob_temp,'NOVEMBER','NOV');
      dob_temp=tranwrd(dob_temp,'DECEMBER','DEC');
      dob_temp=tranwrd(dob_temp,'APPROXIMATELY','');
      dob_temp=tranwrd(dob_temp,'CIRCA','');
      dob_temp=tranwrd(dob_temp,'C.','');
      dob_temp=tranwrd(dob_temp,'ESTIMATED','');
      dob_temp=tranwrd(dob_temp,'DATE','');
      dob_temp=tranwrd(dob_temp,'OF','');
      dob_temp=tranwrd(dob_temp,'BIRTH','');
      dob_temp=tranwrd(dob_temp,'IN','');
      dob_temp=tranwrd(dob_temp,'ON','');
      dob_temp=tranwrd(dob_temp,'ALT','');
      dob_temp=tranwrd(dob_temp,'BETWEEN','');
      dob_temp=translate(dob_temp,'-','/');

      if indexw(dob_temp,'AND') then
         substr(dob_temp,indexw(dob_temp,'AND'),3)='-';

      if kindex(dob_temp, '(') then
         dob_temp=kscan(dob_temp,1,'(');

      dob_temp=kcompress(kstrip(kcompress(dob_temp,'.:')));

	  /* e.g. 1956-1961 */
      if lengthn(dob_temp) eq 9 and
         kindex(dob_temp,'-') then do;

         first_dob=kstrip(kscan(dob_temp,1,'-'));
         last_dob=kstrip(kscan(dob_temp,2,'-'));

         if anyalpha(first_dob) eq 0 and
            lengthn(first_dob)  eq 4 and
            anyalpha(last_dob)  eq 0 and
            lengthn(last_dob)   eq 4 then do;

            do i=input(first_dob,8.) to input(last_dob,8.);
               date_of_birth=.;
               year_of_birth=i;
               output stg_wtch.wl_eufs_transform_08;
            end;
         end;
         else do;
            entity_watch_list_number=kstrip(put(entity_id,best32.));
            watch_list_name='EUFS';
            error_file="&eufs_file";
            error_record_number=_N_;
            error_field='DATE';
            error_value=kstrip(date);
            error_message='WARNING: Invalid date of birth.';
            error_datetime=datetime();
            error_flag=1;

            putlog 'WARNING: FCF: Invalid date of birth. ' entity_watch_list_number= dob_temp=;
            output stg_err.wl_eufs_errors_01;

         end;
      end;
      else if anyalpha(dob_temp) eq 0 and lengthn(dob_temp) eq 4 then do;
         date_of_birth=.;
         year_of_birth=input(dob_temp,best32.);
         output stg_wtch.wl_eufs_transform_08;
      end;
      else do;
         date_of_birth=input(dob_temp,??anydtdte32.);
         if not missing(date_of_birth) then do;
            date_of_birth=dhms(date_of_birth,00,00,00);
            year_of_birth=year(datepart(date_of_birth));
            output stg_wtch.wl_eufs_transform_08;
         end;
         else if missing(date_of_birth) then do;
            entity_watch_list_number=kstrip(put(entity_id,best32.));
            watch_list_name='EUFS';
            error_file="&eufs_file";
            error_record_number=_N_;
            error_field='DATE';
            error_value=kstrip(date);
            error_message='WARNING: Invalid date of birth.';
            error_datetime=datetime();
            error_flag=1;

            putlog 'WARNING: FCF: Invalid date of birth. ' entity_watch_list_number= dob_temp=;
            output stg_err.wl_eufs_errors_01;
         end;
      end;

      if (eof) and (error_flag) then do;
         putlog ' ';
         putlog 'WARNING: FCF: Invalid values were encountered processing the watch list data';
         putlog 'WARNING: FCF: Data values were written to the STG_ERR.WL_EUFS_ERRORS table.';
      end;

   run;

/******************************************************************************/
/* STEP 9                                                                     */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*                                                                            */
/* Create distinct indvidual and organization records for each combination of */
/* name, address, birth date, citizenship and passport                   .    */
/*                                                                            */
/* NOTES:                                                                     */
/*                                                                            */
/* INPUTS(S);                                                                 */
/*                                                                            */
/* stg_wtch.wl_eufs_transform_01 - Entity data                                */
/* stg_wtch.wl_eufs_transform_02 - Name data                                  */
/* stg_wtch.wl_eufs_transform_03 - Address data                               */
/* stg_wtch.wl_eufs_transform_04 - Citizen data                               */
/* stg_wtch.wl_eufs_transform_05 - Passport data                              */
/* stg_wtch.wl_eufs_transform_07 - Birth Place data                           */
/* stg_wtch.wl_eufs_transform_07 - Birth Date data                            */
/*                                                                            */
/* OUTPUT(S):                                                                 */
/*                                                                            */
/* stg_wtch.wl_eufs_transform_09 - Entity/Name                                */
/* stg_wtch.wl_eufs_transform_10 - Entity/Name/Address                        */
/* stg_wtch.wl_eufs_transform_11 - Entity/Name/Address/Citizen                */
/* stg_wtch.wl_eufs_transform_12 - Entity /Name/Address/Citizen/Passport      */
/* stg_wtch.wl_eufs_transform_13 -                                            */
/*   Entity/Name/Address/Citizen/Passport/Birth Place                         */
/* stg_wtch.wl_eufs_transform_14 -                                            */
/*   Entity/Name/Address/Citizen/Passport/Birth Place/Birth Date              */
/******************************************************************************/
   proc sql noprint;

      create table stg_wtch.wl_eufs_transform_09 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_09, output_ds=Y, add_parens=Y)  as
         select a.*,
                b.lastname,
                b.firstname,
                b.middlename,
                b.wholename,
                b.title,
                b.function
         from stg_wtch.wl_eufs_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_01)) as a full join
              stg_wtch.wl_eufs_transform_02(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_02)) as b
         on a.entity_id=b.entity_id;

      create table stg_wtch.wl_eufs_transform_10 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_10, output_ds=Y, add_parens=Y)  as
         select a.*,
                b.number,
                b.street,
                b.zipcode,
                b.city,
                b.country as country_code_
         from stg_wtch.wl_eufs_transform_09(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_09)) as a full join
              stg_wtch.wl_eufs_transform_03(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_03)) as b
         on a.entity_id=b.entity_id;

      create table stg_wtch.wl_eufs_transform_11 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_11, output_ds=Y, add_parens=Y)  as
         select a.*,
                b.country as citizenship_country_code_
         from stg_wtch.wl_eufs_transform_10(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_10)) as a full join
              stg_wtch.wl_eufs_transform_04(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_04)) as b
         on a.entity_id=b.entity_id;

      create table stg_wtch.wl_eufs_transform_12 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_12, output_ds=Y, add_parens=Y)  as
         select a.*,
                b.number as passport_number_,
                b.country as passport_country_
         from stg_wtch.wl_eufs_transform_11(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_11)) as a full join
              stg_wtch.wl_eufs_transform_05(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_05)) as b
         on a.entity_id=b.entity_id
         order by entity_id;

      create table stg_wtch.wl_eufs_transform_13 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_13, output_ds=Y, add_parens=Y)  as
         select a.*,
                b.place as place_of_birth_,
                b.country as place_of_birth_country_
         from stg_wtch.wl_eufs_transform_12(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_12)) as a full join
              stg_wtch.wl_eufs_transform_07(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_07)) as b
         on a.entity_id=b.entity_id;

      create table stg_wtch.wl_eufs_transform_14 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_14, output_ds=Y, add_parens=Y)  as
         select a.*,
                b.date_of_birth,
                b.year_of_birth
         from stg_wtch.wl_eufs_transform_13(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_13)) as a full join
              stg_wtch.wl_eufs_transform_08(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_08)) as b
         on a.entity_id=b.entity_id;

   quit;

/******************************************************************************/
/* STEP 10                                                                    */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*                                                                            */
/* Perform final transformations on individual and organization records.      */
/*                                                                            */
/* NOTES:                                                                     */
/*                                                                            */
/* INPUTS(S);                                                                 */
/*                                                                            */
/* stg_wtch.wl_eufs_transform_14 - Individuals-Organizations                  */
/*                                                                            */
/* OUTPUT(S):                                                                 */
/*                                                                            */
/* stg_wtch.wl_eufs_transform - Transformed Individuals-Organizations         */
/******************************************************************************/
   data
      stg_wtch.wl_eufs_transform(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform, output_ds=Y)
                                 keep=
         entity_watch_list_number
         watch_list_name
         category_desc
         type_desc
         programs
         identification_id
         identification_type_desc
         date_of_birth
         year_of_birth
         place_of_birth
         first_name
         last_name
         middle_name
         entity_name
         entity_title
         occupation_desc
         address
         city_name
         postal_code
         country_code
         country_name
         full_address
         citizenship_country_code
         citizenship_country_name
         create_date
         exclude_ind
   )
      stg_err.wl_eufs_errors_02(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_eufs_errors_02, output_ds=Y)
                                keep=
         entity_watch_list_number
         watch_list_name
         error_file
         error_record_number
         error_field
         error_value
         error_message
         error_datetime
   );

      retain error_flag 0;

      attrib
         /* ERROR FIELDS */
         error_file          length=$200
                             informat=$200.
                             format=$200.
                             label='Error File'

         error_record_number length=8
                             informat=best32.
                             format=best32.
                             label='Error Record Number'

         error_field         length=$32
                             informat=$32.
                             format=$32.
                             label='Error Field'

         error_value         length=$1000
                             informat=$1000.
                             format=$1000.
                             label='Error Value'

         error_message       length=$200
                             informat=$200.
                             format=$200.
                             label='Error Message'

         error_datetime      length=8
                             informat=nldatm.
                             format=nldatm.
                             label='Error Datetime'
         ;

      /* FSC_ENTITY_WATCH_LIST_DIM */
      /* Sets variable attribs to match variables in DBMS */
      if 1 eq 0 then
         set seg_kc.FSC_ENTITY_WATCH_LIST_DIM;

      set stg_wtch.wl_eufs_transform_14(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_transform_14)) end=eof;

      /* ENTITY_WATCH_LIST_NUMBER */
      entity_watch_list_number=kstrip(put(entity_id,best32.));

      /* WATCH_LIST_NAME */
      watch_list_name='EUFS';

      /* PROGRAMS */
      programs=upcase(kstrip(programme));

      /* CATEGORY_DESC */
	  category_desc='';
      if upcase(programs) eq 'TERR' then
         category_desc='TERRORISM';

      /* ADDRESS */
      address=compbl(kstrip(number)) || ' ' || upcase(compbl(kstrip(street)));

      /* CITY_NAME */
      city_name=upcase(compbl(kstrip(city)));

      /* POSTAL_CODE */
      postal_code=compbl(kstrip(zipcode));

      /* COUNTRY_CODE */
      country_code=upcase(kstrip(country_code_));

      /* COUNTRY_NAME */
	  country_name='';
      if not missing(country_code_) then
         country_name=put(country_code_, $country_name.);

      /* FULL_ADDRESS */
      full_address=catx(';', address, city_name, postal_code, country_name);

      /* CREATE_DATE */
	  create_date=.;
      if not missing(reg_date) then
         create_date=dhms(input(reg_date,??anydtdte32.),00,00,00);

      /* EXCLUDE_IND */
      exclude_ind='N';

      /* TYPE_DESC */
	  type_desc='.';
      if upcase(type) eq 'P' then
         type_desc='INDIVIDUAL';
      else if upcase(type) eq 'E' then
         type_desc='ORGANIZATION';
      else do;

         entity_watch_list_number=kstrip(entity_watch_list_number);
         watch_list_name='EUFS';
         error_file="&eufs_file";
         error_record_number=_N_;
         error_field='TYPE';
         error_value=kstrip(type);
         error_message='ERROR: Unknown TYPE.';
         error_datetime=datetime();
         error_flag=1;

         putlog 'ERROR: FCF: Unknown TYPE. ' entity_watch_list_number= type=;
         output stg_err.wl_eufs_errors_02;
      end;

      /* Process INDIVIDUALS */
      identification_type_desc='';
      identification_id='';
	  place_of_birth='';
      if upcase(type_desc) eq 'INDIVIDUAL' then do;

         /* IDENTIFICATION_TYPE_DESC */
         if index(upcase(passport_number_),'PASSPORT') then
            identification_type_desc='PASSPORT';
         else if index(upcase(passport_number_),'NATIONAL')       or
                 index(upcase(passport_number_),'IDENTIFICATION') or
                 index(upcase(passport_number_),'IDENTITY') then
            identification_type_desc='GOVERNMENT ID';

         /* IDENTIFICATION_ID */
         if kindex(passport_number_,'(') then
            identification_id=compbl(kstrip(kscan(passport_number_,1,'(')));
         else
            identification_id=compbl(kstrip(passport_number_));

         if not missing(passport_country_) then
            identification_id=kstrip(identification_id) || ' - ' || kstrip(passport_country_);

         /* PLACE_OF_BIRTH */
         place_of_birth=compbl(kstrip(place_of_birth_));
		 if missing(place_of_birth) and not missing(place_of_birth_country_) then 
            place_of_birth = compbl(kstrip(place_of_birth_country_));

         /* FIRST_NAME */
         first_name=upcase(compbl(kstrip(firstname)));

         /* MIDDLE_NAME */
         middle_name=upcase(compbl(kstrip(middlename)));

         /* LAST_NAME */
         last_name=upcase(compbl(kstrip(lastname)));

         /* ENTITY_NAME */
         if missing(wholename) then
            entity_name=catx(' ',first_name, middle_name, last_name);
         else
            entity_name=upcase(compbl(kstrip(wholename)));

         /* ENTITY_TITLE */
         entity_title=compbl(kstrip(title));

         /* OCCUPATION_DESC */
         occupation_desc=compbl(kstrip(function));

         /* CITIZENSHIP_COUNTRY_CODE */
         citizenship_country_code=upcase(kstrip(citizenship_country_code_));

         /* CITIZENSHIP_COUNTRY_NAME */
         citizenship_country_name='';
         if not missing(citizenship_country_code_) then
            citizenship_country_name=put(citizenship_country_code_,$country_name.);

         output stg_wtch.wl_eufs_transform;

      end;
      /* Process INDIVIDUALS */
      else if upcase(type_desc) eq 'ORGANIZATION' then do;
           /*--------------------------------------------------------------------------------------
            * K Function note:  Stays compress() since kcompress doesn't support modifier argument
            *--------------------------------------------------------------------------------------*/
           entity_name=upcase(compbl(compress(wholename,,'c')));
		   first_name='';
		   last_name='';
		   middle_name='';
           output stg_wtch.wl_eufs_transform;
      end;

      if (eof) and (error_flag) then do;
         putlog ' ';
         putlog 'WARNING: FCF: Invalid values were encountered processing the watch list data';
         putlog 'WARNING: FCF: Data values were written to the STG_ERR.WL_EUFS_ERRORS table.';
      end;

   run;

/******************************************************************************/
/* STEP 11                                                                    */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*                                                                            */
/* Process data errors                       .                                */
/*                                                                            */
/* NOTES:                                                                     */
/*                                                                            */
/* INPUTS(S);                                                                 */
/*                                                                            */
/* STG_ERR.wl_eufs_errors_01 - Invalid data_of_birth values                   */
/* STG_ERR.wl_eufs_errors_02 - Invalid type_desc                              */
/*                                                                            */
/* OUTPUT(S):                                                                 */
/*                                                                            */
/* STG_WTCH.wl_eufs_errors - Combined invalid data values                     */
/*                                                                            */
/******************************************************************************/
  proc append base=stg_err.wl_eufs_errors(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_eufs_errors, output_ds=Y))
              data=stg_err.wl_eufs_errors_01(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_eufs_errors_01))
              force;
   run;

  proc append base=stg_err.wl_eufs_errors(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_eufs_errors, output_ds=Y))
              data=stg_err.wl_eufs_errors_02(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_eufs_errors_02))
              force;
   run;

  proc sort data=stg_err.wl_eufs_errors(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_eufs_errors))
             out=stg_err.wl_eufs_errors(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_eufs_errors, output_ds=Y))
             nodupkey;
    by entity_watch_list_number
       error_value;
   run;

  %MACRO_END:
   
   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %fcf_execute_macro(fcf_error);
   %else %do;
      %fcf_execute_macro(post&_fcfmodule_);
      %fcf_execute_macro(fcf_success);
   %end;

%mend fcf_wl_eufs_transform;
