/*==============================================================================
| NAME:    fcf_wl_ofac_transform
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  UNIX/WINDOWS
================================================================================

WATCH LIST PROVIDER - U.S. Treasury - Office of Foreign Assets Control
WATCH LIST NAME     - Specially Designated Nationals List (SDN List)
WATCH LIST CODE     - OFAC
WEB LINK            - http://www.treas.gov/offices/enforcement/ofac/

MODULE TYPE         - TRANSFORM
MODULE PURPOSE      - Transform data in prepartion for data load.

MODULE NOTES
------------
None

==============================================================================*/

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_wl_ofac_transform;

  /*------------
   * Set module
   *------------*/
  %local _fcfmodule_;
  %let _fcfmodule_=fcf_WL_OFAC_TRANSFORM;

  /*---------------------
   * Call pre macro stub
   *---------------------*/
  %fcf_execute_macro(pre&_fcfmodule_);

  /*------------------------------------------
   * Set error code to 0 and message to blank
   *------------------------------------------*/
  %let trans_rc = 0;
  %let _fcfmsg_=;

  /*------------------------------------------
   * get column names for watch_list_dim
   *------------------------------------------*/
   %fcf_get_column_names(table=seg_kc.fsc_entity_watch_list_dim);


/******************************************************************************/
/* BEGIN: STG_WTCH.wl_ofac_transform_01                                       */
/*        STG_WTCH.wl_ofac_transform_02                                       */
/*        STG_WTCH.wl_ofac_transform_03                                       */
/*                                                                            */
/* DESCRIPTION: Remove records from all SDN/ADD/ALT where SDN_TYPE=VESSEL     */
/*              or SDN_TYPE=AIRCRAFT
/*                                                                            */
/* STG_WTCH.wl_ofac_transform_01 - SDN                                        */
/* STG_WTCH.wl_ofac_transform_02 - ADD                                        */
/* STG_WTCH.wl_ofac_transform_03 - ALT                                        */
/******************************************************************************/
proc sql noprint;

   create table STG_WTCH.wl_ofac_transform_01 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_01, output_ds=Y, add_parens=Y)  as
      select *
      from STG_WTCH.wl_ofac_sdn_extract(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_ofac_sdn_extract))
      where upcase(sdn_type) ne 'VESSEL'
            and upcase(sdn_type) ne 'AIRCRAFT';

   create table STG_WTCH.wl_ofac_transform_02 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_02, output_ds=Y, add_parens=Y)  as
      select *
      from STG_WTCH.wl_ofac_add_extract(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_ofac_add_extract))
      where ent_num in (select ent_num from STG_WTCH.wl_ofac_transform_01(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_ofac_transform_01)));

   create table STG_WTCH.wl_ofac_transform_03 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_03, output_ds=Y, add_parens=Y)  as
      select *
      from STG_WTCH.wl_ofac_alt_extract(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_ofac_alt_extract))
      where ent_num in (select ent_num from STG_WTCH.wl_ofac_transform_01(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_ofac_transform_01)));

quit;
/******************************************************************************/
/* END: STG_WTCH.wl_ofac_transform_01                                         */
/*      STG_WTCH.wl_ofac_transform_02                                         */
/*      STG_WTCH.wl_ofac_transform_03                                         */
/******************************************************************************/

/******************************************************************************/
/* BEGIN: STG_WTCH.wl_ofac_transform_04                                       */
/*        STG_WTCH.wl_ofac_transform_05                                       */
/*        STG_WTCH.wl_ofac_transform_06                                       */
/*        STG_WTCH.wl_ofac_transform_07                                       */
/*        STG_ERR.wl_ofac_errors_01                                           */
/*                                                                            */
/* DESCRIPTION: Output fields from the SDN multiple entry text field REMARKS. */
/* Multiple entries within the REMARKS text field are delimited by ';'.       */
/*                                                                            */
/* STG_WTCH.wl_ofac_transform_04 - TAX_ID,TAX_ID_TYPE_CODE                    */
/* STG_WTCH.wl_ofac_transform_05 - IDENTIFICATION_ID,IDENTIFICATION_TYPE_DESC */
/* STG_WTCH.wl_ofac_transform_06 - DATE_OF_BIRTH,YEAR_OF_BIRTH                */
/* STG_WTCH.wl_ofac_transform_07 - CITIZENSHIP_COUNTRY_NAME                   */
/******************************************************************************/
data

   STG_WTCH.wl_ofac_transform_04(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_04, output_ds=Y)
                                 keep=
      ent_num
      tax_id
      tax_id_type_code
   )

   STG_WTCH.wl_ofac_transform_05(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_05, output_ds=Y)
                                 keep=
      ent_num
      identification_id
      identification_type_desc
   )


   STG_WTCH.wl_ofac_transform_06(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_06, output_ds=Y)
                                 keep=
      ent_num
      date_of_birth
      year_of_birth
	  place_of_birth
   )

   STG_WTCH.wl_ofac_transform_07(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_07, output_ds=Y)
                                 keep=
      ent_num
      citizenship_country_name
	  nationality_country_name
   )

  
   STG_ERR.wl_ofac_errors_01(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_ofac_errors_01, output_ds=Y)
                             keep=
      entity_watch_list_number
      watch_list_name
      error_file
      error_record_number
      error_field
      error_value
      error_message
      error_datetime
   );

   attrib
      /* ERROR FIELDS */
      error_file          length=$200
                          informat=$200.
                          format=$200.
                          label='Error File'

      error_record_number length=8
                          informat=best32.
                          format=best32.
                          label='Error Record Number'

      error_field         length=$32
                          informat=$32.
                          format=$32.
                          label='Error Field'

      error_value         length=$1000
                          informat=$1000.
                          format=$1000.
                          label='Error Value'

      error_message       length=$200
                          informat=$200.
                          format=$200.
                          label='Error Message'

      error_datetime      length=8
                          informat=nldatm.
                          format=nldatm.
                          label='Error Datetime'

      /* TEMP FIELDS */
      remark_text                    length=$1000
      tax_id_text                    length=$1000
      identification_id_text         length=$1000
      date_of_birth_text             length=$1000
      citizenship_country_name_text  length=$1000
	  nationality_country_name_text  length=$1000
   ;

   retain error_flag 0;

   /* FSC_ENTITY_WATCH_LIST_DIM */
   /* Sets variable attribs to match variables in DBMS */
   if 1 eq 0 then
      set seg_kc.FSC_ENTITY_WATCH_LIST_DIM;

   /* Exclude records where remarks=-0- (no remarks) */
   set STG_WTCH.wl_ofac_transform_01(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_ofac_transform_01)
                                     where=(remarks ne '-0-')
                                     keep=ent_num
                                     sdn_type
                                     remarks) end=eof;

   /* Obtain a count of the number of entries within the remarks field */
   num_remarks=countc(remarks,';') + 1;

   /* Iterate thru remarks field */
   /* Output a record for each instance of a valid entry */
   do i=1 to num_remarks;
      remark_text=kscan(remarks,i,';');

      /* sdn_type=INDIVIDUAL */
      if upcase(sdn_type) eq 'INDIVIDUAL' then do;

         /* TAX_ID
            TAX_ID_TYPE_CODE=S */
         if indexw(upcase(remark_text),'SSN') then do;
            tax_id_text=upcase(remark_text);
            tax_id_text=tranwrd(tax_id_text,'SSN','');
            tax_id_text=tranwrd(tax_id_text,'ALT.','');
            tax_id_text=kcompress(kscan(tax_id_text,1,' '),'.-');

            tax_id=tax_id_text;
            tax_id_type_code='S';
            output STG_WTCH.wl_ofac_transform_04;
         end; /* end: tax_id / tax_id_type_code='S' */

         /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=PASSPORT */
         else if indexw(upcase(remark_text),'PASSPORT') then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,'PASSPORT','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
			identification_id_text=tranwrd(identification_id_text,'DIPLOMATIC','');
			identification_id_text=tranwrd(identification_id_text,'BOOKLET','');
			identification_id_text=tranwrd(identification_id_text,'#','');
            identification_id=kscan(identification_id_text,1,' ');
			if klength(identification_id) le 4 then 
			   identification_id=kscan(identification_id_text,1,' ')||" "||kscan(identification_id_text,2,' ');
			location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start and location_name_end then do;
                  citizenship_country_name=ksubstr(identification_id_text,location_name_start+1,
                                                   location_name_end-location_name_start -1);
				  output STG_WTCH.wl_ofac_transform_07;
			end;
            if not missing(identification_id) then do;
               identification_type_desc='PASSPORT';
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='PASSPORT' */

         /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=CEDULA */
         else if indexw(upcase(remark_text),'CEDULA') then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,'CEDULA','');
            identification_id_text=tranwrd(identification_id_text,'NO.','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            identification_id=kscan(identification_id_text,1,' ');
			if klength(identification_id) le 4 then 
			   identification_id=kscan(identification_id_text,1,' ')||" "||kscan(identification_id_text,2,' ');
			location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start and location_name_end then do;
                  citizenship_country_name=ksubstr(identification_id_text,location_name_start+1,
                                                   location_name_end-location_name_start -1);
				  output STG_WTCH.wl_ofac_transform_07;
			end;
            if not missing(identification_id) then do;
               identification_type_desc='CEDULA';
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='CEDULA' */

		           /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=C.U.I.P */
          else if indexw(upcase(remark_text),'C.U.I.P.') then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,'C.U.I.P.','');
            identification_id_text=tranwrd(identification_id_text,'NO.','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
			location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc='C.U.I.P. '||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='C.U.I.P.' */

		 /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=C.U.R.P */
          else if indexw(upcase(remark_text),'C.U.R.P.') then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,'C.U.R.P.','');
            identification_id_text=tranwrd(identification_id_text,'#','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc='C.U.R.P. '||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='C.U.R.P.' */

		  /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=DRIVER'S LICENSE NO. */
          else if indexw(upcase(remark_text),"DRIVER'S LICENSE") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"DRIVER'S LICENSE NO.",'');
            identification_id_text=tranwrd(identification_id_text,'NO.','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="DRIVER'S LICENSE NO. "||location_text;
			   output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc="DRIVER'S LICENSE NO. " */

		  /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=IDENTIFICATION NUMBER */
          else if indexw(upcase(remark_text),"IDENTIFICATION NUMBER") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"IDENTIFICATION NUMBER",'');
            identification_id_text=tranwrd(identification_id_text,'NO.','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="IDENTIFICATION NUMBER "||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='IDENTIFICATION NUMBER' */

		 /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=NATIONAL ID. */
          else if indexw(upcase(remark_text),"NATIONAL") and indexw(upcase(remark_text),"ID") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"NATIONAL ID",'');
			identification_id_text=tranwrd(identification_id_text,"NATIONAL FOREIGN ID",'');
            identification_id_text=tranwrd(identification_id_text,'NO.','');
			 identification_id_text=tranwrd(identification_id_text,'NUMBER.','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
			if klength(identification_id) le 4 then 
			   identification_id=kscan(identification_id_text,1,' ')||" "||kscan(identification_id_text,2,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="NATIONAL ID "||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='National Id' */

		 /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=D.N.I. */
          else if indexw(upcase(remark_text),"D.N.I.") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"D.N.I.",'');
            identification_id_text=tranwrd(identification_id_text,'NO.','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="D.N.I. "||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='D.N.I.' */

		 /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=ELECTORAL REGISTRY NO. */
          else if indexw(upcase(remark_text),"ELECTORAL REGISTRY") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"ELECTORAL REGISTRY",'');
            identification_id_text=tranwrd(identification_id_text,'NO.','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="ELECTORAL REGISTRY NO. "||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='Electroal Registry No.' */

		  /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=CREDENTIAL ELECTORAL */
          else if indexw(upcase(remark_text),"CREDENTIAL ELECTORAL") or indexw(upcase(remark_text),"CREDENCIAL ELECTORAL")  then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"CREDENTIAL ELECTORAL",'');
			identification_id_text=tranwrd(identification_id_text,"CREDENCIAL ELECTORAL",'');
            identification_id_text=tranwrd(identification_id_text,'NO.','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="CREDENTIAL ELECTORAL NO. "||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='Credencial Electoral No.' */

		  /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=ITALIAN FISCAL CODE */
          else if indexw(upcase(remark_text),"ITALIAN FISCAL ") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"ITALIAN FISCAL CODE",'');
			identification_id_text=tranwrd(identification_id_text,":",'');
            identification_id_text=tranwrd(identification_id_text,'NO.','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="ITALIAN FISCAL CODE";
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='Italian Fiscal Code No.' */

		  /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=LE NUMBER */
          else if indexw(upcase(remark_text),"LE NUMBER") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"LE NUMBER",'');
            identification_id_text=tranwrd(identification_id_text,'NO.','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="LE NUMBER. "||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='LE Number' */

		 /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=NIT */
          else if indexw(upcase(remark_text),"NIT") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"NIT",'');
            identification_id_text=tranwrd(identification_id_text,'NO.','');
			 identification_id_text=tranwrd(identification_id_text,'#','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="NIT # "||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='NIT #' */

		  /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=RFC */
          else if indexw(upcase(remark_text),"RFC") or indexw(upcase(remark_text),"R.F.C.") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"RFC",'');
			identification_id_text=tranwrd(identification_id_text,"R.F.C.",'');
            identification_id_text=tranwrd(identification_id_text,'NO.','');
			 identification_id_text=tranwrd(identification_id_text,'#','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="R.F.C. NUMBER "||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='RFC' */

		   /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=NUMERO DE IDENTIDAD */  
          else if indexw(upcase(remark_text),"NUMERO DE IDENTIDAD") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"NUMERO DE IDENTIDAD",'');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="NUMERO DE IDENTIDAD "||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='NUMERO DE IDENTIDAD' */

		  /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=PERSONAL ID CARD */
          else if indexw(upcase(remark_text),"PERSONAL ID CARD") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"PERSONAL ID CARD",'');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="PERSONAL ID CARD "||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='PERSONAL ID CARD' */
		 
		  /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=REGISTRATION ID */
          else if indexw(upcase(remark_text),"REGISTRATION ID") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"REGISTRATION ID",'');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="REGISTRATION ID "||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='REGISTRATION ID' */

		  /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=RESIDENCY NUMBER */
          else if indexw(upcase(remark_text),"RESIDENCY NUMBER") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"RESIDENCY NUMBER",'');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="RESIDENCY NUMBER "||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='RESIDENCY NUMBER' */

		 /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=TRAVEL DOCUMENT NUMBER */
          else if indexw(upcase(remark_text),"TRAVEL DOCUMENT NUMBER") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"TRAVEL DOCUMENT NUMBER",'');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="TRAVEL DOCUMENT NUMBER "||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='TRAVEL DOCUMENT NUMBER' */

		 /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC= IDENTIFICATION NUMBER */
          else if indexw(upcase(remark_text),"IDENTIFICATION NUMBER") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"IDENTIFICATION NUMBER",'');
			identification_id_text=tranwrd(identification_id_text,"TURKISH",'');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="IDENTIFICATION NUMBER "||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='IDENTIFICATION NUMBER' */

		  /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC= TURKISH IDENTIFICATION NUMBER */
          else if indexw(upcase(remark_text),"IDENTIFICIATION NUMBER") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"IDENTIFICIATION NUMBER",'');
			identification_id_text=tranwrd(identification_id_text,"TURKISH",'');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="IDENTIFICATION NUMBER "||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='IDENTIFICATION NUMBER' */

		 /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC= CARTILLA DE SERVICIO MILITAR NACIONAL  */
          else if indexw(upcase(remark_text),"CARTILLA DE SERVICIO MILITAR NACIONAL") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"CARTILLA DE SERVICIO MILITAR NACIONAL",'');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="CARTILLA DE SERVICIO MILITAR NACIONAL "||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='CARTILLA DE SERVICIO MILITAR NACIONAL' */

		  /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC= MATRICULA MERCANTIL NO  */
          else if indexw(upcase(remark_text),"MATRICULA MERCANTIL NO ") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"MATRICULA MERCANTIL NO ",'');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="MATRICULA MERCANTIL NO  "||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='MATRICULA MERCANTIL NO ' */

		  /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=RUC # */

		  else if indexw(upcase(remark_text),'RUC') then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,'RUC','');
            identification_id_text=tranwrd(identification_id_text,'#','');
			identification_id_text=tranwrd(identification_id_text,'NUMBER','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
			location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc='RUC # '||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='RUC #' */

		  /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=TAX ID NO */

		  else if indexw(upcase(remark_text),'TAX ID') then do; 
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,'TAX','');
            identification_id_text=tranwrd(identification_id_text,'ID','');
			identification_id_text=tranwrd(identification_id_text,'NO.','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
			location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc='TAX ID NO '||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='TAX ID NO' */


		  /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC= VISANUMBERID */
          else if indexw(upcase(remark_text),"VISANUMBERID") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"VISANUMBERID",'');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="VISANUMBERID"||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='MATRICULA MERCANTIL NO ' */
         /* DATE_OF_BIRTH
            YEAR_OF_BIRTH */
         else if indexw(remark_text,'DOB') then do;
            date_of_birth_text=kstrip(upcase(remark_text));
            date_of_birth_text=tranwrd(date_of_birth_text,'DOB','');
            date_of_birth_text=tranwrd(date_of_birth_text,'ALT.','');
            date_of_birth_text=tranwrd(date_of_birth_text,'CIRCA','');
            date_of_birth_text=tranwrd(date_of_birth_text,'C.','');
            date_of_birth_text=kcompress(kcompress(date_of_birth_text,'.'));

            /* OFAC DOB appears to be in YYYY, DDMMMYYY or MMMYYYY format */

            /* Only year_of_birth available */
            if (lengthn(date_of_birth_text) eq 4 and
               anyalpha(date_of_birth_text) eq 0) then do;
               date_of_birth=.;
               year_of_birth=input(kstrip(date_of_birth_text),best32.);
               output STG_WTCH.wl_ofac_transform_06;
            end;  /* Only year_of_birth available */
            else do;
               /* Standardize month portion of OFAC DOB's */
               /* to 3 digits (i.e., JANUARY=JAN)         */
               date_of_birth_text=tranwrd(date_of_birth_text,'JANUARY','JAN');
               date_of_birth_text=tranwrd(date_of_birth_text,'FEBRUARY','FEB');
               date_of_birth_text=tranwrd(date_of_birth_text,'MARCH','MAR');
               date_of_birth_text=tranwrd(date_of_birth_text,'APRIL','APR');
               date_of_birth_text=tranwrd(date_of_birth_text,'MAY','MAY');
               date_of_birth_text=tranwrd(date_of_birth_text,'JUNE','JUN');
               date_of_birth_text=tranwrd(date_of_birth_text,'JULY','JUL');
               date_of_birth_text=tranwrd(date_of_birth_text,'AUGUST','AUG');
               date_of_birth_text=tranwrd(date_of_birth_text,'SEPTEMBER','SEP');
               date_of_birth_text=tranwrd(date_of_birth_text,'SEPT','SEP');
               date_of_birth_text=tranwrd(date_of_birth_text,'OCTOBER','OCT');
               date_of_birth_text=tranwrd(date_of_birth_text,'NOVEMBER','NOV');
               date_of_birth_text=tranwrd(date_of_birth_text,'DECEMBER','DEC');

               if index(date_of_birth_text,'JAN') or
                  index(date_of_birth_text,'FEB') or
                  index(date_of_birth_text,'MAR') or
                  index(date_of_birth_text,'APR') or
                  index(date_of_birth_text,'MAY') or
                  index(date_of_birth_text,'JUN') or
                  index(date_of_birth_text,'JUL') or
                  index(date_of_birth_text,'AUG') or
                  index(date_of_birth_text,'SEP') or
                  index(date_of_birth_text,'OCT') or
                  index(date_of_birth_text,'NOV') or
                  index(date_of_birth_text,'DEC') then do;

                  date_of_birth=input(date_of_birth_text,??date9.);

                  /* Complete date converted correctly */
                  if not missing(date_of_birth) then do;
                     date_of_birth=dhms(date_of_birth,00,00,00);
                     year_of_birth=year(datepart(date_of_birth));
                     output STG_WTCH.wl_ofac_transform_06;
                  end;
               end;  /* end if index */
               else do;
                  /* Value could be a range of years eg. 1973TO1974  */
				  /*                                 eg. 1973-1974   */
				  /*       or month and year MMMYYYY, eg. MAY1937    */
                  check_range=kindex(upcase(date_of_birth_text),"TO");
				  if check_range > 0 then do;  /* Range using "TO" */
				     first_year=input(ksubstr(date_of_birth_text,1,check_range-1),?? 8.);
                     last_year=input(ksubstr(date_of_birth_text,check_range+2),?? 8.);
					 do year_of_birth=first_year to last_year;
					    date_of_birth=.;
						output STG_WTCH.wl_ofac_transform_06;
					 end;
				  end; /* Range using "TO" */
				  /* Check for date range with -  eg. 1973-1975 */
				  else do;
					 check_range=kindex(date_of_birth_text,"-");
				     if check_range > 0 then do;  /* Range using "-" */
						   first_year=input(ksubstr(date_of_birth_text,1,check_range-1),?? 8.);
						   last_year=input(ksubstr(date_of_birth_text,check_range+1),?? 8.);
						   do year_of_birth=first_year to last_year;
						      date_of_birth=.;
							  output STG_WTCH.wl_ofac_transform_06;
						   end;
					 end; /* Range using "-" */
				
					 else do;
                                /* WARNING: Invalid date of birth.                     */
                                entity_watch_list_number=kstrip(put(ent_num,best32.));
                                watch_list_name='OFAC';
                                error_file="&ofac_sdn_file";
                                error_record_number=_N_;
                                error_field='REMARKS';
                                error_value=kstrip(remark_text);
                                error_message='WARNING: Invalid date of birth.';
                                error_datetime=datetime();

                                putlog 'WARNING: FCF: Invalid date of birth. ' ent_num= date_of_birth_text=;
                                error_flag = 1;

                                output STG_ERR.wl_ofac_errors_01;
                      end;  /* Warning: Invalid date of birth */
					end; /* else do for date range */
				  end; /* else do for index */
				end; /* else do only birth year */                 
			 end;/* else do DOB */
        
         /* PLACE_OF_BIRTH */
         else if indexw(upcase(remark_text),'POB') then do;
            place_of_birth_text=upcase(remark_text);
            place_of_birth_text=tranwrd(place_of_birth_text,'POB','');
            place_of_birth_text=tranwrd(place_of_birth_text,'ALT.','');
            place_of_birth=kstrip(compbl(kcompress(place_of_birth_text,'.')));
            output STG_WTCH.wl_ofac_transform_06;
         end;

         /* CITIZENSHIP_COUNTRY_NAME */
         else if indexw(upcase(remark_text),'CITIZEN') then do;
            citizenship_country_name_text=upcase(remark_text);
            citizenship_country_name_text=tranwrd(citizenship_country_name_text,'CITIZEN','');
            citizenship_country_name_text=tranwrd(citizenship_country_name_text,'ALT.','');
            citizenship_country_name=kstrip(compbl(kcompress(citizenship_country_name_text,'.')));
            output STG_WTCH.wl_ofac_transform_07;
         end;

		 /* NATIONALITY_COUNTRY_NAME */
         else if indexw(upcase(remark_text),'NATIONALITY') then do;
            nationality_country_name_text=upcase(remark_text);
            nationality_country_name_text=tranwrd(nationality_country_name_text,'NATIONALITY','');
            nationality_country_name_text=tranwrd(nationality_country_name_text,'ALT.','');
            nationality_country_name=kstrip(compbl(kcompress(nationality_country_name_text,'.')));
            output STG_WTCH.wl_ofac_transform_07;
         end;

      end; /* end: sdn_type='INDIVIDUAL' */


      /* sdn_type=ORGANIZATION */
      /* TAX_ID
         TAX_ID_TYPE_CODE=E */
      if sdn_type eq '-0-' then do;
         if indexw(upcase(remark_text),'FEIN') then do;
            tax_id_text=upcase(remark_text);
            tax_id_text=tranwrd(tax_id_text,'US','');
            tax_id_text=tranwrd(tax_id_text,'FEIN','');
            tax_id_text=tranwrd(tax_id_text,'ALT.','');
            tax_id_text=tranwrd(tax_id_text,'UNITED','');
            tax_id_text=tranwrd(tax_id_text,'STATES','');
            tax_id_text=kcompress(kscan(kcompress(tax_id_text,' '),1,' '),'().');
            tax_id=tax_id_text;
            if not missing(tax_id) then do;
               tax_id_type_code='E';
               output STG_WTCH.wl_ofac_transform_04;
            end;
         end;  /* end: tax_id / tax_id_type_code='E' */
/*      end;*/
	  /* end: tax_id / tax_id_type_code='E' */

		 /* IDENTIFICATION_ID
	        IDENTIFICATION_TYPE_DESC=REGISTRATION ID */

          else if indexw(upcase(remark_text),"REGISTRATION ID") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"REGISTRATION ID",'');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="REGISTRATION ID "||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='REGISTRATION ID' */

         /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=COMMERCIAL REGISTRY NUMBER */

          else if indexw(upcase(remark_text),'COMMERCIAL REGISTRY NUMBER') then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,'COMMERCIAL','');
            identification_id_text=tranwrd(identification_id_text,'REGISTRY','');
			identification_id_text=tranwrd(identification_id_text,'DOCUMENT','');
			identification_id_text=tranwrd(identification_id_text,'NUMBER','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
			location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc='COMMERCIAL REGISTRY NUMBER '||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='COMMERCIAL REGISTRY NUMBER' */

         /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=BUSINESS REGISTRATION DOCUMENT */

          else if indexw(upcase(remark_text),'BUSINESS REGISTRATION DOCUMENT') then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,'BUSINESS','');
            identification_id_text=tranwrd(identification_id_text,'REGISTRATION','');
			identification_id_text=tranwrd(identification_id_text,'DOCUMENT','');
			identification_id_text=tranwrd(identification_id_text,'#','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
			location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc='BUSINESS REGISTRATION DOCUMENT '||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='BUSINESS REGISTRATION DOCUMENT' */

		 
		  /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC= MATRICULA MERCANTIL NO  */
          else if indexw(upcase(remark_text),"MATRICULA MERCANTIL NO ") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"MATRICULA MERCANTIL NO ",'');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="MATRICULA MERCANTIL NO  "||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='MATRICULA MERCANTIL NO ' */


		 /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=NIT */
          else if indexw(upcase(remark_text),"NIT") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"NIT",'');
            identification_id_text=tranwrd(identification_id_text,'NO.','');
			 identification_id_text=tranwrd(identification_id_text,'#','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="NIT # "||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='NIT #' */

		 /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=FOLIO MERCANTIL NO */

		  else if indexw(upcase(remark_text),'FOLIO MERCANTIL') then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,'FOLIO','');
            identification_id_text=tranwrd(identification_id_text,'MERCANTIL','');
			identification_id_text=tranwrd(identification_id_text,'NO.','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
			location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc='FOLIO MERCANTIL NO '||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='FOLIO MERCANTIL NO' */

         /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=GOVERNMENT GAZETTE */

		  else if indexw(upcase(remark_text),'GOVERNMENT GAZETTE') then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,'GOVERNMENT','');
            identification_id_text=tranwrd(identification_id_text,'GAZETTE','');
			identification_id_text=tranwrd(identification_id_text,'NUMBER','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
			location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc='GOVERNMENT GAZETTE NUMBER '||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='GOVERNMENT GAZETTE NUMBER' */

		 /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=RUC # */

		  else if indexw(upcase(remark_text),'RUC') then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,'RUC','');
            identification_id_text=tranwrd(identification_id_text,'#','');
			identification_id_text=tranwrd(identification_id_text,'NUMBER','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
			location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc='RUC # '||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='RUC #' */

		  /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=TAX ID NO */

		  else if indexw(upcase(remark_text),'TAX ID') then do; 
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,'TAX','');
            identification_id_text=tranwrd(identification_id_text,'ID','');
			identification_id_text=tranwrd(identification_id_text,'NO.','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
			location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc='TAX ID NO '||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='TAX ID NO' */

		  /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=CIF */

		  else if indexw(upcase(remark_text),'C.I.F.') then do; 
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,'C.I.F.','');
            identification_id_text=tranwrd(identification_id_text,'ID','');
			identification_id_text=tranwrd(identification_id_text,'NO','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
			location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc='C.I.F. NO '||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='C.I.F. NO' */

		 	  /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=REGISTERED CHARITY NUMBER */

		  else if indexw(upcase(remark_text),'REGISTERED CHARITY') then do; 
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,'REGISTERED','');
            identification_id_text=tranwrd(identification_id_text,'CHARITY','');
			identification_id_text=tranwrd(identification_id_text,'NO.','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
			location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc='REGISTERED CHARITY NO '||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='REGISTERED CHARITY NO' */

		 /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=COMPANY NUMBER */

		  else if indexw(upcase(remark_text),'COMPANY NUMBER') then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,'UK','');
			identification_id_text=tranwrd(identification_id_text,'COMPANY','');
			identification_id_text=tranwrd(identification_id_text,'NUMBER','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
			location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc='COMPANY NUMBER '||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='COMPANY NUMBER' */

		 /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=IDENTIFICATION NUMBER */
          else if indexw(upcase(remark_text),"IDENTIFICATION NUMBER") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"IDENTIFICATION NUMBER",'');
            identification_id_text=tranwrd(identification_id_text,'NO.','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="IDENTIFICATION NUMBER "||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='IDENTIFICATION NUMBER' */

		  /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=CERTIFICATE OF INCORPORATION */

		  else if indexw(upcase(remark_text),'CERTIFICATE OF INCORPORATION') then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,'CERTIFICATE','');
			identification_id_text=tranwrd(identification_id_text,'OF','');
			identification_id_text=tranwrd(identification_id_text,'INCORPORATION','');
			identification_id_text=tranwrd(identification_id_text,'NUMBER','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
			location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc='CERTIFICATE OF INCORPORATION NUMBER '||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='CERTIFICATE OF INCORPORATION NUMBER' */

		 /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=ITALIAN FISCAL CODE */
          else if indexw(upcase(remark_text),"ITALIAN FISCAL ") then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,"ITALIAN FISCAL CODE",'');
			identification_id_text=tranwrd(identification_id_text,":",'');
            identification_id_text=tranwrd(identification_id_text,'NO.','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
            identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc="ITALIAN FISCAL CODE";
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='Italian Fiscal Code No.' */

		 /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=R.F.C. */
		  else if indexw(upcase(remark_text),'R.F.C.') OR  indexw(upcase(remark_text),'RFC') then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,'R.F.C.','');
			 identification_id_text=tranwrd(identification_id_text,'RFC','');
			identification_id_text=tranwrd(identification_id_text,'#','');
			identification_id_text=tranwrd(identification_id_text,'NUMBER','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
			location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc='R.F.C. NUMBER '||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='R.F.C. NUMBER' */

		 /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=NATIONAL ID */

		  else if indexw(upcase(remark_text),'NATIONAL ID')  then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,'NATIONAL','');
			identification_id_text=tranwrd(identification_id_text,'ID','');
			identification_id_text=tranwrd(identification_id_text,'#','');
			identification_id_text=tranwrd(identification_id_text,'NUMBER','');
			identification_id_text=tranwrd(identification_id_text,'NO.','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
			location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc='NATIONAL ID '||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='NATIONAL ID NUMBER' */

		 /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=C.R. NO */

		  else if indexw(upcase(remark_text),'C.R.')  then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,'C.R.','');
			identification_id_text=tranwrd(identification_id_text,'NO.','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
			location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc='C.R. NO '||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='C.R. NO' */

		 /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=RTN */

		  else if indexw(upcase(remark_text),'RTN')  then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,'RTN','');
			identification_id_text=tranwrd(identification_id_text,'NO','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
			location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc='RTN '||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='RTN' */

		 /* IDENTIFICATION_ID
            IDENTIFICATION_TYPE_DESC=V.A.T. NUMBER */

		   else if indexw(upcase(remark_text),'V.A.T. NUMBER') then do;
            identification_id_text=upcase(remark_text);
            identification_id_text=tranwrd(identification_id_text,'V.A.T.','');
            identification_id_text=tranwrd(identification_id_text,'COMPANY','');
			identification_id_text=tranwrd(identification_id_text,'NUMBER','');
            identification_id_text=tranwrd(identification_id_text,'ALT.','');
			location_name_start=kindex(identification_id_text,"(");
			location_name_end=kindex(identification_id_text,")");
			if location_name_start then do;
			   identification_id=ksubstr(identification_id_text,1,location_name_start-1);
			   if location_name_end then 
                  location_text=ksubstr(identification_id_text,location_name_start,location_name_end-location_name_start+1);
			end;
            else identification_id=kscan(identification_id_text,1,' ');
			if length(identification_id) < 3 then identification_id=kscan(identification_id_text,2,' ');
            if not missing(identification_id) then do;
			   identification_id=kleft(identification_id);
               identification_type_desc='V.A.T. NUMBER '||location_text;
               output STG_WTCH.wl_ofac_transform_05;
            end;
         end; /* end: identification_id / identification_type_desc='V.A.T. NUMBER' */
      end; /* origanization */
   end; /* end: process remarks field */

   if (eof) and (error_flag) then do;
      putlog ' ';
      putlog 'WARNING: FCF: Invalid values were encountered processing the watch list data';
      putlog 'WARNING: FCF: Data values were written to the STG_ERR.WL_OFAC_ERRORS table.';
   end;

/* Set all columns in if set 1=0 column to missing */
   call missing(&col_name_list);
run;

/* Remove potential duplicate entries */
proc sort data=STG_WTCH.wl_ofac_transform_04(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_04))
          out=STG_WTCH.wl_ofac_transform_04(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_04, output_ds=Y))
          nodupkey;
by ent_num
   tax_id
   tax_id_type_code;
run;

proc sort data=STG_WTCH.wl_ofac_transform_05(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_05))
          out=STG_WTCH.wl_ofac_transform_05(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_05, output_ds=Y))
          nodupkey;
by ent_num
   identification_id
   identification_type_desc;
run;

proc sort data=STG_WTCH.wl_ofac_transform_06(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_06))
          out=STG_WTCH.wl_ofac_transform_06(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_06, output_ds=Y))
          nodupkey;
by ent_num
   date_of_birth
   year_of_birth
   place_of_birth;
run;

proc sort data=STG_WTCH.wl_ofac_transform_07(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_07))
          out=STG_WTCH.wl_ofac_transform_07(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_07, output_ds=Y))
          nodupkey;
by ent_num
   citizenship_country_name
   nationality_country_name;
run;
/******************************************************************************/
/* END:   STG_WTCH.wl_ofac_transform_04                                       */
/*        STG_WTCH.wl_ofac_transform_05                                       */
/*        STG_WTCH.wl_ofac_transform_06                                       */
/*        STG_WTCH.wl_ofac_transform_07                                       */
/*        STG_WTCH.wl_ofac_errors_01                                          */
/******************************************************************************/

/******************************************************************************/
/* BEGIN: STG_WTCH.wl_ofac_transform_08                                       */
/*        STG_WTCH.wl_ofac_transform_09                                       */
/*        STG_WTCH.wl_ofac_transform_10                                       */
/*        STG_WTCH.wl_ofac_transform_11                                       */
/*        STG_WTCH.wl_ofac_transform_12                                       */
/*        STG_WTCH.wl_ofac_transform_13                                       */
/*                                                                            */
/* DESCRIPTION: The final output table 'STG_WTCH.wl_ofac_transform' from this */
/* step contains all possible data permutations for each unique ENT_NUM.      */
/******************************************************************************/

proc sql noprint;

   create table STG_WTCH.wl_ofac_transform_08 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_08, output_ds=Y, add_parens=Y)  as
      select a.*,
             b.tax_id,
             b.tax_id_type_code
      from STG_WTCH.wl_ofac_transform_01(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_ofac_transform_01)) as a full join
           STG_WTCH.wl_ofac_transform_04(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_ofac_transform_04)) as b
      on a.ent_num=b.ent_num;

   create table STG_WTCH.wl_ofac_transform_09 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_09, output_ds=Y, add_parens=Y)  as
      select a.*,
             b.identification_id,
             b.identification_type_desc
      from STG_WTCH.wl_ofac_transform_08(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_ofac_transform_08)) as a full join
           STG_WTCH.wl_ofac_transform_05(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_ofac_transform_05)) as b
      on a.ent_num=b.ent_num;

   create table STG_WTCH.wl_ofac_transform_10 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_10, output_ds=Y, add_parens=Y)  as
      select a.*,
             b.date_of_birth,
             b.year_of_birth,
             b.place_of_birth
      from STG_WTCH.wl_ofac_transform_09(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_ofac_transform_09)) as a full join
           STG_WTCH.wl_ofac_transform_06(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_ofac_transform_06)) as b
      on a.ent_num=b.ent_num;

   create table STG_WTCH.wl_ofac_transform_11 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_11, output_ds=Y, add_parens=Y)  as
      select a.*,
             b.citizenship_country_name,
             b.nationality_country_name
      from STG_WTCH.wl_ofac_transform_10(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_ofac_transform_10)) as a full join
           STG_WTCH.wl_ofac_transform_07(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_ofac_transform_07)) as b
      on a.ent_num=b.ent_num;

   create table STG_WTCH.wl_ofac_transform_12 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_12, output_ds=Y, add_parens=Y)  as
      select a.*,
             b.add_num,
             b.address,
             b.city,
             b.country,
             b.add_remarks
      from STG_WTCH.wl_ofac_transform_11(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_ofac_transform_11)) as a full join
           STG_WTCH.wl_ofac_transform_02(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_ofac_transform_02)) as b
      on a.ent_num=b.ent_num;

   create table STG_WTCH.wl_ofac_transform_13 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_13, output_ds=Y, add_parens=Y)  as
      select a.*,
             b.alt_num,
             b.alt_type,
             b.alt_name,
             b.alt_remarks
      from STG_WTCH.wl_ofac_transform_12(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_ofac_transform_12)) as a full join
           STG_WTCH.wl_ofac_transform_03(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_ofac_transform_03)) as b
      on a.ent_num=b.ent_num;

quit;
/******************************************************************************/
/* END: STG_WTCH.wl_ofac_transform_08                                         */
/*      STG_WTCH.wl_ofac_transform_09                                         */
/*      STG_WTCH.wl_ofac_transform_10                                         */
/*      STG_WTCH.wl_ofac_transform_11                                         */
/*      STG_WTCH.wl_ofac_transform_12                                         */
/*      STG_WTCH.wl_ofac_transform_13                                         */
/******************************************************************************/

/******************************************************************************/
/* BEGIN: STG_WTCH.wl_ofac_transform_14                                       */
/* DESCRIPTION: Populate SDN_NAME field with ALT_NAME value if it exists.     */
/******************************************************************************/
data STG_WTCH.wl_ofac_transform_14(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_14, output_ds=Y) drop=alt_name);
   set STG_WTCH.wl_ofac_transform_13(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_13) drop=sdn_name);
   if alt_name ne '' then do;
      sdn_name=alt_name;
      output;
   end;
run;
/******************************************************************************/
/* END: STG_WTCH.wl_ofac_transform_14                                         */
/******************************************************************************/

/******************************************************************************/
/* BEGIN: STG_WTCH.wl_ofac_transform_15                                       */
/* DESCRIPTION: Append all final data permutations.                           */
/******************************************************************************/
proc append base=STG_WTCH.wl_ofac_transform_15(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_15, output_ds=Y))
            data=STG_WTCH.wl_ofac_transform_14(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_14))
            force;
run;

proc append base=STG_WTCH.wl_ofac_transform_15(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_15, output_ds=Y) drop=alt_num alt_type alt_remarks)
            data=STG_WTCH.wl_ofac_transform_12(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform_12))
            force;
run;
/******************************************************************************/
/* END: STG_WTCH.wl_ofac_transform_15                                         */
/* DESCRIPTION: Append all final data permutations.                           */
/******************************************************************************/

/******************************************************************************/
/* BEGIN: STG_WTCH.wl_ofac_transform                                          */
/*        STG_ERR.wl_ofac_errors_02                                           */
/* DESCRIPTION: Output table 'STG_WTCH.wl_ofac_transform' contains all final  */
/*              data transformations.                                         */
/******************************************************************************/
data

  STG_WTCH.wl_ofac_transform(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_ofac_transform, output_ds=Y)
                             keep=
     entity_watch_list_key
     entity_watch_list_number
     watch_list_name
     category_desc
     type_desc
     programs
     tax_id
     tax_id_type_code
     identification_id
     identification_type_desc
     date_of_birth
     year_of_birth
     place_of_birth
     deceased_ind
     first_name
     last_name
     middle_name
     entity_name
     match_code_individual
     match_code_organization
     entity_title
     occupation_desc
     address
     city_name
     state_name
     postal_code
     country_code
     country_name
     full_address
     match_code_full_address
     citizenship_country_code
     citizenship_country_name
     match_code_citizenship
     nationality_country_code
     nationality_country_name
     match_code_nationality
     org_country_of_business_code
     org_country_of_business_name
     match_code_org_country
     politically_exposed_person_ind
     create_date
     update_date
     exclude_ind
     change_begin_date
     change_end_date
     change_current_ind
  )

  STG_ERR.wl_ofac_errors_02(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_ofac_errors_02, output_ds=Y)
                            keep=
     entity_watch_list_number
     watch_list_name
     error_file
     error_record_number
     error_field
     error_value
     error_message
     error_datetime
  );

    /* FSC_ENTITY_WATCH_LIST_DIM */
    if 1 eq 0 then
      set seg_kc.FSC_ENTITY_WATCH_LIST_DIM;

    /* ERROR FIELDS */
    attrib
      error_file          length=$200
                          informat=$200.
                          format=$200.
                          label='Error File'

      error_record_number length=8
                          informat=best32.
                          format=best32.
                          label='Error Record Number'

      error_field         length=$32
                          informat=$32.
                          format=$32.
                          label='Error Field'

      error_value         length=$1000
                          informat=$1000.
                          format=$1000.
                          label='Error Value'

      error_message       length=$200
                          informat=$200.
                          format=$200.
                          label='Error Message'

      error_datetime      length=8
                          informat=nldatm.
                          format=nldatm.
                          label='Error Datetime'
;

    set STG_WTCH.wl_ofac_transform_15(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_ofac_transform_15) rename=(address=ofac_address)) end=eof;


/******************************************************************************/
/* NOTE:                                                                      */
/*                                                                            */
/* Initial values for the fields listed below are set to missing as these     */
/* fields values are set by later processes.                                  */
/******************************************************************************/

    /* Fields set by load */
    entity_watch_list_key=.;
    change_begin_date=.;
    change_end_date=.;
    change_current_ind='';

    retain error_flag 0;

/* ************************************************************************ */
/* NOTE:                                                                    */
/*                                                                          */
/* The following fields are set to null as this watch list extract does not */
/* currently include this data.                                             */
/* ************************************************************************ */
    deceased_ind='';
    middle_name='';
    org_country_of_business_name='';
    politically_exposed_person_ind='';
    create_date=.;
    update_date=.;

    /* initialize values */
    state_name='';
    postal_code='';

/* ************************************************************************ */
/* NOTE:                                                                    */
/*                                                                          */
/* The following fields are set to null due to the complexity of required   */
/* transformations. This does not preclude users from writing               */
/* transformation code which could populate these fields.                   */
/* ************************************************************************ */
    occupation_desc=''; /* SDN - REMARKS field */

    /* ENTITY_WATCH_LIST_NUMBER */
    /* NOTE: watch_list_number is composed of ent_num, add_num and alt_num */
    if not missing(alt_num) then
      entity_watch_list_number=kstrip(put(ent_num,best32.)) || '-' ||
                               kstrip(put(add_num,best32.)) || '-' ||
                               kstrip(put(alt_num,best32.));
    else if missing(alt_num) then
      entity_watch_list_number=kstrip(put(ent_num,best32.)) || '-' ||
                               kstrip(put(add_num,best32.)) || '-0';

    /* WATCH_LIST_NAME */
    watch_list_name='OFAC';

    /* CATEGORY_DESC */
    if index(upcase(program),'SDGT') or
      index(upcase(program),'SDT') then
      category_desc='TERRORISM';

    /* ENTITY_TITLE */
    if title eq '-0-' then
      entity_title='';
    else
      entity_title=compbl(kstrip(title));

    /* TYPE_DESC */
    if upcase(sdn_type) eq 'INDIVIDUAL' then
      type_desc='INDIVIDUAL';
    else if sdn_type eq '-0-' then
      type_desc='ORGANIZATION';

    /* ERROR: Unknown SDN_TYPE */
    else if upcase(sdn_type) not in('-0-','INDIVIDUAL') then do;

      error_file="&ofac_sdn_file";
      error_record_number=_N_;
      error_field='SDN_TYPE';
      error_value=kstrip(sdn_type);
      error_message='ERROR: Unknown SDN_TYPE.';
      error_datetime=datetime();

      putlog 'WARNING: FCF: Unknown SDN_TYPE.' SDN_TYPE=;
      error_flag = 1;

    end;

    /* PROGRAMS */
    programs=upcase(program);

    /* type_desc='INDIVIDUAL' */
    if type_desc eq 'INDIVIDUAL' then do;
      if countc(sdn_name,',') eq 1 then do;

         /* FIRST_NAME */
         first_name=upcase(compbl(kstrip(kscan(sdn_name,2,','))));

         /* LAST_NAME */
         last_name=upcase(compbl(kstrip(kscan(sdn_name,1,','))));

         /* ENTITY_NAME */
         entity_name=catx(' ',first_name,last_name);

      end;

      /* WARNING: Cannot determine first/last name. */
      else if countc(sdn_name,',') ne 1 then do;

         first_name='';
         last_name='';
         entity_name=upcase(compbl(kstrip(sdn_name)));

         error_file="&ofac_sdn_file";
         error_record_number=_N_;
         error_field='SDN_NAME';
         error_value=kstrip(sdn_name);
         error_message='WARNING: Cannot determine first/last name.';
         error_datetime=datetime();

         putlog 'WARNING: FCF: Cannot determine first/last name. ' sdn_name=;
         error_flag = 1;

      end;
    end; /* type desc='INDIVIDUAL' */

    /* type_desc='ORGANIZATION' */
    else if type_desc eq 'ORGANIZATION' then do;

      /* ENTITY_NAME */
      entity_name=upcase(compbl(kstrip(sdn_name)));

/* ************************************************************************ */
/* NOTE: The following fields are not applicable where                      */
/*       type_desc='ORGANIZATION'. These fields are therefore set to null.  */
/* ************************************************************************ */
      date_of_birth=.;
      year_of_birth=.;
      first_name='';
      last_name='';
      middle_name='';

    end;  /* type_desc='ORGANIZATION' */

    /* ADDRESS */
    if ofac_address ne '-0-' then
      address=upcase(compbl(kstrip(ofac_address)));
    else address='';

    /* CITY, STATE/PROVINCE, POSTAL CODE */
    if city ne '-0-' then do;

      /* Spec states these fields are comma separated */
      /* but it appears the postal code (i.e. zip) is */
      /* acutally separated by two spaces instead.    */
      postal_code_position = index(kstrip(city), '  ');
      if postal_code_position > 0 then do;
         postal_code = substr(city, postal_code_position + 2);
         city = substr(city, 1, postal_code_position);
      end;

      /* Find STATE/PROVINCE by searching from right of string for comma */
      find_rc = findc(city, ',', 'b');
      if find_rc > 0 then do;
         city_name = upcase(substr(city, 1, find_rc -1));
         state_name = upcase(kstrip(substr(city, find_rc+1)));
      end;
      else do;
        city_name=upcase(compbl(kstrip(city)));
        state_name = '';
      end;

    end;
    else if city eq '-0-' then do;
      city_name='';
      state_name='';
      postal_code='';
    end;

    /* COUNTRY_NAME */
    if country ne '-0-' then do;
      country_name=upcase(compbl(kstrip(country)));
    end;
    else if country eq '-0-' then do;
      country_name='';
    end;

    /* FULL_ADDRESS */
    full_address=catx(';', address, city_name, state_name, postal_code, country_name);

    /* EXCLUDE_IND */
    exclude_ind='N';

    if not missing(error_field) then
      output STG_WTCH.wl_ofac_transform
             STG_ERR.wl_ofac_errors_02;
    else
      output STG_WTCH.wl_ofac_transform;

/* Set all columns in if set 1=0 column to missing */
   call missing(&col_name_list);

    if (eof) and (error_flag) then do;
      putlog ' ';
      putlog 'WARNING: FCF: Invalid values were encountered processing the watch list data';
      putlog 'WARNING: FCF: Data values were written to the STG_ERR.WL_OFAC_ERRORS table.';
    end;

run;

/******************************************************************************/
/* BEGIN: STG_ERR.wl_ofac_errors                                              */
/* DESCRIPTION: Append all final error records.                               */
/******************************************************************************/

  proc append base=STG_ERR.wl_ofac_errors(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_ofac_errors, output_ds=Y))
              data=STG_ERR.wl_ofac_errors_01(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_ofac_errors_01))
              force;
   run;

  proc append base=STG_ERR.wl_ofac_errors(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_ofac_errors, output_ds=Y))
              data=STG_ERR.wl_ofac_errors_02(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_ofac_errors_02))
              force;
   run;

  /****************************************************************************/
  /* END: STG_ERR.wl_ofac_errors                                              */
  /****************************************************************************/

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
     %fcf_execute_macro(fcf_error);
  %else %do;
     %fcf_execute_macro(post&_fcfmodule_);
     %fcf_execute_macro(fcf_success);
  %end;

%mend fcf_wl_ofac_transform;
