/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    rc_resp_fincen_process
| PRODUCT: SAS Regulatory Console
| SYSTEM:  Windows, Unix
| DESC:    Driver program to process the response files that are   
|          from FINCEN.
|          
+------------------------------------------------------------------------------+
| USAGE:    RESPONSE_FILE_EXT=extension of the response files to be           
|                                 processed.  
|                FORM_CONFIG_RK=key of the fincen form (301) 
|                RESPONSE_TYPE_CODE=response file table (ASC for acknowledgement     
|                                 receipts.                                        
|                INFREF = File reference of the directory where the response files 
|                         can be found.                                             
|                PRINTYN=Y to generate report.
|                TESTYN = N to update SEG_KC data 
|                RENAMEYN=Y to rename response file by adding 'p' to the end of file name
|                         when it is processed  
|                SDTMYN=Y to look for response files in &rc_sdtm_resp_path_*
|                SDTMYN=N to look for respinse files in &rc_efile_resp_path
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2016-2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/



%macro rc_resp_fincen_process (      response_file_ext=
                                   , form_config_key=
                                   , response_type_code=  
                                   , printYN=Y
                                   , testYN=N
                                   , renameYN=Y
                                   , sdtmYN=N 
                                  )
;

%let trans_rc=0;
%let _fcfmsg_=;
 
    
    %let testYN=%upcase(&testYN);

    %if &testYN ne Y %then %let testYN=N;

    %let printYN=%upcase(&printYN);

    %if &printYN ne Y %then %let printYN=N;
    
    %let renameYN=%upcase(&renameYN);

    %if &renameYN ne Y %then %let renameYN=N;

    

    %let successYN=Y;

    %*let form_path=&rc_config_path&slash.Source&slash.form_template;

    %local start_dttm end_dttm;

    /* Save the job start time and round it to seconds */
    data _null_;
    start_dttm=int(datetime());
    call symput('start_dttm',put(start_dttm,best.));
    call symput('start_dttm_fmt',put(start_dttm,nldatm20.));
    run;

    %put start: sysmacroname,&start_dttm_fmt;

    /* Validate input parameters.                      */

    data _null_;
    length msg $ 4096;
   
    valid=1;

    if "&response_file_ext"="" then
    do;
       valid=0;
       msg='Parameter RESPONSE_FILE_EXT is missing.';
       put msg;
    end;

    if "&FORM_CONFIG_KEY"="" then
    do;
       valid=0;
       msg='Parameter FORM_CONFIG_KEY is missing.';
       put msg;
    end;

    if "&response_type_code"="" then
    do;
       valid=0;
       msg='Paramter RESPONSE_TYPE_CODE is missing.';
       put msg;
    end;
   
 
    if valid=0 then call symput('TRANS_RC','99');   
    run;
     
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %goto MACRO_END;

     
    %let response_pgm_prefix=rc_resp;
    %let form_type_code=;
    %let form_exist=0; 
     proc sql noprint;
      select 1
             , efile_file_prefix
             , "&response_pgm_prefix"||'_'||strip(form_agency_code)||'_'||strip(form_type_code) 
             , form_type_code
             into
               :form_exist 
             , :efile_file_prefix
             , :response_pgm_name
             , :form_type_code
         from seg_kc.fsk_rc_form_config
         where form_config_key=&form_config_key;
        quit;

     %if &form_exist=0 %then
     %do;
          %fcf_rcset(99);                        
          %let _fcfmsg_=ERROR: RC: Invalid form key &form_config_key;
          %fcf_put_error;
          %goto MACRO_END;         
     %end; 

    %let form_type_code=%trim(&form_type_code);

    %if &sdtmYN=Y and %upcase(&&&rc_sdtm_YN_&form_type_code)=Y %then  
         %let respfref_path=%trim(&&&rc_sdtm_resp_path_&form_type_code);
    %else %let respfref_path=%trim(&rc_efile_resp_path.&slash.%trim(&efile_file_prefix));
    
    filename dirlist pipe "&dirCmd &respfref_path.&slash.*.&response_file_ext";
     
    %let file_cnt=0; 
    data dirlist (drop=form_config_key efile_key coverage_begin_date coverage_end_date output_report_count) 
         response_meta_all ;
         if 0 then set RC_DB.RESP_FINCEN_SUMMARY 
                                  (
                                   %fcf_get_ds_opts(lib_name=RC_DB, ds_name=RESP_FINCEN_SUMMARY)
                                   keep=response_id response_file_name response_file_ext response_type_code
                                        coverage_start_date_str coverage_end_date_str);
         retain file_cnt 0;
         length line $150;
         drop file_cnt line;
         infile dirlist length=reclen ;
         
         input response_file_name $varying256. reclen ;
         if response_file_name > '';
         
         /* if the path is included in the file name, remove the path */
         if indexc(response_file_name,"&slash") then
             response_file_name=reverse(scan(reverse(response_file_name),1,"&slash")); 
         /* finding the last dot to extract the extension */
         response_file_ext=reverse(scan(reverse(response_file_name),1,"."));
         response_id=substr(response_file_name,1,index(response_file_name,response_file_ext)-2);
         
         /* for some reason, windows may return files with other extensions */
         /* Verify the extension is correct before using it.                */
         if response_file_ext="&response_file_ext";
         response_type_code=upcase("&response_type_code");
        
         if response_type_code ne 'XML' then
         do;
             link GET_EFILE_KEY;
             
             if resp_efile_key > 0 then 
             do;  
                  file_cnt+1;
                  output dirlist;
             end;
         end;
         else 
         do;
             /*
                Skip efile key verification and create response_meta_all directly      
                because XML respones file does not have any e-file key            
                Efile key will be extracted after rc_resp_fincen_*_read is executed.             
             */
             file_cnt+1;
             form_config_key=&form_config_key;
             coverage_begin_date=.;
             coverage_end_date=.;
             efile_key=.;
             output_report_count=.;
             output response_meta_all ;
         end;
         
         call symput('FILE_CNT',put(file_cnt,best.));
         return;
         
    GET_EFILE_KEY:
        /*
           For non xml response file only
           Get the e-file key from the last 10 charaters of 1A record 
           Also get the coverage period from 1A 
       */
        drop line fid rc ;
        rc=filename('respfref',"%trim(&respfref_path)&slash"||strip(response_file_name));
        fid=fopen('respfref','I',150,'D');
        resp_efile_key=0;
        if fid > 0 then
        do; 
           rc=fread(fid);
           rc=fget(fid, line, 150);
           if substr(line,1,2)='1A' then
           do;
               coverage_start_date_str=substr(line,3,8);
               coverage_end_date_str=substr(line,11,8);
               %if &form_config_key =303 %then 
               %do;
                   resp_efile_key=input(substr(line,60,10),best.);
               %end;
               %else
               %do;
                   resp_efile_key=input(substr(line,120,10),best.);
               %end;
           end;
           rc=fclose(fid);
        end;
        rc=filename('respfref');
    return;

       
    run;

     %fcf_rcset(&syserr);
                        

     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmsg_=ERROR: RC: Unable to read response files;
         %fcf_put_error;
         %goto MACRO_END;
     %end;
    

     %if &file_cnt=0 %then %do;
        %let _fcfmsg_=NOTE: RC: No %trim(&form_type_code) Response Found..;
        %fcf_put_error;
        %if %upcase(&printYN)=Y %then
        %do;
        
            options ls=72;
            data _null_;
                file print notitles;
                put "No %trim(&form_type_code) Response Found.";
            run;

        %end;

        %goto MACRO_END;
    %end; 

    %if "%upcase(&response_type_code)" ne "XML" %then
    %do; 

    /* Join the efile table to get efile key and information.                                                  */
    /* The information can be used make sure that transmittal information in the response file and the efile   */
    /* match.                                                                                                  */  
   
    
    proc sql noprint; 
         create table response_meta_all as 
         select distinct
                  ack.*
                , ef.* 
                , &form_config_key as form_config_key 
         from dirlist as ack, 
              seg_kc.fsk_rc_efile (rename=(form_config_key=efile_form_config_key) drop=response_file_name
                   %if %upcase(&dbflavor)=TERADATA %then
                   %do;
                        sasdatefmt=(coverage_begin_date=datetime20. coverage_end_date=datetime20.)
                   %end;
                   %else
                   %do;
                        sasdatefmt=(coverage_begin_date=&rc_dtfmt.. coverage_end_date=&rc_dtfmt..)
                   %end;
                                      )  as ef
         where ack.resp_efile_key=ef.efile_key
               and ef.efile_form_config_key=&form_config_key
    ;
    quit;
    %if %upcase(&dbflavor)=TERADATA %then
    %do;
       data response_meta_all;
            set  response_meta_all ;
            coverage_begin_date=datepart(coverage_begin_date);
            coverage_end_date=datepart(coverage_end_date);
            format coverage_begin_date coverage_end_date &rc_dtfmt..;
       run;
             
       %end;

     %fcf_rcset(&sqlrc);
                        

     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmsg_=ERROR: RC: Unable to join response data with efile data;
         %fcf_put_error;
         %goto MACRO_END;
     %end;

    %end;   
     
    %let file_nobs=%rc_nobs(response_meta_all );

    %if &file_nobs ne &file_cnt %then
    %do;
         
         %let _fcfmsg_=ERROR: RC: Some response files cannot be matched with any e-files.;
         %fcf_put_error;
         %goto MACRO_END;
    %end;

    /* Process each response file */
     %do ifile=1 %to &file_nobs;          
       
        data response_meta_one;
             set response_meta_all (firstobs=&ifile);
             call symput('response_file_name',strip(response_file_name));
             output;
             stop;
        run;          
       
        filename respfref "%trim(&respfref_path)&slash.%trim(&response_file_name)";
        %&response_pgm_name( 
                                 indsn_response_meta=response_meta_one
                               , response_type_code=&response_type_code
                               , infref=respfref
                               , testYN=&testYN

                               );
         %if &trans_rc eq 3 or &trans_rc ge 5 %then %goto MACRO_END;
         
      
                              
        %if &renameYN=Y %then
        %do;
            filename pfref "&respfref_path.&slash.&response_file_name.p";
            %rc_movefile(infref=respfref,outfref=pfref,rcvar=RC);
            filename pfref clear;
        %end;
        filename respfref clear;
        
    %end; /* end file loop */
     
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %goto MACRO_END;
         
    
    /* Report the processing results */
    
    %rc_resp_fincen_report(  input_type=from_dttm
                               , from_dttm=&start_dttm
                               , form_config_key=&form_config_key
                               , printYN=&printYN
                              );

     %if &trans_rc eq 3 or &trans_rc ge 5 %then %goto MACRO_END;
         
    
   
   
%MACRO_END:    

  
    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
      %fcf_execute_macro(fcf_error);
    %else %do;
      %fcf_execute_macro(fcf_success);
    %end;
   
     
 

 %if &rc_debug=N %then 
   %do;
    proc datasets lib=work nolist nowarn;
         delete RESPONSE_META_ALL
                RESPONSE_META_ONE
                _T_RESPONSE_ERROR_CNT
                _1A
                _2A _2B _2C 
                _3A _3B _3C _3D _3E _3F
                _4A _4B  _4C  _4D _4E _4F _4G _4H _4I
                _9Z
                _NEWCTR_ERROR_CODES
                _SAR_ERROR_CODES
                DIRLIST
    ;
    QUIT;
  %end;



  data _null_;
         end_dttm=ceil(datetime());
         duration=end_dttm - &start_dttm;
         call symput('END_DTTM', put(end_dttm,best.));
         call symput('DURATION', strip(put(duration,best.))||' seconds.');
         call symput('END_DTTM_FMT', put(end_dttm,nldatm20.));
    run;

  
        %put End Time: &end_dttm_fmt duration: &duration; 


   

%mend; 
