/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    rc_resp_fincen_updt_kc
| PRODUCT: SAS Regulatory Console
| SYSTEM:  Windows, Unix
| DESC:    to update seg_kc data and workflows after a response file         
|          is processed.               
|                The updates are:                                                  
|                    updating SEG_KC.FSK_RC_EFILE.EFILE_STATUS_CODE=A, AW or AE                                  
|                    updating SEG_KC.FSK_RC_EFILE_REPORT.REPORT_AGENCY_REF_ID=<BSA-ID> and
|                               FILE_STATUS_CODE= A, AW, or AE 
|                      of each report in the efile
|                    updating SEG_KC.FSK_RC_REPORT.REPORT_STATUS_CODE=A, AW or AE 
|                    updating SEG_KC_FSK_CASE.CASE_STATUS_CODE=R of each parent case      
|                    adding SEG_KC_FSK_COMMENT (containing the report BSA-ID) for each case        
+------------------------------------------------------------------------------+
| USAGE:   RESP_SUMMARY_DSN=sas table containing efile submission result.    
|          RESP_REPORT_DSN=sas table containing submission result at        
|                          report level. It should also contain the BSA-ID (DCN).                                     
|          TESTYN= Y to update seg_kc.                                       
|                  N to generate temp tables only.  
|           FORM_TYPE_CODE=form type code. 
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/


%macro rc_resp_fincen_updt_kc (   resp_summary_dsn=                                     
                                , resp_report_dsn=                                      
                                , testYN=N
                                , form_type_code=
                                     )
;
   
%let trans_rc=0;
%let _fcfmsg_=; 
     
     %let response_list=;
     proc sql noprint;
          select distinct quote(strip(response_id)) into :response_list separated by ' '
          from &resp_summary_dsn(%fcf_get_ds_opts(ds_name=&resp_summary_dsn))
          where response_processing_msg=''
          ; 
     %if %str(&response_list) = %str() %then
     %do;
          %fcf_rcset(999);                       

         %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
             %let _fcfmsg_=ERROR: RC: Errors found in response files, no update to knowledge center.;
             %fcf_put_error;
             %goto MACRO_END;
         %end;
        
     %end; 

     /*******************************/
     /* Update Efile Status         */
     /*******************************/
    
        

   
    data efile_resp_info;
         set &resp_summary_dsn(%fcf_get_ds_opts(ds_name=&resp_summary_dsn));
             where response_id in (&response_list);

         length EFILE_STATUS_CODE $32;  
         if efile_key > .;
         
         EFILE_STATUS_CODE='A';
         if secondary_error_count > 0 then efile_Status_code='AW';
         if primary_error_count > 0 then efile_Status_code='AE'; 
          
         keep efile_key 
              efile_status_code 
              response_file_name
              processed_date;
         ;

    run;


    %let efile_cnt=%rc_nobs(efile_resp_info);
    %if &efile_cnt > 0 and &testYN = N %then
    %do;  

     proc sql;
     update seg_kc.fsk_rc_efile as ef
     set efile_status_code=(select  efile_status_code     
                            from efile_resp_info resp
                            where ef.efile_key=resp.efile_key),
          response_file_name=(select  response_file_name     
                            from efile_resp_info resp
                            where ef.efile_key=resp.efile_key),
          response_process_date=(select  processed_date     
                            from efile_resp_info resp
                            where ef.efile_key=resp.efile_key),
          lstupdate_date=datetime(),
          lstupdate_user_id="&sysuserid",
          version_number=version_number+1
     where efile_key in (select distinct efile_key from efile_resp_info)  ;
     quit;

     %fcf_rcset(&sqlrc);                      
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmsg_=ERROR: RC: Unable to update SEG_KC.FSK_RC_EFILE Response status.;
         %fcf_put_error;
         %goto MACRO_END;
     %end;
	 
	 /* One more pass, but only setting transmission_date if a missing value. */
	 proc sql;
     update seg_kc.fsk_rc_efile as ef
        set transmission_date=(select processed_date from efile_resp_info resp
           where ef.efile_key=resp.efile_key)
        where efile_key in (select distinct efile_key from efile_resp_info) 
           and ef.transmission_date = .;
     quit;
	 
	 %fcf_rcset(&sqlrc);                      
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmsg_=ERROR: RC: Unable to update SEG_KC.FSK_RC_EFILE TRANSMISSION_DATE.;
         %fcf_put_error;
         %goto MACRO_END;
     %end;
   
     %end;

     /******************************************/
     /* Update Report DCN (REPORT_AGENCY_REF_ID    */
     /******************************************/


    proc sql;
         create table _t_report_resp_full as 
         select resp.*                 
                , rr.parent_object_name
                , rr.parent_object_key
         from &resp_report_dsn(%fcf_get_ds_opts(ds_name=&resp_report_dsn)) as resp
         left join seg_kc.fsk_rc_report as rr
         on resp.report_key=rr.report_key 
         where resp.report_key > 0 and resp.response_id in (&response_list)
         ; 
    quit;
     
    %fcf_rcset(&sqlrc);                        

     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmsg_=ERROR: RC: Unable to _T_REPORT_RESP_FULL.;
         %fcf_put_error;
         %goto MACRO_END;
     %end; 


    data report_resp_info (keep=efile_key report_key report_agency_ref_id file_status_code lstupdate_date lstupdate_user_id)
         case_info    (keep=case_id case_status_code report_key)
         comment   (keep=object_id object_type_cd comment_category_cd comment_text)
         ;
         set _t_report_resp_full;
          
         length comment_text $ 1024;
         
         length file_status_code $ 32;
         file_status_code='A';

        if secondary_error_count > 0 then file_status_code='AW';
        if primary_error_count > 0 then file_status_code='AE';
       
         REPORT_AGENCY_REF_ID=bsa_id;
         lstupdate_user_id="&sysuserid";
         lstupdate_date=datetime(); 
         output report_resp_info;
         
         if upcase(parent_object_name)='CASE' then 
         do;
            
            case_id=parent_object_key;  
            case_status_code='R';
            output case_info;            

         end;
         
           
 
          if upcase(parent_object_name) = 'CASE' then 
          do;
              *  subject=sasmsg("&smdds","SAR_RESP_PARENT_COMMENT_SUBJ","NOQUOTE",strip(put(report_key,best.))) ;
              *  comment=sasmsg("&smdds","SAR_RESP_PARENT_COMMENT_BODY","NOQUOTE",strip(put(report_key,best.),strip(bsa_id)) ;
          
            object_type_cd='CAS';
            object_id=strip(put(parent_object_key,best.));
            comment_category_cd='GEN';
            subject='Report '||strip(put(report_key,best.))||' Update' ;
            comment_text='Report '||strip(put(report_key,best.))||' BSA-ID is '|| strip(bsa_id);
            output comment;
         end;

    run;

     %fcf_rcset(&syserr);                        

     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmsg_=ERROR: RC: Unable to create multiple tables.;
         %fcf_put_error;
         %goto MACRO_END;
     %end; 
     
     
    %if %rc_nobs(report_resp_info) > 0 and &testYN = N %then
    %do;
    
    
        proc sql;
            %fcf_begin_pass_through(dbflavor=&dbflavor, authenticationDomain=&segkcauthdomain, connOpts=&segKCDBConnOpts);
            execute (delete from &segKCSchema..fsk_temp_table) by &dbflavor;
            %fcf_end_pass_through(dbflavor=&dbflavor, commit=yes);
        quit;

        %fcf_rcset(&sqlrc.);
        %if &trans_rc. eq 3 or &trans_rc. ge 5 %then %do;
            %let _fcfmsg_=ERROR: RC: cannot clear FSK_TEMP_TABLE. ; 
                                  );
            %fcf_put_error;
            %goto MACRO_END;
        %end;

         /*----------------------------------------------------------
          * Load report status data to temp table before master update 
          *---------------------------------------------------------*/


        proc append base=seg_kc.fsk_temp_table (dbforce=yes)
                   data=report_resp_info
                        (rename=(
                                 efile_key=temp_num_1_value
                                 report_key=temp_num_2_value
                                 report_agency_ref_id=temp_char_1_value
                                 file_status_code=temp_char_2_value
                                 lstupdate_user_id=temp_char_3_value
                                 lstupdate_date=temp_date_1_value
                          )
                         )
                         force nowarn
                         ;
        
        run;
        %fcf_rcset(&syserr);
        %if &trans_rc. eq 3 or &trans_rc. ge 5 %then %do;
             %let _fcfmsg_=ERROR: RC: Unable to add date to FSK_TEMP_TABLE.;

            %fcf_put_error;
            %goto MACRO_END;
        %end;


        proc sql noprint;
            connect to &dbflavor as &dbflavor (authdomain="&segkcAuthDomain."
                                               &segKCDBConnOpts);
            /*----------------------------------------------------------
            * Update FSK_RC_EFILE_REPORT
            *---------------------------------------------------------*/
            execute(
                MERGE INTO &segKCSchema..fsk_rc_efile_report master
                USING (
                    SELECT
                        temp_num_1_value AS efile_key,
                        temp_num_2_value AS report_key,
                        temp_char_1_value as report_agency_ref_id,
                        temp_char_2_value as file_status_code
                    FROM &segKCSchema..fsk_temp_table
                ) data
                ON
                (
                    master.efile_key=data.efile_key AND
                    master.report_key=data.report_key
                )
                WHEN MATCHED THEN UPDATE SET
                    file_status_code=data.file_status_code,
                    report_agency_ref_id=data.report_agency_ref_id
                   
                %if %upcase(&dbflavor) eq ODBC %then %do;
                    ;
                %end;                
                    
            ) by &dbflavor. ;
            %if %upcase(&dbflavor) eq TERADATA %then %do;
	    execute( 
	             commit                   
	    ) by &dbflavor. ;
            %end;

            /*----------------------------------------------------------
            * Update FSK_RC_REPORT
            *---------------------------------------------------------*/
            execute(
                MERGE INTO &segKCSchema..fsk_rc_report master
                USING (
                    SELECT
                        temp_num_2_value AS report_key,
                        temp_char_2_value as file_status_code,
                        temp_char_3_value as lstupdate_user_id,
                        temp_date_1_value as lstupdate_date
                    FROM &segKCSchema..fsk_temp_table
                ) data
                ON
                (             
                    master.report_key=data.report_key
                )
                WHEN MATCHED THEN UPDATE SET
                    report_status_code=data.file_status_code,
					report_ready_code='A',    
                    lstupdate_user_id=data.lstupdate_user_id,
                    lstupdate_date=data.lstupdate_date,
                    version_number=version_number+1
                    
                   
                %if %upcase(&dbflavor) eq ODBC %then %do;
                    ;
                %end;                
                   
            ) by &dbflavor. ;
            %if %upcase(&dbflavor) eq TERADATA %then %do;
            execute( 
                  commit                   
	    ) by &dbflavor. ;
            %end;
            disconnect from &dbflavor;
        quit;

        %fcf_rcset(&sqlrc);
        %if &trans_rc. eq 3 or &trans_rc. ge 5 %then %do;

             %let _fcfmsg_=ERROR: RC: Unable to update SEG_KC.FSK_RC_REPORT and FSK_RC_EFILE_REPORT.;
             %fcf_put_error;
            %goto MACRO_END;
        %end;

        %let obs_count = %fcf_get_observation_count(table=report_resp_info,SASDATASET=Y);
        %let numevents = &obs_count;

        %fcf_get_sequence_ids(numIDs=&numevents);     
     
        proc sql;
            create table fsk_rc_report_event as
                select
                    symgetn('seqId'||strip(put(monotonic(),best.))) as EVENT_ID,
                    report_key,
                    "FSR" as EVENT_TYPE_CODE,
                    strip(put(efile_key,best.)) as EVENT_DESCRIPTION,
                    datetime() as CREATE_DATE,
                    "&sysuserid" as CREATE_USER_ID
                from report_resp_info
            ;
        quit;  
        

        proc append base = &db_kc_libname..FSK_RC_REPORT_EVENT 
            data = fsk_rc_report_event force;
        run;   


      
             
    %end;


    

    /**********************************************/
    /* close case with status code = R (reported) */
    /**********************************************/ 

    %let case_cnt=%rc_nobs(case_info);

    %if &case_cnt > 0 and %upcase(&&&rc_resp_case_updt_YN_&form_type_code)=Y and &testYN = N %then
    %do;
    %let case_list=;
    
    proc sql noprint;
     select distinct strip(put(case_id,best.)) into :case_list separated by ' '
     from case_info;
     
     update seg_kc.fsk_case
     set case_status_code="R",
	     case_close_date=datetime(),
         lstupdate_date=datetime(),
         lstupdate_user_id="&sysuserid",
         version_number=version_number+1
    /* where case_id in (select distinct case_id from case_info) */  
       where case_id in (&case_list) 
    ;
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc. eq 3 or &trans_rc. ge 5 %then %do;

             %let _fcfmsg_=ERROR: RC: Unable to update SEG_KC.FSK_CASE.;
             %fcf_put_error;
            %goto MACRO_END;
     %end;

     
     proc sql  ;
     
             delete from seg_kc.fsk_case_transaction             
             where case_id in (&case_list)
             ;
     quit;  
      %fcf_rcset(&sqlrc);
      %if &trans_rc. eq 3 or &trans_rc. ge 5 %then %do;
     
          %let _fcfmsg_=WARNING: RC: Unable to delete case data from SEG_KC.FSK_CASE_TRANSACTION.;
          %fcf_put_error;
                 
     %end;
     proc sql  ;
         
          delete from seg_kc.fsk_case_entity_transaction             
          where case_id in (&case_list)
          ;
     quit;  
     %fcf_rcset(&sqlrc);
     %if &trans_rc. eq 3 or &trans_rc. ge 5 %then %do;
         
         %let _fcfmsg_=WARNING: RC: Unable to delete case data from SEG_KC.FSK_CASE_ENTITY_TRANSACTION.;
         %fcf_put_error;
                      
     %end;


    %let obs_count = %fcf_get_observation_count(table=case_info,SASDATASET=Y);
    %let numevents = &obs_count;

    %fcf_get_sequence_ids(numIDs=&numevents);     
 
    proc sql;
        create table fsk_case_event as
            select
                symgetn('seqId'||strip(put(monotonic(),best.))) as EVENT_ID,
                case_id,
                "CLS" as EVENT_TYPE_CODE,
                strip(put(report_key,best.)) as EVENT_DESCRIPTION,
                datetime() as CREATE_DATE,
                "&sysuserid" as CREATE_USER_ID
            from case_info
        ;
    quit;  
    

    proc append base = &db_kc_libname..FSK_CASE_EVENT 
        data = fsk_case_event force;
    run;   

    %end;
 

    /**************************************************/
    /* Add comment to parent (case or incident).      */
    /**************************************************/         

    %let comment_cnt=%rc_nobs(comment);
    %if &comment_cnt > 0  and %upcase(&rc_resp_parent_comment_updt_YN)=Y and &testYN = N %then
    %do; 

        %let seqId=;
        %fcf_get_sequence_ids(numIDs=&comment_cnt);

            
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
             %let _fcfmsg_=ERROR: RC: Cannot get sequence numbers for COMMENT_ID. ;
             %fcf_put_error;
            %goto MACRO_END;

        %end;

        
        proc sql;
            create table fsk_comment as
                select *,
                    symgetn('seqId'||strip(put(monotonic(),best.))) as comment_id,                             
                    datetime() as CREATE_DATE,
                    "&sysuserid" as CREATE_USER_ID,
                    datetime() as LSTUPDATE_DATE,
                    "&sysuserid" as LSTUPDATE_USER_ID,
                    "N" as LOGICAL_DELETE_IND
                from comment
            ;
        quit;

       proc append base = seg_kc.FSK_COMMENT (dbforce=yes)
            data = fsk_comment force nowarn;
       run;


       %fcf_rcset(&syserr);
        %if &trans_rc. eq 3 or &trans_rc. ge 5 %then %do;

             %let _fcfmsg_=ERROR: RC: Unable add coments to SEG_KC.FSK_COMMENT.;
             %fcf_put_error;
            %goto MACRO_END;
        %end;


    %fcf_get_sequence_ids(numIDs=&comment_cnt);
     
 
    proc sql;
        create table fsk_case_event as
            select
                symgetn('seqId'||strip(put(monotonic(),best.))) as EVENT_ID,
                input(object_id,best.) as case_id,
                "CMT" as EVENT_TYPE_CODE,
                strip(put(comment_id,best.)) as EVENT_DESCRIPTION,
                datetime() as CREATE_DATE,
                "&sysuserid" as CREATE_USER_ID
            from fsk_comment
        ;
    quit;  
    

    proc append base = &db_kc_libname..FSK_CASE_EVENT 
        data = fsk_case_event force;
    run;   




    %end;

   
     
%MACRO_END:    

   
    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
      %fcf_execute_macro(fcf_error);
    %else %do;
      %fcf_execute_macro(fcf_success);
    %end;
  

    
          
 %if &rc_debug=N %then 
   %do;
    proc datasets lib=work nolist nowarn;
         delete RESPONSE_META_ONE
                 
    ;
    QUIT;
  %end;

     

%mend; 
