/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    rc_validate_institution_data.sas
| PRODUCT: SAS Regulatory Console
| SYSTEM:  Windows, Unix
| DESC:    validate instittution/branch data before loading into configuration tables. 
|          
+------------------------------------------------------------------------------+
| USAGE: 
|          new_records      = work dataset of records that will be validated.
|          record_type      = type of data to validate (transmitter, filer). 
|          key_variable     = unique key in new_records dataset that can be used to identify record with error.
|          form_type        = type of form (CTRX, SARX) that will be used to pull specific error messages.
|          form_config_key  = report type code to subset data for validation
|          form_type_code   = future use if need this level of detail.
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2018 - 2021 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro rc_validate_institution_data(new_records=,record_type=,key_variable=,form_type=,form_config_key=,form_type_code=);

    %let trans_rc=0;
    %let _fcfmsg_=;

    %let form_type=%trim(&form_type.);
    %let form_type_code=%trim(&form_type_code.);
    %let record_type=%trim(&record_type.);

    /* check for existence of form specif macro, if not use the generic one                        */

    %if %sysfunc(fileexist(&rcdata_path.&slash.rc_template&slash.rc_fincen_inst_data_validation_%lowcase(&form_type.)_macro.sas)) %then
        %let validation_macro=rc_fincen_inst_data_validation_%lowcase(&form_type.)_macro.sas;
    %else  
        %let validation_macro=rc_fincen_inst_data_validation_macro.sas;

    %inc rc_tmplt(&validation_macro.);
   
    data work.&record_type._error;
        length error_record_type $ 32
             error_code          $ 4
             target_field_name   $ 32
             target_field_value  $ 4000
             &key_variable.         8
        ;
        %if &form_config_key. ne %then %do;
            %let where_cl=(where=(form_config_key=&form_config_key.));
        %end;
        %else 
            %let where_cl=;
            
        set &new_records. &where_cl.;

        keep &key_variable. entity_name error_record_type error_code target_field_name target_field_value;
            
        error_record_type="&record_type.";
        %validate_fincen_&record_type.; 

    run;           

    %fcf_rcset(&syserr);                       

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: RC: Unable to validate &new_records.;
        %fcf_put_error;
        %goto MACRO_END;
    %end;     
         
    %fcf_rcset(&syserr);   


    /**************************************/
    /* Obtain error description and types */
    /**************************************/

    %let error_cnt=0;
    proc sql noprint;
        create table work.&record_type._error_more as
        select a.*
            , b.error_desc
            , b.field_reference
        from work.&record_type._error as a
            left join rc_cfg.&form_type._fincen_error_code(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=&form_type_code._fincen_error_code)) as b
               on a.error_code=b.error_code
               /*   and a.error_record_type=b.error_record_type */
        order by &key_variable., error_code
        ;
        %let error_cnt=&sqlobs;
    quit;  

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: RC: Unable to create table work.&record_type._error_more;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    %if &error_cnt > 0 %then
        %do;     
        /* Produce error reports */  
         proc report data=work.&record_type._error_more nowindows center ;
 
             column error_record_type error_code target_field_name target_field_value &key_variable. entity_name error_desc;

             define error_record_type /width=10 flow "Record Type";            
             define error_code /width=5 "Error Code" ;
             define &key_variable. /width=10 "Record Key"  ;
             define entity_name /width=20 flow "Entity Name";
             define error_desc /width=30 flow "Error Description" ;  
             define target_field_name /width=32 flow "Error Field" ; 
             define target_field_value /width=32 flow "Error Field Value" ;   

             title "%upcase(%trim(&new_records)) Validation Results - Error Report";  
             title2 "Form &form_config_key (&form_type) %trim(&record_type) Data"; 
          run;
          title;
          %end;
    %else
        %do;
        options ls=120;
        data _null_;
            file print;
            put "No Validation errors found in %trim(&new_records.) for &form_config_key (&form_type) %trim(&record_type) Data";
        run;
        %end;     


    %MACRO_END:    

  
    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
      %fcf_execute_macro(fcf_error);
    %else %do;
      %fcf_execute_macro(fcf_success);
    %end;
   

%mend; /* rc_validate_sample_data */
 
