/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    rc_validate_xml_driver.sas
| PRODUCT: SAS Regulatory Console
| SYSTEM:  Windows, Unix
| DESC:    Create temporary copy of XML file that would be e-filed and validate the 
|          XML against a schema file(s). 
|          
+------------------------------------------------------------------------------+
| USAGE: 
|          efile_key       = counter for sequence number
|          report_key      = report number key
|          report_file_xml = report data that might of been created in previous step (ex. with SARX). 
|                            If file exists, can skip the creation of a new XML file.
|          form_type_code  = type of form, such as CTRX, SARX
|          form_config_key = form type key, such as 312 for CTRX, 311 for SARX
|          efile_path      = location to create temporary e-file. Will determine pathname for the libref.
|                            (most likely will be WORK)
|          error_file      = dataset to contain the XML validation error(s), if any
|          xsl_loc         = location for XSL file to generate XML
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/
%MACRO rc_validate_xml_driver(efile_key=
		              , report_key=
					  , report_file_xml=
					  , form_type_code=
					  , form_config_key=
					  , efile_path=WORK
					  , error_file=WORK.XML_ERRORS
					  );

    %let trans_rc=0;
    %let _fcfmsg_=;
	
	data seq_report;
       efile_key=&efile_key.;
       report_key=&report_key.;
    run;
	
    /* Allow a flag to ALWAYS show XML Source file, even if no errors found.                 */
	/* When set to N, XML Source displayed ONLY when finding an XML schema validation error. */
	/* Global setting for macro variable exists in rc_global_mvar.                           */
	%if not %symexist(rc_validate_xml_source_YN) %then %let rc_validate_xml_source_YN=Y;
     	
	/* if report_file is blank, will need to generate the XML Raw file to translate.            */
	/* Using bquote because pathname may have /'s that cause macro to think its a math compare. */
	%if (%bquote(&report_file_xml.) eq %str()) %then %do;
					  
	
       /* use skipGetNodeYN=Y if rc_report_node exists */
       %rc_efile_fincen_detail_x(report_list=&report_key
                               , form_config_key=&form_config_key
                               , form_type_code=&form_type_code
                               , efile_key=&efile_key
                               , skipGetNodeYN=Y
                               , seq_dsn=seq_report
                               );
    
	
       %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
           %goto MACRO_END;
       %end;
	
       %rc_efile_fincen_sum_x(form_config_key=&form_config_key
                            , form_type_code=&form_type_code
                            , in_dsn=rc_report_node_more_x
                            , efile_key=&efile_key
                            , seq_dsn=seq_report
                            );


       %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
          %goto MACRO_END;
          %end;
	
       %inc rc_tmplt(%upcase(&form_type_code)_macro_wrt_xml.sas);
  
       filename rawxml "%sysfunc(pathname(&efile_path.))&slash.%lowcase(&form_type_code)Raw.xml";

       %rc_efile_fincen_gen_rawxml(form_config_key=&form_config_key
                                 , form_type_code=&form_type_code
                                 , efile_key=&efile_key
                                 , in_dsn=rc_report_node_more_x
                                 , outfref=rawxml 
                                 );

       %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
           %goto MACRO_END;
           %end;    
 	   %end; /* case where report_file was not passed in */
	   
	   	   
    /* If report_file was passed in, it will be an XML File that is different than RAW file that will need */
	/* a different translation file, and then sequence numbers added when complete.                        */
	/* The different xslt will be handled by form_type_code and sequence number is generic.                */
	%else %do;
	   filename rawxml "&report_file_xml.";	
	   %end; /* case where report_file was passed. */
	   
	filename xslt "&rc_template_path&slash.rc_%lowcase(&form_type_code)_efile.xsl";

    filename outxml  "%sysfunc(pathname(&efile_path.))&slash%lowcase(&form_type_code)Temp.xml";
    filename efilenm "%sysfunc(pathname(&efile_path.))&slash%lowcase(&form_type_code)Efile.xml";
    
    proc xsl in=rawxml xsl=xslt out=outxml;
    run;

	%fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: Problem applying PROC XSL to raw XML file;
        %fcf_put_error;
        %goto MACRO_END;
        %end;
	
	/* Add one more test to look for the Temp xml file, if does not exist, make a note and stop validation */
	%if %sysfunc(fexist(OUTXML)) eq 0 %then %do;
	    %let trans_rc=3;
	    %let _fcfmsg_=ERROR: Problem creating %lowcase(&form_type_code)Temp.xml with PROC XSL;
        %fcf_put_error;
        %goto MACRO_END;
	    %end;
	
	
     /******************************************/
    /* add unique sequence numbers to the xml */
    /******************************************/
    %let line_cnt=0;
  
    %rc_efile_xml_seqnum(exclude_dsn=seq_report 
                       , exclude_var=report_key
                       , infname=outxml 
                       , outfname=efilenm
                       , lineCntVar=line_cnt
					   );
					  

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
    %end;	
	
	/* If the XML file has been generated, validate the XML and build a dataset of errors.   */
	/* Errors or not, also display the XML file.                                             */
	proc sql noprint;
       select trim(xpath) into :fullpathnm from sashelp.vextfl
           where fileref='EFILENM';
    quit;
    %let fullpathnm=&fullpathnm.;
    %rc_validate_xml(xml_file=&fullpathnm.
                      , xsd_file=&rc_template_path.&slash.schema&slash.FINCEN&slash.%lowcase(&form_type_code.)_schema.xsd
					  , error_file=&error_file.); 


    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;       
        %goto MACRO_END;
    %end;		

	/* Display the XML source file before the validation so user will have a copy to compare */
	/* with validation results.                                                              */
	/* Have to close ODS HTML which is turned on by default for STP in order for _WEBOUT to  */
	/* not be 'in use'.  Once datastep is done writing XML content, then reset back default. */
	/* Will allow this to be a conditional display based on a Flag.                          */
	%let check_errors=0;
	proc sql noprint;
	    select nlobs into :check_errors 
	       from SASHELP.VTABLE
		where libname="WORK" and memname='XML_ERRORS';
	quit;
	%if (&check_errors. > 0) or (&rc_validate_xml_source_YN.=Y) %then
        %do;
	    ods html close;
	    data _null_;
	       file _webout;
		   infile efilenm recfm=v lrecl=32000 end=done;
		   input @;
		   if _n_=1 then do;
		       /*put "<textarea rows='20' cols='120' style=border:none;>";*/
			   put "<h3>XML Source (click on row to expand view)</h3>";
			   put "<textarea rows='1' cols='120' onfocus='this.rows=20;' style='resize: none;' readonly>";
			   end;
           if _infile_ ^= '';
		       put _infile_;
		   if done then do;
	           put "</textarea><p>";
               end;
        run; 
	    ods html file=_webout;	
		%end;

%MACRO_END:    

  
    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
      %fcf_execute_macro(fcf_error);
    %else %do;
      %fcf_execute_macro(fcf_success);
    %end;

	
%mend; /* rc_validate_xml_driver */